@@ -5,13 +5,13 @@
  *
  *      Description:  Type definitions for benchmarking framework
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,7 +38,8 @@ typedef void (*FuncPrototype)();
 
 typedef enum {
     SINGLE = 0,
-    DOUBLE} DataType;
+    DOUBLE,
+    INT} DataType;
 
 typedef enum {
     STREAM_1 = 1,
@@ -89,6 +90,13 @@ typedef struct {
     FuncPrototype kernel;
     int  flops;
     int  bytes;
+    char* desc;
+    int loads;
+    int stores;
+    int branches;
+    int instr_const;
+    int instr_loop;
+    int uops;
 } TestCase;
 
 typedef struct {
