@@ -5,13 +5,13 @@
  *
  *      Description:  Implementation of allocator module.
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -33,6 +33,7 @@
 #include <stdio.h>
 #include <string.h>
 
+#include <allocator_types.h>
 #include <allocator.h>
 #include <likwid.h>
 
@@ -45,8 +46,8 @@
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
 static int numberOfAllocatedVectors = 0;
-static void** allocations;
-static AffinityDomains_t domains;
+static allocation* allocList;
+static AffinityDomains_t domains = NULL;
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
@@ -56,7 +57,7 @@ static AffinityDomains_t domains;
 void
 allocator_init(int numVectors)
 {
-    allocations = (void**) malloc(numVectors * sizeof(void*));
+    allocList = (allocation*) malloc(numVectors * sizeof(allocation));
     domains = get_affinityDomains();
 }
 
@@ -68,8 +69,12 @@ allocator_finalize()
 
     for (i=0; i<numberOfAllocatedVectors; i++)
     {
-        free(allocations[i]);
+        free(allocList[i].ptr);
+        allocList[i].ptr = NULL;
+        allocList[i].size = 0;
+        allocList[i].offset = 0;
     }
+    numberOfAllocatedVectors = 0;
 }
 
 void
@@ -83,20 +88,42 @@ allocator_allocateVector(
 {
     int i;
     size_t bytesize = 0;
-    const AffinityDomain* domain;
+    const AffinityDomain* domain = NULL;
     int errorCode;
+    int elements = 0;
 
     switch ( type )
     {
+        case INT:
+            bytesize = (size+offset) * sizeof(int);
+            elements = alignment / sizeof(int);
+            break;
+
         case SINGLE:
             bytesize = (size+offset) * sizeof(float);
+            elements = alignment / sizeof(float);
             break;
 
         case DOUBLE:
             bytesize = (size+offset) * sizeof(double);
+            elements = alignment / sizeof(double);
             break;
     }
 
+    for (i=0;i<domains->numberOfAffinityDomains;i++)
+    {
+        if (biseq(domainString, domains->domains[i].tag))
+        {
+            domain = domains->domains + i;
+        }
+    }
+    if (!domain)
+    {
+        fprintf(stderr, "Error: Cannot use desired domain %s for vector placement, Domain %s does not exist.\n",
+                        bdata(domainString), bdata(domainString));
+        exit(EXIT_FAILURE);
+    }
+
     errorCode =  posix_memalign(ptr, alignment, bytesize);
 
     if (errorCode)
@@ -104,42 +131,53 @@ allocator_allocateVector(
         if (errorCode == EINVAL)
         {
             fprintf(stderr,
-                    "Alignment parameter is not a power of two\n");
+                    "Error: Alignment parameter is not a power of two\n");
             exit(EXIT_FAILURE);
         }
         if (errorCode == ENOMEM)
         {
             fprintf(stderr,
-                    "Insufficient memory to fulfill the request\n");
+                    "Error: Insufficient memory to fulfill the request\n");
             exit(EXIT_FAILURE);
         }
     }
 
     if ((*ptr) == NULL)
     {
-            fprintf(stderr, "posix_memalign failed!\n");
-            exit(EXIT_FAILURE);
-
+        fprintf(stderr, "Error: posix_memalign failed!\n");
+        exit(EXIT_FAILURE);
     }
 
-    allocations[numberOfAllocatedVectors] = *ptr;
+    allocList[numberOfAllocatedVectors].ptr = *ptr;
+    allocList[numberOfAllocatedVectors].size = bytesize;
+    allocList[numberOfAllocatedVectors].offset = offset;
+    allocList[numberOfAllocatedVectors].type = type;
     numberOfAllocatedVectors++;
-    for (i=0;i<domains->numberOfAffinityDomains;i++)
-    {
-        if (biseq(domainString, domains->domains[i].tag))
-        {
-            domain = domains->domains + i;
-        }
-    }
-    affinity_pinProcess(domain->processorList[0]);
 
-    printf("Allocate: Process running on core %d - Vector length %llu Offset %d\n",
+    affinity_pinProcess(domain->processorList[0]);
+    printf("Allocate: Process running on core %d (Domain %s) - Vector length %llu Offset %d Alignment %llu\n",
             affinity_processGetProcessorId(),
-            LLU_CAST size,
-            offset);
+            bdata(domain->tag),
+            LLU_CAST bytesize,
+            offset,
+            LLU_CAST elements);
 
     switch ( type )
     {
+        case INT:
+            {
+                int* sptr = (int*) (*ptr);
+                sptr += offset;
+
+                for ( uint64_t i=0; i < size; i++ )
+                {
+                    sptr[i] = 1;
+                }
+                *ptr = (void*) sptr;
+
+            }
+            break;
+
         case SINGLE:
             {
                 float* sptr = (float*) (*ptr);
