@@ -5,13 +5,13 @@
  *
  *      Description:  Implementation of threaded spin loop barrier
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -156,4 +156,12 @@ barrier_synchronize(BarrierData* barr)
     barr->offset = !barr->offset;
 }
 
-
+void barrier_destroy(BarrierData* barr)
+{
+    if (currentGroupId > maxGroupId)
+    {
+        fprintf(stderr, "ERROR: Group ID %d larger than maxGroupID %d\n",currentGroupId,maxGroupId);
+    }
+    free(barr->index);
+    free(groups[currentGroupId].groupBval);
+}
