@@ -5,15 +5,15 @@
  *
  *      Description:  Implementation of access daemon.
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Authors:  Michael Meier, michael.meier@rrze.fau.de
  *                Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -47,75 +47,53 @@
 #include <sys/fsuid.h>
 #include <getopt.h>
 
-#include <pci_types.h>
+#include <types.h>
+#include <registers.h>
+#include <perfmon_haswellEP_counters.h>
+#include <perfmon_ivybridgeEP_counters.h>
+#include <perfmon_sandybridgeEP_counters.h>
+#include <perfmon_broadwelld_counters.h>
+#include <perfmon_broadwellEP_counters.h>
+#include <topology.h>
+#include <cpuid.h>
 #include <lock.h>
-#include <accessClient_types.h>
 
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 #define SA struct sockaddr
 #define str(x) #x
 
-#define CHECK_ERROR(func, msg)  \
-    if ((func) < 0) { syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); }
-
 #define CHECK_FILE_ERROR(func, msg)  \
     if ((func) == 0) { syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); }
 
 
-#define EXIT_IF_ERROR(func, msg)  \
-    if ((func) < 0) { syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); stop_daemon(); exit(EXIT_FAILURE); }
 
 
-#define CPUID                    \
-    __asm__ volatile ("cpuid"    \
-            : "=a" (eax),            \
-            "=b" (ebx)             \
-            : "0" (eax))
 
 
-#define  P6_FAMILY        0x6U
-#define  K8_FAMILY        0xFU
-#define  K10_FAMILY       0x10U
-#define  K15_FAMILY       0x15U
-#define  K16_FAMILY       0x16U
-
-#define SANDYBRIDGE          0x2AU
-#define SANDYBRIDGE_EP       0x2DU
-#define IVYBRIDGE            0x3AU
-#define IVYBRIDGE_EP         0x3EU
-#define HASWELL              0x3CU
-#define HASWELL_EP           0x3FU
-#define ATOM_SILVERMONT_E    0x37U
-#define ATOM_SILVERMONT_C    0x4DU
-#define ATOM_SILVERMONT_Z1   0x4AU
-#define ATOM_SILVERMONT_Z2   0x5AU
-#define ATOM_SILVERMONT_F    0x5DU
-#define BROADWELL            0x3DU
-#define BROADWELL_E          0x4FU
-#define BROADWELL_D          0x56U
-
 #define PCI_ROOT_PATH    "/proc/bus/pci/"
-#define MAX_PATH_LENGTH   60
-#define MAX_NUM_NODES    4
+#define MAX_PATH_LENGTH   80
+//#define MAX_NUM_NODES    4
 
 /* Lock file controlled from outside which prevents likwid to start.
  * Can be used to synchronize access to the hardware counters
  * with an external monitoring system. */
 
 /* #####   TYPE DEFINITIONS   ########### */
-typedef int (*FuncPrototype)(uint32_t);
+typedef int (*AllowedPrototype)(uint32_t);
+typedef int (*AllowedPciPrototype)(PciDeviceType, uint32_t);
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 static int sockfd = -1;
 static int connfd = -1; /* temporary in to make it compile */
 static char* filepath;
 static const char* ident = "accessD";
-static FuncPrototype allowed = NULL;
+static AllowedPrototype allowed = NULL;
+static AllowedPciPrototype allowedPci = NULL;
 static int FD_MSR[MAX_NUM_THREADS];
 static int FD_PCI[MAX_NUM_NODES][MAX_NUM_PCI_DEVICES];
 static int isPCIUncore = 0;
-static PciDevice* pci_devices = NULL;
+static PciDevice* pci_devices_daemon = NULL;
 static char pci_filepath[MAX_PATH_LENGTH];
 
 /* Socket to bus mapping -- will be determined at runtime;
@@ -129,76 +107,6 @@ static char pci_filepath[MAX_PATH_LENGTH];
 static char* socket_bus[MAX_NUM_NODES] = { [0 ... (MAX_NUM_NODES-1)] = NULL};
 
 
-static PciDevice sandybridgeEP_pci_devices[MAX_NUM_PCI_DEVICES] = {
- [MSR_DEV] = {NONE, NULL, NULL, NULL, 0x0, 0},
- [PCI_R3QPI_DEVICE_LINK_0] = {R3QPI, "13.5", "PCI_R3QPI_DEVICE_LINK_0", "RBOX0", 0x3c44, 0},
- [PCI_R3QPI_DEVICE_LINK_1] = {R3QPI, "13.6", "PCI_R3QPI_DEVICE_LINK_1", "RBOX1", 0x3c45, 0},
- [PCI_R2PCIE_DEVICE] = {R2PCIE, "13.1", "PCI_R2PCIE_DEVICE", "PBOX0", 0x3c43, 0},
- [PCI_IMC_DEVICE_0_CH_0] = {IMC, "10.0", "PCI_IMC_DEVICE_CH_0", "MBOX0", 0x3cb0, 0},
- [PCI_IMC_DEVICE_0_CH_1] = {IMC, "10.1", "PCI_IMC_DEVICE_CH_1", "MBOX1", 0x3cb1, 0},
- [PCI_IMC_DEVICE_0_CH_2] = {IMC, "10.4", "PCI_IMC_DEVICE_CH_2", "MBOX2", 0x3cb4, 0},
- [PCI_IMC_DEVICE_0_CH_3] = {IMC, "10.5", "PCI_IMC_DEVICE_CH_3", "MBOX3", 0x3cb5, 0},
- [PCI_HA_DEVICE_0] = {HA, "0e.1", "PCI_HA_DEVICE", "BBOX", 0x3c46, 0},
- [PCI_QPI_DEVICE_PORT_0] = {QPI, "08.2", "PCI_QPI_DEVICE_PORT_0", "SBOX0", 0x3c41, 0},
- [PCI_QPI_DEVICE_PORT_1] = {QPI, "09.2", "PCI_QPI_DEVICE_PORT_1", "SBOX1", 0x3c42, 0},
- [PCI_QPI_MASK_DEVICE_PORT_0] = {QPI, "08.6", "PCI_QPI_MASK_DEVICE_PORT_0", NULL, 0x3c86, 0},
- [PCI_QPI_MASK_DEVICE_PORT_1] = {QPI, "09.6", "PCI_QPI_MASK_DEVICE_PORT_1", NULL, 0x3c96, 0},
- [PCI_QPI_MISC_DEVICE_PORT_0] = {QPI, "08.0", "PCI_QPI_MISC_DEVICE_PORT_0", "SBOX0FIX",0x3c80, 0},
- [PCI_QPI_MISC_DEVICE_PORT_1] = {QPI, "09.0", "PCI_QPI_MISC_DEVICE_PORT_1", "SBOX1FIX", 0x3c91, 0},
-};
-
-
-static PciDevice ivybridgeEP_pci_devices[MAX_NUM_PCI_DEVICES] = {
- [MSR_DEV] = {NONE, "", "", "", 0x0, 0},
- [PCI_R3QPI_DEVICE_LINK_0] = {R3QPI, "13.5", "PCI_R3QPI_DEVICE_LINK_0", "RBOX0", 0x0e36, 0},
- [PCI_R3QPI_DEVICE_LINK_1] = {R3QPI, "13.6", "PCI_R3QPI_DEVICE_LINK_1", "RBOX1", 0x0e37, 0},
- [PCI_R3QPI_DEVICE_LINK_2] = {R3QPI, "12.5", "PCI_R3QPI_DEVICE_LINK_2", "RBOX2", 0x0e3e, 0},
- [PCI_R2PCIE_DEVICE] = {R2PCIE, "13.1", "PCI_R2PCIE_DEVICE", "PBOX0", 0x0e34, 0},
- [PCI_IMC_DEVICE_0_CH_0] = {IMC, "10.4", "PCI_IMC_DEVICE_0_CH_0", "MBOX0", 0x0eb4, 0},
- [PCI_IMC_DEVICE_0_CH_1] = {IMC, "10.5", "PCI_IMC_DEVICE_0_CH_1", "MBOX1", 0x0eb5, 0},
- [PCI_IMC_DEVICE_0_CH_2] = {IMC, "10.0", "PCI_IMC_DEVICE_0_CH_2", "MBOX2", 0x0eb0, 0},
- [PCI_IMC_DEVICE_0_CH_3] = {IMC, "10.1", "PCI_IMC_DEVICE_0_CH_3", "MBOX3", 0x0eb1, 0},
- [PCI_HA_DEVICE_0] = {HA, "0e.1", "PCI_HA_DEVICE_0", "BBOX0", 0x0e30, 0},
- [PCI_HA_DEVICE_1] = {HA, "1c.1", "PCI_HA_DEVICE_1", "BBOX1", 0x0e38, 0},
- [PCI_IMC_DEVICE_1_CH_0] = {IMC, "1e.4", "PCI_IMC_DEVICE_1_CH_0", "MBOX0", 0x0ef4, 0},
- [PCI_IMC_DEVICE_1_CH_1] = {IMC, "1e.5", "PCI_IMC_DEVICE_1_CH_1", "MBOX1", 0x0ef5, 0},
- [PCI_IMC_DEVICE_1_CH_2] = {IMC, "1e.0", "PCI_IMC_DEVICE_1_CH_2", "MBOX2", 0x0ef0, 0},
- [PCI_IMC_DEVICE_1_CH_3] = {IMC, "1e.1", "PCI_IMC_DEVICE_1_CH_3", "MBOX3", 0x0ef1, 0},
- [PCI_IRP_DEVICE] = {IRP, "05.6", "PCI_IRP_DEVICE", NULL, 0x0e39, 0},
- [PCI_QPI_DEVICE_PORT_0] = {QPI, "08.2", "PCI_QPI_DEVICE_PORT_0", "SBOX0", 0x0e32, 0},
- [PCI_QPI_DEVICE_PORT_1] = {QPI, "09.2", "PCI_QPI_DEVICE_PORT_1", "SBOX1", 0x0e33, 0},
- [PCI_QPI_DEVICE_PORT_2] = {QPI, "0a.2", "PCI_QPI_DEVICE_PORT_2", "SBOX2", 0x0e3a, 0},
- [PCI_QPI_MASK_DEVICE_PORT_0] = {QPI, "08.6", "PCI_QPI_MASK_DEVICE_PORT_0", NULL, 0x0e86, 0},
- [PCI_QPI_MASK_DEVICE_PORT_1] = {QPI, "09.6", "PCI_QPI_MASK_DEVICE_PORT_1", NULL, 0x0e96, 0},
- [PCI_QPI_MASK_DEVICE_PORT_2] = {QPI, "0a.6", "PCI_QPI_MASK_DEVICE_PORT_2", NULL, 0x0ec6, 0},
- [PCI_QPI_MISC_DEVICE_PORT_0] = {QPI, "08.0", "PCI_QPI_MISC_DEVICE_PORT_0/1", "SBOX01FIX",0x0e80, 0},
- [PCI_QPI_MISC_DEVICE_PORT_2] = {QPI, "0a.0", "PCI_QPI_MISC_DEVICE_PORT_2", "SBOX2FIX", 0x0ec0, 0},
-};
-
-static PciDevice haswellEP_pci_devices[MAX_NUM_PCI_DEVICES] = {
- [MSR_DEV] = {NONE, NULL, NULL, NULL, 0x0, 0},
- [PCI_R3QPI_DEVICE_LINK_0] = {R3QPI, "0b.1", "PCI_R3QPI_DEVICE_LINK_0", "RBOX0", 0x2f36, 0},
- [PCI_R3QPI_DEVICE_LINK_1] = {R3QPI, "0b.2", "PCI_R3QPI_DEVICE_LINK_1", "RBOX1", 0x2f37, 0},
- [PCI_R2PCIE_DEVICE] = {R2PCIE, "10.1", "PCI_R2PCIE_DEVICE", "PBOX0", 0x2f34, 0},
- [PCI_IMC_DEVICE_0_CH_0] = {IMC, "14.0", "PCI_IMC_DEVICE_0_CH_0", "MBOX0", 0x2fb4, 0},
- [PCI_IMC_DEVICE_0_CH_1] = {IMC, "14.1", "PCI_IMC_DEVICE_0_CH_1", "MBOX1", 0x2fb5, 0},
- [PCI_IMC_DEVICE_0_CH_2] = {IMC, "15.0", "PCI_IMC_DEVICE_0_CH_2", "MBOX2", 0x2fb0, 0},
- [PCI_IMC_DEVICE_0_CH_3] = {IMC, "15.1", "PCI_IMC_DEVICE_0_CH_3", "MBOX3", 0x2fb1, 0},
- [PCI_HA_DEVICE_0] = {HA, "12.1", "PCI_HA_DEVICE_0", "BBOX0", 0x2f30, 0},
- [PCI_HA_DEVICE_1] = {HA, "12.5", "PCI_HA_DEVICE_1", "BBOX1", 0x2f38, 0},
- [PCI_IMC_DEVICE_1_CH_0] = {IMC, "17.0", "PCI_IMC_DEVICE_1_CH_0", "MBOX4", 0x2fd4, 0},
- [PCI_IMC_DEVICE_1_CH_1] = {IMC, "17.1", "PCI_IMC_DEVICE_1_CH_1", "MBOX5", 0x2fd5, 0},
- [PCI_IMC_DEVICE_1_CH_2] = {IMC, "18.0", "PCI_IMC_DEVICE_1_CH_2", "MBOX6", 0x2fd0, 0},
- [PCI_IMC_DEVICE_1_CH_3] = {IMC, "18.1", "PCI_IMC_DEVICE_1_CH_3", "MBOX7", 0x2fd1, 0},
- [PCI_IRP_DEVICE] = {IRP, "05.6", "PCI_IRP_DEVICE", NULL, 0x2f39, 0},
- [PCI_QPI_DEVICE_PORT_0] = {QPI, "08.2", "PCI_QPI_DEVICE_PORT_0", "SBOX0", 0x2f32, 0},
- [PCI_QPI_DEVICE_PORT_1] = {QPI, "09.2", "PCI_QPI_DEVICE_PORT_1", "SBOX1", 0x2f33, 0},
- [PCI_QPI_MASK_DEVICE_PORT_0] = {QPI, "08.6", "PCI_QPI_MASK_DEVICE_PORT_0", NULL, 0x2f86, 0},
- [PCI_QPI_MASK_DEVICE_PORT_1] = {QPI, "09.6", "PCI_QPI_MASK_DEVICE_PORT_1", NULL, 0x2f96, 0},
- [PCI_QPI_MISC_DEVICE_PORT_0] = {QPI, "08.0", "PCI_QPI_MISC_DEVICE_PORT_0", "SBOX0FIX", 0x2f80, 0},
- [PCI_QPI_MISC_DEVICE_PORT_1] = {QPI, "09.0", "PCI_QPI_MISC_DEVICE_PORT_1", "SBOX1FIX", 0x2f80, 0},
-};
-
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
 static int allowed_intel(uint32_t reg)
@@ -209,7 +117,9 @@ static int allowed_intel(uint32_t reg)
             ((reg & 0xF00U) == 0xC00U) ||
             ((reg & 0xF00U) == 0xD00U) ||
             ((reg & 0xF00U) == 0xE00U) ||
+            ((reg & 0xF00U) == 0xF00U) ||
             (reg == 0x1A0)  ||
+            (reg == 0x1A4)  ||
             (reg == 0x0CE)  ||
             (reg == 0x19C)  ||
             (reg == 0x1A2)  ||
@@ -235,6 +145,144 @@ static int allowed_sandybridge(uint32_t reg)
     return 0;
 }
 
+static int allowed_pci_sandybridge(PciDeviceType type, uint32_t reg)
+{
+    switch (type)
+    {
+        case NODEVTYPE:
+            return 1;
+            break;
+        case R3QPI:
+            if ((reg == PCI_UNC_R3QPI_PMON_BOX_CTL) ||
+                (reg == PCI_UNC_R3QPI_PMON_BOX_STATUS) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTL_0) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTL_1) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTL_2) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTR_0_A) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTR_1_A) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTR_2_A) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTR_0_B) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTR_1_B) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTR_2_B))
+            {
+                return 1;
+            }
+            return 0;
+            break;
+        case R2PCIE:
+            if ((reg == PCI_UNC_R2PCIE_PMON_BOX_CTL) ||
+                (reg == PCI_UNC_R2PCIE_PMON_BOX_STATUS) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTL_0) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTL_1) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTL_2) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTL_3) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_0_A) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_1_A) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_2_A) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_3_A) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_0_B) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_1_B) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_2_B) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_3_B))
+            {
+                return 1;
+            }
+            return 0;
+            break;
+        case IMC:
+            if ((reg == PCI_UNC_MC_PMON_BOX_CTL) ||
+                (reg == PCI_UNC_MC_PMON_BOX_STATUS) ||
+                (reg == PCI_UNC_MC_PMON_CTL_0) ||
+                (reg == PCI_UNC_MC_PMON_CTL_1) ||
+                (reg == PCI_UNC_MC_PMON_CTL_2) ||
+                (reg == PCI_UNC_MC_PMON_CTL_3) ||
+                (reg == PCI_UNC_MC_PMON_CTR_0_A) ||
+                (reg == PCI_UNC_MC_PMON_CTR_1_A) ||
+                (reg == PCI_UNC_MC_PMON_CTR_2_A) ||
+                (reg == PCI_UNC_MC_PMON_CTR_3_A) ||
+                (reg == PCI_UNC_MC_PMON_CTR_0_B) ||
+                (reg == PCI_UNC_MC_PMON_CTR_1_B) ||
+                (reg == PCI_UNC_MC_PMON_CTR_2_B) ||
+                (reg == PCI_UNC_MC_PMON_CTR_3_B) ||
+                (reg == PCI_UNC_MC_PMON_FIXED_CTL) ||
+                (reg == PCI_UNC_MC_PMON_FIXED_CTR_A) ||
+                (reg == PCI_UNC_MC_PMON_FIXED_CTR_B))
+            {
+                return 1;
+            }
+            return 0;
+            break;
+        case HA:
+            if ((reg == PCI_UNC_HA_PMON_BOX_CTL) ||
+                (reg == PCI_UNC_HA_PMON_BOX_STATUS) ||
+                (reg == PCI_UNC_HA_PMON_CTL_0) ||
+                (reg == PCI_UNC_HA_PMON_CTL_1) ||
+                (reg == PCI_UNC_HA_PMON_CTL_2) ||
+                (reg == PCI_UNC_HA_PMON_CTL_3) ||
+                (reg == PCI_UNC_HA_PMON_CTR_0_A) ||
+                (reg == PCI_UNC_HA_PMON_CTR_1_A) ||
+                (reg == PCI_UNC_HA_PMON_CTR_2_A) ||
+                (reg == PCI_UNC_HA_PMON_CTR_3_A) ||
+                (reg == PCI_UNC_HA_PMON_CTR_0_B) ||
+                (reg == PCI_UNC_HA_PMON_CTR_1_B) ||
+                (reg == PCI_UNC_HA_PMON_CTR_2_B) ||
+                (reg == PCI_UNC_HA_PMON_CTR_3_B) ||
+                (reg == PCI_UNC_HA_PMON_OPCODEMATCH) ||
+                (reg == PCI_UNC_HA_PMON_ADDRMATCH0) ||
+                (reg == PCI_UNC_HA_PMON_ADDRMATCH1))
+            {
+                return 1;
+            }
+            return 0;
+            break;
+        case QPI:
+            if ((reg == PCI_UNC_QPI_PMON_BOX_CTL) ||
+                (reg == PCI_UNC_QPI_PMON_BOX_STATUS) ||
+                (reg == PCI_UNC_QPI_PMON_CTL_0) ||
+                (reg == PCI_UNC_QPI_PMON_CTL_1) ||
+                (reg == PCI_UNC_QPI_PMON_CTL_2) ||
+                (reg == PCI_UNC_QPI_PMON_CTL_3) ||
+                (reg == PCI_UNC_QPI_PMON_CTR_0_A) ||
+                (reg == PCI_UNC_QPI_PMON_CTR_1_A) ||
+                (reg == PCI_UNC_QPI_PMON_CTR_2_A) ||
+                (reg == PCI_UNC_QPI_PMON_CTR_3_A) ||
+                (reg == PCI_UNC_QPI_PMON_CTR_0_B) ||
+                (reg == PCI_UNC_QPI_PMON_CTR_1_B) ||
+                (reg == PCI_UNC_QPI_PMON_CTR_2_B) ||
+                (reg == PCI_UNC_QPI_PMON_CTR_3_B) ||
+                (reg == PCI_UNC_QPI_PMON_MASK_0) ||
+                (reg == PCI_UNC_QPI_PMON_MASK_1) ||
+                (reg == PCI_UNC_QPI_PMON_MATCH_0) ||
+                (reg == PCI_UNC_QPI_PMON_MATCH_1) ||
+                (reg == PCI_UNC_QPI_RATE_STATUS))
+            {
+                return 1;
+            }
+            return 0;
+            break;
+        case IRP:
+            if ((reg == PCI_UNC_IRP_PMON_BOX_STATUS) ||
+                (reg == PCI_UNC_IRP_PMON_BOX_CTL) ||
+                (reg == PCI_UNC_IRP0_PMON_CTL_0) ||
+                (reg == PCI_UNC_IRP0_PMON_CTL_1) ||
+                (reg == PCI_UNC_IRP0_PMON_CTR_0) ||
+                (reg == PCI_UNC_IRP0_PMON_CTR_1) ||
+                (reg == PCI_UNC_IRP1_PMON_CTL_0) ||
+                (reg == PCI_UNC_IRP1_PMON_CTL_1) ||
+                (reg == PCI_UNC_IRP1_PMON_CTR_0) ||
+                (reg == PCI_UNC_IRP1_PMON_CTR_1))
+            {
+                return 1;
+            }
+            return 0;
+            break;
+        default:
+            return 0;
+            break;
+    }
+    return 0;
+}
+
 static int allowed_haswell(uint32_t reg)
 {
     if ((allowed_intel(reg)) ||
@@ -249,6 +297,133 @@ static int allowed_haswell(uint32_t reg)
     }
 }
 
+static int allowed_pci_haswell(PciDeviceType type, uint32_t reg)
+{
+    switch (type)
+    {
+        case NODEVTYPE:
+            return 1;
+            break;
+        case R3QPI:
+            if ((reg == PCI_UNC_R3QPI_PMON_BOX_CTL) ||
+                (reg == PCI_UNC_R3QPI_PMON_BOX_STATUS) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTL_0) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTL_1) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTL_2) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTR_0_A) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTR_1_A) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTR_2_A) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTR_0_B) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTR_1_B) ||
+                (reg == PCI_UNC_R3QPI_PMON_CTR_2_B))
+            {
+                return 1;
+            }
+            return 0;
+            break;
+        case R2PCIE:
+            if ((reg == PCI_UNC_R2PCIE_PMON_BOX_CTL) ||
+                (reg == PCI_UNC_R2PCIE_PMON_BOX_STATUS) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTL_0) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTL_1) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTL_2) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTL_3) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_0_A) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_1_A) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_2_A) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_3_A) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_0_B) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_1_B) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_2_B) ||
+                (reg == PCI_UNC_R2PCIE_PMON_CTR_3_B))
+            {
+                return 1;
+            }
+            return 0;
+            break;
+        case IMC:
+            if ((reg == PCI_UNC_MC_PMON_BOX_CTL) ||
+                (reg == PCI_UNC_MC_PMON_BOX_STATUS) ||
+                (reg == PCI_UNC_MC_PMON_CTL_0) ||
+                (reg == PCI_UNC_MC_PMON_CTL_1) ||
+                (reg == PCI_UNC_MC_PMON_CTL_2) ||
+                (reg == PCI_UNC_MC_PMON_CTL_3) ||
+                (reg == PCI_UNC_MC_PMON_CTR_0_A) ||
+                (reg == PCI_UNC_MC_PMON_CTR_1_A) ||
+                (reg == PCI_UNC_MC_PMON_CTR_2_A) ||
+                (reg == PCI_UNC_MC_PMON_CTR_3_A) ||
+                (reg == PCI_UNC_MC_PMON_CTR_0_B) ||
+                (reg == PCI_UNC_MC_PMON_CTR_1_B) ||
+                (reg == PCI_UNC_MC_PMON_CTR_2_B) ||
+                (reg == PCI_UNC_MC_PMON_CTR_3_B) ||
+                (reg == PCI_UNC_MC_PMON_FIXED_CTL) ||
+                (reg == PCI_UNC_MC_PMON_FIXED_CTR_A) ||
+                (reg == PCI_UNC_MC_PMON_FIXED_CTR_B))
+            {
+                return 1;
+            }
+            return 0;
+            break;
+        case HA:
+            if ((reg == PCI_UNC_HA_PMON_BOX_CTL) ||
+                (reg == PCI_UNC_HA_PMON_BOX_STATUS) ||
+                (reg == PCI_UNC_HA_PMON_CTL_0) ||
+                (reg == PCI_UNC_HA_PMON_CTL_1) ||
+                (reg == PCI_UNC_HA_PMON_CTL_2) ||
+                (reg == PCI_UNC_HA_PMON_CTL_3) ||
+                (reg == PCI_UNC_HA_PMON_CTR_0_A) ||
+                (reg == PCI_UNC_HA_PMON_CTR_1_A) ||
+                (reg == PCI_UNC_HA_PMON_CTR_2_A) ||
+                (reg == PCI_UNC_HA_PMON_CTR_3_A) ||
+                (reg == PCI_UNC_HA_PMON_CTR_0_B) ||
+                (reg == PCI_UNC_HA_PMON_CTR_1_B) ||
+                (reg == PCI_UNC_HA_PMON_CTR_2_B) ||
+                (reg == PCI_UNC_HA_PMON_CTR_3_B) ||
+                (reg == PCI_UNC_HA_PMON_OPCODEMATCH) ||
+                (reg == PCI_UNC_HA_PMON_ADDRMATCH0) ||
+                (reg == PCI_UNC_HA_PMON_ADDRMATCH1))
+            {
+                return 1;
+            }
+            return 0;
+            break;
+        case QPI:
+            if ((reg == PCI_UNC_V3_QPI_PMON_BOX_CTL) ||
+                (reg == PCI_UNC_V3_QPI_PMON_BOX_STATUS) ||
+                (reg == PCI_UNC_V3_QPI_PMON_CTL_0) ||
+                (reg == PCI_UNC_V3_QPI_PMON_CTL_1) ||
+                (reg == PCI_UNC_V3_QPI_PMON_CTL_2) ||
+                (reg == PCI_UNC_V3_QPI_PMON_CTL_3) ||
+                (reg == PCI_UNC_V3_QPI_PMON_CTR_0_A) ||
+                (reg == PCI_UNC_V3_QPI_PMON_CTR_1_A) ||
+                (reg == PCI_UNC_V3_QPI_PMON_CTR_2_A) ||
+                (reg == PCI_UNC_V3_QPI_PMON_CTR_3_A) ||
+                (reg == PCI_UNC_V3_QPI_PMON_CTR_0_B) ||
+                (reg == PCI_UNC_V3_QPI_PMON_CTR_1_B) ||
+                (reg == PCI_UNC_V3_QPI_PMON_CTR_2_B) ||
+                (reg == PCI_UNC_V3_QPI_PMON_CTR_3_B) ||
+                (reg == PCI_UNC_V3_QPI_PMON_RX_MASK_0) ||
+                (reg == PCI_UNC_V3_QPI_PMON_RX_MASK_1) ||
+                (reg == PCI_UNC_V3_QPI_PMON_RX_MATCH_0) ||
+                (reg == PCI_UNC_V3_QPI_PMON_RX_MATCH_1) ||
+                (reg == PCI_UNC_V3_QPI_PMON_TX_MASK_0) ||
+                (reg == PCI_UNC_V3_QPI_PMON_TX_MASK_1) ||
+                (reg == PCI_UNC_V3_QPI_PMON_TX_MATCH_0) ||
+                (reg == PCI_UNC_V3_QPI_PMON_TX_MATCH_1) ||
+                (reg == PCI_UNC_V3_QPI_RATE_STATUS) ||
+                (reg == PCI_UNC_V3_QPI_LINK_LLR) ||
+                (reg == PCI_UNC_V3_QPI_LINK_IDLE))
+            {
+                return 1;
+            }
+            return 0;
+            break;
+        default:
+            return 0;
+            break;
+    }
+    return 0;
+}
 
 static int allowed_silvermont(uint32_t reg)
 {
@@ -323,9 +498,14 @@ static void msr_read(AccessDataRecord * dRecord)
     dRecord->errorcode = ERR_NOERROR;
     dRecord->data = 0;
 
+    if (FD_MSR[cpu] <= 0)
+    {
+        dRecord->errorcode = ERR_NODEV;
+        return;
+    }
+
     if (!allowed(reg))
     {
-        syslog(LOG_ERR, "Attempt to read to restricted register 0x%x on core %u", reg, cpu);
         dRecord->errorcode = ERR_RESTREG;
         return;
     }
@@ -333,10 +513,10 @@ static void msr_read(AccessDataRecord * dRecord)
     if (pread(FD_MSR[cpu], &data, sizeof(data), reg) != sizeof(data))
     {
         syslog(LOG_ERR, "Failed to read data to register 0x%x on core %u", reg, cpu);
+        syslog(LOG_ERR, "%s", strerror(errno));
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
-
     dRecord->data = data;
 }
 
@@ -347,6 +527,12 @@ static void msr_write(AccessDataRecord * dRecord)
     uint64_t data = dRecord->data;
 
     dRecord->errorcode = ERR_NOERROR;
+    
+    if (FD_MSR[cpu] <= 0)
+    {
+        dRecord->errorcode = ERR_NODEV;
+        return;
+    }
 
     if (!allowed(reg))
     {
@@ -358,11 +544,25 @@ static void msr_write(AccessDataRecord * dRecord)
     if (pwrite(FD_MSR[cpu], &data, sizeof(data), reg) != sizeof(data))
     {
         syslog(LOG_ERR, "Failed to write data to register 0x%x on core %u", reg, cpu);
+        syslog(LOG_ERR, "%s", strerror(errno));
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
 }
 
+static void msr_check(AccessDataRecord * dRecord)
+{
+    uint32_t cpu = dRecord->cpu;
+    dRecord->errorcode = ERR_NOERROR;
+
+    if (FD_MSR[cpu] < 0)
+    {
+        dRecord->errorcode = ERR_NODEV;
+        return;
+    }
+    return;
+}
+
 static void pci_read(AccessDataRecord* dRecord)
 {
     uint32_t socketId = dRecord->cpu;
@@ -379,28 +579,34 @@ static void pci_read(AccessDataRecord* dRecord)
         return;
     }
 
+    if (allowedPci)
+    {
+        if (!allowedPci(pci_devices_daemon[device].type, reg))
+        {
+        dRecord->errorcode = ERR_RESTREG;
+        return;
+        }
+    }
     if ( !FD_PCI[socketId][device] )
     {
-        strncpy(pci_filepath, PCI_ROOT_PATH, 30);
-        strncat(pci_filepath, socket_bus[socketId], 10);
-        strncat(pci_filepath, pci_devices[device].path, 20);
+        snprintf(pci_filepath, MAX_PATH_LENGTH-1, "%s%s%s", PCI_ROOT_PATH, socket_bus[socketId], pci_devices_daemon[device].path);
         FD_PCI[socketId][device] = open( pci_filepath, O_RDWR);
 
         if ( FD_PCI[socketId][device] < 0)
         {
             syslog(LOG_ERR, "Failed to open device file %s for device %s (%s) on socket %u", pci_filepath,
-                    pci_types[pci_devices[device].type].name, pci_devices[device].name, socketId);
+                    pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
             dRecord->errorcode = ERR_OPENFAIL;
             return;
         }
         syslog(LOG_ERR, "Open device file %s for device %s (%s) on socket %u", pci_filepath,
-                    pci_types[pci_devices[device].type].name, pci_devices[device].name, socketId);
+                    pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
     }
 
     if (FD_PCI[socketId][device] > 0 && pread(FD_PCI[socketId][device], &data, sizeof(data), reg) != sizeof(data))
     {
         syslog(LOG_ERR, "Failed to read data from pci device file %s for device %s (%s) on socket %u",
-                pci_filepath,pci_types[pci_devices[device].type].name, pci_devices[device].name,socketId);
+                pci_filepath,pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name,socketId);
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
@@ -425,39 +631,58 @@ static void pci_write(AccessDataRecord* dRecord)
         return;
     }
 
+    if (allowedPci)
+    {
+        if (!allowedPci(pci_devices_daemon[device].type, reg))
+        {
+        dRecord->errorcode = ERR_RESTREG;
+        return;
+        }
+    }
+
     if ( !FD_PCI[socketId][device] )
     {
-        strncpy(pci_filepath, PCI_ROOT_PATH, 30);
-        strncat(pci_filepath, socket_bus[socketId], 10);
-        strncat(pci_filepath, pci_devices[device].path, 20);
+        snprintf(pci_filepath, MAX_PATH_LENGTH-1, "%s%s%s", PCI_ROOT_PATH, socket_bus[socketId], pci_devices_daemon[device].path);
 
         FD_PCI[socketId][device] = open( pci_filepath, O_RDWR);
 
         if ( FD_PCI[socketId][device] < 0)
         {
             syslog(LOG_ERR, "Failed to open device file %s for device %s (%s) on socket %u", pci_filepath,
-                        pci_types[pci_devices[device].type].name, pci_devices[device].name, socketId);
+                        pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
             dRecord->errorcode = ERR_OPENFAIL;
             return;
         }
         syslog(LOG_ERR, "Open device file %s for device %s (%s) on socket %u", pci_filepath,
-                    pci_types[pci_devices[device].type].name, pci_devices[device].name, socketId);
+                    pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
     }
 
     if (FD_PCI[socketId][device] > 0 && pwrite(FD_PCI[socketId][device], &data, sizeof data, reg) != sizeof data)
     {
         syslog(LOG_ERR, "Failed to write data to pci device file %s for device %s (%s) on socket %u",pci_filepath,
-                pci_types[pci_devices[device].type].name, pci_devices[device].name, socketId);
+                pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
 }
 
 
+static void pci_check(AccessDataRecord* dRecord)
+{
+    uint32_t socketId = dRecord->cpu;
+    uint32_t device = dRecord->device;
+    dRecord->errorcode = ERR_NOERROR;
+
+    if (FD_PCI[socketId][device] == -2)
+    {
+        dRecord->errorcode = ERR_NODEV;
+        return;
+    }
+    return;
+}
+
 static void kill_client(void)
 {
-    syslog(LOG_NOTICE, "daemon dropped client");
-
     if (connfd != -1)
     {
         CHECK_ERROR(close(connfd), socket close failed);
@@ -469,7 +694,6 @@ static void kill_client(void)
 static void stop_daemon(void)
 {
     kill_client();
-    syslog(LOG_NOTICE, "daemon exiting");
     for (int i=0;i<MAX_NUM_NODES;i++)
     {
         if (socket_bus[i] != NULL)
@@ -497,7 +721,7 @@ int getBusFromSocket(const uint32_t socket)
     int ret = 0;
     while(cur_socket <= socket)
     {
-        sprintf(pci_filepath, "%s%02x/05.0", PCI_ROOT_PATH, cur_bus);
+        snprintf(pci_filepath, MAX_PATH_LENGTH-1, "%s%02x/05.0", PCI_ROOT_PATH, cur_bus);
         fp = open(pci_filepath, O_RDONLY);
         if (fp < 0)
         {
@@ -601,10 +825,10 @@ int main(void)
     mode_t oldumask;
     uint32_t numHWThreads = sysconf(_SC_NPROCESSORS_CONF);
     uint32_t model;
-#ifdef REVERSE_HASWELL_PCI_SOCKETS
-    char** socket_bus_copy;
-#endif
-
+    for (int i=0;i<MAX_NUM_THREADS;i++)
+    {
+        FD_MSR[i] = -1;
+    }
 
     openlog(ident, 0, LOG_USER);
 
@@ -619,15 +843,17 @@ int main(void)
     {
         uint32_t  eax = 0x00;
         uint32_t  ebx = 0x00;
+        uint32_t  ecx = 0x00;
+        uint32_t  edx = 0x00;
         /*int isIntel = 1;
-        CPUID;
+        CPUID(eax, ebx, ecx, edx);
         if (ebx == 0x68747541U)
         {
             isIntel = 0;
         }*/
 
         eax = 0x01;
-        CPUID;
+        CPUID(eax, ebx, ecx, edx);
         uint32_t family = ((eax >> 8) & 0xFU) + ((eax >> 20) & 0xFFU);
         model  = (((eax >> 16) & 0xFU) << 4) + ((eax >> 4) & 0xFU);
 
@@ -643,25 +869,42 @@ int main(void)
                 else if ((model == SANDYBRIDGE_EP) || (model == IVYBRIDGE_EP))
                 {
                     allowed = allowed_sandybridge;
+                    allowedPci = allowed_pci_sandybridge;
                     isPCIUncore = 1;
                 }
                 else if ((model == HASWELL) ||
+                         (model == HASWELL_M1) ||
+                         (model == HASWELL_M2) ||
                          (model == BROADWELL) ||
-                         (model == BROADWELL_D) ||
-                         (model == BROADWELL_E))
+                         (model == SKYLAKE1) ||
+                         (model == SKYLAKE2))
                 {
                     allowed = allowed_haswell;
                 }
+                else if (model == BROADWELL_D)
+                {
+                    allowed = allowed_haswell;
+                    isPCIUncore = 1;
+                    allowedPci = allowed_pci_haswell;
+                }
                 else if (model == HASWELL_EP)
                 {
                     isPCIUncore = 1;
                     allowed = allowed_haswell;
+                    allowedPci = allowed_pci_haswell;
+                }
+                else if (model == BROADWELL_E)
+                {
+                    isPCIUncore = 1;
+                    allowed = allowed_haswell;
+                    allowedPci = allowed_pci_haswell;
                 }
                 else if ((model == ATOM_SILVERMONT_C) ||
                          (model == ATOM_SILVERMONT_E) ||
                          (model == ATOM_SILVERMONT_Z1) ||
                          (model == ATOM_SILVERMONT_Z2) ||
-                         (model == ATOM_SILVERMONT_F))
+                         (model == ATOM_SILVERMONT_F) ||
+                         (model == ATOM_SILVERMONT_AIR))
                 {
                     allowed = allowed_silvermont;
                 }
@@ -717,8 +960,6 @@ int main(void)
         sigaction(SIGTERM, &sia, NULL);
     }
 
-    syslog(LOG_NOTICE, "daemon started");
-
     /* setup an alarm to stop the daemon if there is no connect.*/
     alarm(15U);
 
@@ -738,7 +979,6 @@ int main(void)
 
     alarm(0);
     CHECK_ERROR(unlink(filepath), unlink of socket failed);
-    syslog(LOG_NOTICE, "daemon accepted client");
 
     /* Restore the old umask and fs ids. */
     (void) umask(oldumask);
@@ -774,27 +1014,39 @@ int main(void)
             if (model == SANDYBRIDGE_EP)
             {
                 //testDevice = 0x80863c44;
-                pci_devices = sandybridgeEP_pci_devices;
+                pci_devices_daemon = sandybridgeEP_pci_devices;
             }
             else if (model == IVYBRIDGE_EP)
             {
                 //testDevice = 0x80860e36;
-                pci_devices = ivybridgeEP_pci_devices;
+                pci_devices_daemon = ivybridgeEP_pci_devices;
             }
             else if (model == HASWELL_EP)
             {
                 //testDevice = 0x80862f30;
-                pci_devices = haswellEP_pci_devices;
+                pci_devices_daemon = haswellEP_pci_devices;
+            }
+            else if (model == BROADWELL_D)
+            {
+                //testDevice = 0x80862f30;
+                pci_devices_daemon = broadwelld_pci_devices;
+            }
+            else if (model == BROADWELL_E)
+            {
+                //testDevice = 0x80862f30;
+                pci_devices_daemon = broadwellEP_pci_devices;
             }
             else
             {
                 //testDevice = 0;
                 syslog(LOG_NOTICE, "PCI Uncore not supported on this system");
+                goto LOOP;
             }
 
             for (int j=0; j<MAX_NUM_NODES; j++)
             {
-                socket_bus[j] = "N-A";
+                socket_bus[j] = (char*)malloc(4);
+                sprintf(socket_bus[j], "N-A");
                 for (int i=0; i<MAX_NUM_PCI_DEVICES; i++)
                 {
                     FD_PCI[j][i] = -2;
@@ -807,7 +1059,6 @@ int main(void)
             sbus = getBusFromSocket(cntr);
             while (sbus != -1)
             {
-                socket_bus[cntr] = (char*)malloc(4);
                 sprintf(socket_bus[cntr], "%02x/", sbus);
                 cntr++;
                 sbus = getBusFromSocket(cntr);
@@ -825,19 +1076,19 @@ int main(void)
                 {
                     for (int i=1; i<MAX_NUM_PCI_DEVICES; i++)
                     {
-                        if (pci_devices[i].path)
+                        if (pci_devices_daemon[i].path)
                         {
-                            sprintf(pci_filepath, "%s%s%s", PCI_ROOT_PATH, socket_bus[j], pci_devices[i].path);
+                            sprintf(pci_filepath, "%s%s%s", PCI_ROOT_PATH, socket_bus[j], pci_devices_daemon[i].path);
                             fd = open(pci_filepath, O_RDWR);
                             if (fd > 0)
                             {
                                 FD_PCI[j][i] = 0;
-                                pci_devices[i].online = 1;
+                                pci_devices_daemon[i].online = 1;
                                 close(fd);
                             }
                             else if (j==0)
                             {
-                                syslog(LOG_NOTICE, "Device %s for socket %d not found at path %s, excluded it from device list: %s\n",pci_devices[i].name,j, pci_filepath, strerror(errno));
+                                syslog(LOG_NOTICE, "Device %s for socket %d not found at path %s, excluded it from device list: %s\n",pci_devices_daemon[i].name,j, pci_filepath, strerror(errno));
                             }
                         }
                     }
@@ -845,15 +1096,13 @@ int main(void)
             }
         }
     }
-
+LOOP:
     while (1)
     {
         ret = read(connfd, (void*) &dRecord, sizeof(AccessDataRecord));
 
         if (ret < 0)
         {
-            syslog(LOG_ERR, "ERROR - [%s:%d] read from client failed  - %s \n",
-                    __FILE__, __LINE__, strerror(errno));
             stop_daemon();
         }
         else if ((ret == 0) && (dRecord.type != DAEMON_EXIT))
@@ -870,7 +1119,7 @@ int main(void)
 
         if (dRecord.type == DAEMON_READ)
         {
-            if (dRecord.device == DAEMON_AD_MSR)
+            if (dRecord.device == MSR_DEV)
             {
                 msr_read(&dRecord);
             }
@@ -881,7 +1130,7 @@ int main(void)
         }
         else if (dRecord.type == DAEMON_WRITE)
         {
-            if (dRecord.device == DAEMON_AD_MSR)
+            if (dRecord.device == MSR_DEV)
             {
                 msr_write(&dRecord);
                 dRecord.data = 0x0ULL;
@@ -892,6 +1141,17 @@ int main(void)
                 dRecord.data = 0x0ULL;
             }
         }
+        else if (dRecord.type == DAEMON_CHECK)
+        {
+            if (dRecord.device == MSR_DEV)
+            {
+                msr_check(&dRecord);
+            }
+            else
+            {
+                pci_check(&dRecord);
+            }
+        }
         else if (dRecord.type == DAEMON_EXIT)
         {
             stop_daemon();
