@@ -5,14 +5,14 @@
  *
  *      Description:  Implementation of frequency daemon
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -33,6 +33,9 @@
 #include <stdio.h>
 #include <string.h>
 
+char setfiles[3][100] = {"scaling_min_freq", "scaling_max_freq", "scaling_setspeed"};
+char getfiles[3][100] = {"cpuinfo_min_freq", "cpuinfo_max_freq", "cpuinfo_cur_freq"};
+
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 static int get_numCPUs()
 {
@@ -55,8 +58,10 @@ static int get_numCPUs()
 /* #####  MAIN FUNCTION DEFINITION   ################## */
 int main (int argn, char** argv)
 {
+    int i = 0;
+    int tmp;
     int cpuid;
-    int freq;
+    int freq = 0;
     int numCPUs = 0;
     char* gov;
     char* gpath = malloc(100);
@@ -74,7 +79,7 @@ int main (int argn, char** argv)
     numCPUs = get_numCPUs();
     if (cpuid < 0 || cpuid > numCPUs)
     {
-        fprintf(stderr, "CPU %d not a valid CPU ID. Range from 0 to %d.\n",cpuid,numCPUs);
+        fprintf(stderr, "CPU %d not a valid CPU ID. Range from 0 to %d.\n", cpuid, numCPUs);
         free(gpath);
         free(fpath);
         exit(EXIT_FAILURE);
@@ -90,22 +95,49 @@ int main (int argn, char** argv)
 
     if (argn == 4)
     {
+        FILE* f;
         gov = argv[3];
 
-        if ((strncmp(gov,"ondemand",8)) &&
-            (strncmp(gov,"performance",11)) &&
-            (strncmp(gov,"conservative",12)) &&
-            (strncmp(gov,"powersave",9))) {
+        if ((strncmp(gov,"ondemand",8) != 0) &&
+            (strncmp(gov,"performance",11) != 0) &&
+            (strncmp(gov,"conservative",12) != 0) &&
+            (strncmp(gov,"powersave",9) != 0)) {
             fprintf(stderr, "Invalid governor %s!\n",gov);
             free(gpath);
             free(fpath);
             return (EXIT_FAILURE);
         }
-        snprintf(gpath, 60, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_governor", cpuid);
+        
+        for (i=0; i<2; i++)
+        {
+            snprintf(fpath, 99, "/sys/devices/system/cpu/cpu%d/cpufreq/%s", cpuid, getfiles[i]);
+            f = fopen(fpath, "r");
+            if (f == NULL) {
+                fprintf(stderr, "Unable to open path %s for writing\n", fpath);
+                free(gpath);
+                free(fpath);
+                return (EXIT_FAILURE);
+            }
+            tmp = fread(fpath, 100, sizeof(char), f);
+            freq = atoi(fpath);
+            fclose(f);
+            snprintf(fpath, 99, "/sys/devices/system/cpu/cpu%d/cpufreq/%s", cpuid, setfiles[i]);
+            f = fopen(fpath, "w");
+            if (f == NULL) {
+                fprintf(stderr, "Unable to open path %s for writing\n",fpath);
+                free(gpath);
+                free(fpath);
+                return (EXIT_FAILURE);
+            }
+            fprintf(f,"%d",freq);
+            fclose(f);
 
-        FILE* f = fopen(gpath, "w");
+        }
+        snprintf(gpath, 99, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_governor", cpuid);
+
+        f = fopen(gpath, "w");
         if (f == NULL) {
-            fprintf(stderr, "Unable to open path for writing\n");
+            fprintf(stderr, "Unable to open path %s for writing\n", gpath);
             free(gpath);
             free(fpath);
             return (EXIT_FAILURE);
@@ -117,28 +149,44 @@ int main (int argn, char** argv)
         return(EXIT_SUCCESS);
     }
 
-    snprintf(gpath, 60, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_governor", cpuid);
-    snprintf(fpath, 60, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_setspeed", cpuid);
+    snprintf(gpath, 99, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_governor", cpuid);
 
     FILE* f = fopen(gpath, "w");
     if (f == NULL) {
-        fprintf(stderr, "Unable to open path for writing\n");
+        fprintf(stderr, "Unable to open path %s for writing\n", gpath);
         free(gpath);
         free(fpath);
         return (EXIT_FAILURE);
     }
-    fprintf(f,"userspace");
+    if ((argn == 4) &&
+        ((strncmp(argv[3],"ondemand",8) == 0) ||
+        (strncmp(argv[3],"performance",11) == 0) ||
+        (strncmp(argv[3],"conservative",12) == 0) ||
+        (strncmp(argv[3],"powersave",9) == 0)))
+    {
+        fprintf(f, "%s", argv[3]);
+        tmp = 1;
+    }
+    else
+    {
+        fprintf(f, "%s", "userspace");
+        tmp = 3;
+    }
     fclose(f);
 
-    f = fopen(fpath, "w");
-    if (f == NULL) {
-        fprintf(stderr, "Unable to open path for writing\n");
-        free(gpath);
-        free(fpath);
-        return (EXIT_FAILURE);
+    for (i=0;i<tmp;i++)
+    {
+        snprintf(fpath, 99, "/sys/devices/system/cpu/cpu%d/cpufreq/%s", cpuid, setfiles[i]);
+        f = fopen(fpath, "w");
+        if (f == NULL) {
+            fprintf(stderr, "Unable to open path %s for writing\n",fpath);
+            free(gpath);
+            free(fpath);
+            return (EXIT_FAILURE);
+        }
+        fprintf(f,"%d",freq);
+        fclose(f);
     }
-    fprintf(f,"%d",freq);
-    fclose(f);
     free(gpath);
     free(fpath);
     return(EXIT_SUCCESS);
