@@ -1,4 +1,4 @@
-#!<PREFIX>/bin/likwid-lua
+#!<INSTALLED_BINPREFIX>/likwid-lua
 --[[
  * =======================================================================================
  *
@@ -7,13 +7,13 @@
  *      Description:  An application to read out performance counter registers
  *                    on x86 processors
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -30,7 +30,7 @@
  * =======================================================================================
 ]]
 
-package.path = '<PREFIX>/share/lua/?.lua;' .. package.path
+package.path = '<INSTALLED_PREFIX>/share/lua/?.lua;' .. package.path
 
 local likwid = require("likwid")
 
@@ -63,6 +63,7 @@ local function usage()
     print("-E <string>\t\t List available events and corresponding counters that match <string>")
     print("-i, --info\t\t Print CPU info")
     print("-T <time>\t\t Switch eventsets with given frequency")
+    print("-f, --force\t\t Force overwrite of registers if they are in use")
     print("Modes:")
     print("-S <time>\t\t Stethoscope mode with duration in s, ms or us, e.g 20ms")
     print("-t <time>\t\t Timeline mode with frequency in s, ms or us, e.g. 300ms")
@@ -70,6 +71,7 @@ local function usage()
     print("Output options:")
     print("-o, --output <file>\t Store output to file. (Optional: Apply text filter according to filename suffix)")
     print("-O\t\t\t Output easily parseable CSV instead of fancy tables")
+    print("--stats\t\t\t Always print statistics table")
     print("\n")
     examples()
 end
@@ -93,7 +95,7 @@ group_list = {}
 group_ids = {}
 activeGroup = 0
 print_group_help = false
-skip_mask = "0x0"
+skip_mask = nil
 counter_mask = {}
 access_flags = "e"
 if config["daemonMode"] < 0 then
@@ -114,11 +116,14 @@ duration = 2.E06
 switch_interval = 5
 output = ""
 use_csv = false
+print_stats = false
 execString = nil
 outfile = nil
+forceOverwrite = 0
 gotC = false
-markerFile = string.format("/tmp/likwid_%d.txt",likwid.getpid("pid"))
+markerFile = string.format("/tmp/likwid_%d.txt",likwid.getpid())
 print_stdout = print
+cpuClock = 1
 likwid.catchSignal()
 
 if #arg == 0 then
@@ -126,7 +131,7 @@ if #arg == 0 then
     os.exit(0)
 end
 
-for opt,arg in likwid.getopt(arg, {"a", "c:", "C:", "e", "E:", "g:", "h", "H", "i", "m", "M:", "o:", "O", "P", "s:", "S:", "t:", "v", "V:","T:", "group:", "help", "info", "version", "verbose:", "output:", "skip:", "marker"}) do
+for opt,arg in likwid.getopt(arg, {"a", "c:", "C:", "e", "E:", "g:", "h", "H", "i", "m", "M:", "o:", "O", "P", "s:", "S:", "t:", "v", "V:", "T:", "f", "group:", "help", "info", "version", "verbose:", "output:", "skip:", "marker", "force", "stats"}) do
     if (type(arg) == "string") then
         local s,e = arg:find("-");
         if s == 1 then
@@ -157,6 +162,8 @@ for opt,arg in likwid.getopt(arg, {"a", "c:", "C:", "e", "E:", "g:", "h", "H", "
         print_events = true
     elseif (opt == "E") then
         print_event = arg
+    elseif opt == "f" or opt == "force" then
+        forceOverwrite = 1
     elseif opt == "g" or opt == "group" then
         table.insert(event_string_list, arg)
     elseif (opt == "H") then
@@ -199,7 +206,10 @@ for opt,arg in likwid.getopt(arg, {"a", "c:", "C:", "e", "E:", "g:", "h", "H", "
     elseif (opt == "T") then
         duration = likwid.parse_time(arg)
     elseif opt == "o" or opt == "output" then
-        local suffix = string.match(arg, ".-[^\\/]-%.?([^%.\\/]*)$")
+        local suffix = ""
+        if string.match(arg, "%.") then
+            suffix = string.match(arg, ".-[^\\/]-%.?([^%.\\/]*)$")
+        end
         if suffix ~= "txt" then
             use_csv = true
         end
@@ -207,18 +217,24 @@ for opt,arg in likwid.getopt(arg, {"a", "c:", "C:", "e", "E:", "g:", "h", "H", "
         outfile = outfile:gsub("%%p", likwid.getpid())
         outfile = outfile:gsub("%%j", likwid.getjid())
         outfile = outfile:gsub("%%r", likwid.getMPIrank())
-        io.output(outfile:gsub(string.match(arg, ".-[^\\/]-%.?([^%.\\/]*)$"),"tmp"))
+        io.output(outfile..".tmp")
         print = function(...) for k,v in pairs({...}) do io.write(v .. "\n") end end
     elseif (opt == "O") then
         use_csv = true
+    elseif (opt == "stats") then
+        print_stats = true
     elseif opt == "?" then
         print("Invalid commandline option -"..arg)
         os.exit(1)
+    elseif opt == "!" then
+        print("Option requires an argument")
+        os.exit(1)
     end
 end
 
 io.stdout:setvbuf("no")
 cpuinfo = likwid.getCpuInfo()
+cputopo = likwid.getCpuTopology()
 
 if not likwid.msr_available(access_flags) then
     if access_mode == 1 then
@@ -279,6 +295,7 @@ if print_events == true then
         end
         print_stdout(outstr)
     end
+    print_stdout("\n\n")
     print_stdout(string.format("This architecture has %d events.",#tab["Events"]))
     print_stdout("Event tags (tag, id, umask, counters<, options>):")
     for _, eventTab in pairs(tab["Events"]) do
@@ -294,12 +311,22 @@ if print_events == true then
 end
 
 if print_event ~= nil then
+    function case_insensitive_pattern(pattern)
+        local p = pattern:gsub("(%%?)(.)", function(percent, letter)
+            if percent ~= "" or not letter:match("%a") then
+              return percent .. letter
+            else
+                return string.format("[%s%s]", letter:lower(), letter:upper())
+            end
+        end)
+        return p
+    end
     local tab = likwid.getEventsAndCounters()
     local events = {}
     local counters = {}
     local outstr = ""
     for _, eventTab in pairs(tab["Events"]) do
-        if eventTab["Name"]:match(print_event) then
+        if eventTab["Name"]:match(case_insensitive_pattern(print_event)) then
             table.insert(events, eventTab)
         end
     end
@@ -333,14 +360,12 @@ if print_event ~= nil then
     os.exit(0)
 end
 
-num_avail_groups, avail_groups = likwid.get_groups()
-
+avail_groups = likwid.getGroups()
 if print_groups == true then
+    print_stdout(string.format("%11s\t%s","Group name", "Description"))
+    print_stdout(likwid.hline)
     for i,g in pairs(avail_groups) do
-        local gdata = likwid.get_groupdata(g)
-        if gdata ~= nil then
-            print_stdout(string.format("%10s\t%s",g,gdata["ShortDescription"]))
-        end
+        print_stdout(string.format("%11s\t%s",g["Name"], g["Info"]))
     end
     likwid.putTopology()
     likwid.putConfiguration()
@@ -359,10 +384,9 @@ if print_group_help == true then
             os.exit(1)
         end
         for i,g in pairs(avail_groups) do
-            if event_string == g then
-                local gdata = likwid.get_groupdata(event_string)
-                print_stdout(string.format("Group %s:",event_string))
-                print_stdout(gdata["LongDescription"])
+            if event_string == g["Name"] then
+                print_stdout(string.format("Group %s:",g["Name"]))
+                print_stdout(g["Long"])
             end
         end
     end
@@ -379,20 +403,23 @@ if #event_string_list == 0 and not print_info then
     os.exit(1)
 end
 
+if (cpuinfo["clock"] > 0) then
+    cpuClock = cpuinfo["clock"]
+else
+    cpuClock = likwid.getCpuClock()
+end
+
 if outfile == nil then
     print_stdout(likwid.hline)
     print_stdout(string.format("CPU name:\t%s",cpuinfo["osname"]))
     print_stdout(string.format("CPU type:\t%s",cpuinfo["name"]))
-    if (cpuinfo["clock"] > 0) then
-        print_stdout(string.format("CPU clock:\t%3.2f GHz",cpuinfo["clock"] * 1.E-09))
-    else
-        print_stdout(string.format("CPU clock:\t%3.2f GHz",likwid.getCpuClock() * 1.E-09))
-    end
+    print_stdout(string.format("CPU clock:\t%3.2f GHz",cpuClock * 1.E-09))
 end
 
 if print_info or verbose > 0 then
     print_stdout(string.format("CPU family:\t%u", cpuinfo["family"]))
     print_stdout(string.format("CPU model:\t%u", cpuinfo["model"]))
+    print_stdout(string.format("CPU short:\t%s", cpuinfo["short_name"]))
     print_stdout(string.format("CPU stepping:\t%u", cpuinfo["stepping"]))
     print_stdout(string.format("CPU features:\t%s", cpuinfo["features"]))
     P6_FAMILY = 6
@@ -404,10 +431,12 @@ if print_info or verbose > 0 then
         print_stdout(string.format("PERFMON number of fixed counters:\t%u",cpuinfo["perf_num_fixed_ctr"]))
     end
     print_stdout(likwid.hline)
-    if not print_info then likwid.printSupportedCPUs() end
-    likwid.putTopology()
-    likwid.putConfiguration()
-    if print_info then os.exit(0) end
+    if print_info then
+        likwid.printSupportedCPUs()
+        likwid.putTopology()
+        likwid.putConfiguration()
+        os.exit(0)
+    end
 end
 
 if use_stethoscope == false and use_timeline == false and use_marker == false then
@@ -415,48 +444,74 @@ if use_stethoscope == false and use_timeline == false and use_marker == false th
 end
 
 if use_wrapper and likwid.tablelength(arg)-2 == 0 and print_info == false then
-    print_stdout("No Executable can be found on commanlikwid.dline")
+    print_stdout("No Executable can be found on commandline")
     usage()
     likwid.putTopology()
     likwid.putConfiguration()
     os.exit(0)
 end
 
+if use_marker then
+    if likwid.access(markerFile, "rw") ~= -1 then
+        print_stdout(string.format("ERROR: MarkerAPI file %s not accessible. Maybe a remaining file of another user.", markerFile))
+        print_stdout("Please purge all MarkerAPI files from /tmp.")
+        os.exit(1)
+    end
+    if not pin_cpus then
+        print_stdout("Warning: The Marker API requires the application to run on the selected CPUs.")
+        print_stdout("Warning: likwid-perfctr pins the application only when using the -C command line option.")
+        print_stdout("Warning: LIKWID assumes that the application does it before the first instrumented code region is started.")
+        print_stdout("Warning: You can use the string in the environment variable LIKWID_THREADS to pin you application to")
+        print_stdout("Warning: to the CPUs specified after the -c command line option.")
+    end
+end
 
+if verbose == 0 then
+    likwid.setenv("LIKWID_SILENT","true")
+end
 
 if pin_cpus then
     local omp_threads = os.getenv("OMP_NUM_THREADS")
     if omp_threads == nil then
-        likwid.setenv("OMP_NUM_THREADS",tostring(num_cpus))
+        likwid.setenv("OMP_NUM_THREADS",tostring(math.tointeger(num_cpus)))
     elseif num_cpus > tonumber(omp_threads) then
         print_stdout(string.format("Environment variable OMP_NUM_THREADS already set to %s but %d cpus required", omp_threads,num_cpus))
     end
-    
-    if num_cpus > 1 then
-        local preload = os.getenv("LD_PRELOAD")
-        local pinString = tostring(cpulist[2])
-        for i=3,likwid.tablelength(cpulist) do
-            pinString = pinString .. "," .. cpulist[i]
-        end
-        pinString = pinString .. "," .. cpulist[1]
-        skipString = skip_mask
+    if os.getenv("CILK_NWORKERS") == nil then
+        likwid.setenv("CILK_NWORKERS", tostring(math.tointeger(num_cpus)))
+    end
+    if skip_mask then
+        likwid.setenv("LIKWID_SKIP",skip_mask)
+    end
+    likwid.setenv("KMP_AFFINITY","disabled")
 
-        likwid.setenv("KMP_AFFINITY","disabled")
+    if num_cpus > 1 then
+        local pinString = tostring(math.tointeger(cpulist[2]))
+        for i=3,likwid.tablelength(cpulist) do
+            pinString = pinString .. "," .. tostring(math.tointeger(cpulist[i]))
+        end
+        pinString = pinString .. "," .. tostring(math.tointeger(cpulist[1]))
         likwid.setenv("LIKWID_PIN", pinString)
-        likwid.setenv("LIKWID_SKIP",skipString)
-        likwid.setenv("LIKWID_SILENT","true")
+
+        local preload = os.getenv("LD_PRELOAD")
         if preload == nil then
             likwid.setenv("LD_PRELOAD",likwid.pinlibpath)
         else
             likwid.setenv("LD_PRELOAD",likwid.pinlibpath .. ":" .. preload)
         end
+    elseif num_cpus == 1 then
+        likwid.setenv("LIKWID_PIN", tostring(math.tointeger(cpulist[1])))
+        if verbose > 0 then
+            likwid.pinProcess(cpulist[1], 0)
+        else
+            likwid.pinProcess(cpulist[1], 1)
+        end
     end
-    likwid.pinProcess(cpulist[1], 1)
 end
 
 
 
-for i, event_string in pairs(event_string_list) do
+--[[for i, event_string in pairs(event_string_list) do
     local groupdata = likwid.get_groupdata(event_string)
     if groupdata == nil then
         print_stdout("Cannot read event string, it's neither a performance group nor a proper event string <event>:<counter>:<options>,...")
@@ -467,7 +522,7 @@ for i, event_string in pairs(event_string_list) do
     end
     table.insert(group_list, groupdata)
     event_string_list[i] = groupdata["EventString"]
-end
+end]]
 
 
 if set_access_modes then
@@ -483,6 +538,7 @@ if likwid.init(num_cpus, cpulist) < 0 then
     os.exit(1)
 end
 
+likwid.setenv("LIKWID_FORCE", tostring(forceOverwrite))
 for i, event_string in pairs(event_string_list) do
     if event_string:len() > 0 then
         local gid = likwid.addEventSet(event_string)
@@ -516,132 +572,136 @@ if use_marker == true then
     local str = table.concat(event_string_list, "|")
     likwid.setenv("LIKWID_EVENTS", str)
     likwid.setenv("LIKWID_THREADS", table.concat(cpulist,","))
+    likwid.setenv("LIKWID_FORCE", "-1")
 end
 
 execString = table.concat(arg," ",1, likwid.tablelength(arg)-2)
 if verbose == true then
     print_stdout(string.format("Executing: %s",execString))
 end
+local ldpath = os.getenv("LD_LIBRARY_PATH")
+local libpath = likwid.pinlibpath:match("([/%g]+)/%g+.so")
+if ldpath == nil then
+    likwid.setenv("LD_LIBRARY_PATH", libpath)
+elseif not ldpath:match(libpath) then
+    likwid.setenv("LD_LIBRARY_PATH", libpath..":"..ldpath)
+end
 
 
 if use_timeline == true then
     local cores_string = "CORES: "
     for i, cpu in pairs(cpulist) do
-        cores_string = cores_string .. tostring(cpu) .. " "
+        cores_string = cores_string .. tostring(cpu) .. "|"
+    end
+    io.stderr:write("# "..cores_string:sub(1,cores_string:len()-1).."\n")
+    for gid, group in pairs(group_list) do
+        local strlist = {}
+        if group["Metrics"] == nil then
+            for i,e in pairs(group["Events"]) do
+                table.insert(strlist, e["Event"])
+            end
+        else
+            for i,e in pairs(group["Metrics"]) do
+                table.insert(strlist, e["description"])
+            end
+        end
+        io.stderr:write("# "..table.concat(strlist, "|").."\n")
     end
-    print_stdout(cores_string:sub(1,cores_string:len()-1))
 end
 
-local ret = likwid.startCounters()
-if ret < 0 then
-    print_stdout(string.format("Error starting counters for cpu %d.",cpulist[ret * (-1)]))
-    os.exit(1)
-end
 
 
 io.stdout:flush()
-local int_results = {}
+local groupTime = {}
 if use_wrapper or use_timeline then
     local start = likwid.startClock()
     local stop = 0
     local alltime = 0
-    local groupsums = {}
     local nr_events = likwid.getNumberOfEvents(activeGroup)
     local nr_threads = likwid.getNumberOfThreads()
-    for i,g in pairs(group_ids) do
-        groupsums[g] = {}
-        for j=1, likwid.getNumberOfEvents(g) do
-            groupsums[g][j] = {}
-            for k=1,nr_threads do
-                groupsums[g][j][k] = 0
-            end
-        end
-    end
+    local firstrun = true
     
     if use_wrapper and #group_ids == 1 then
         duration = 30.E06
     end
 
-    local pid = likwid.startProgram(execString)
-    if not pid then
-        print_stdout("Failed to execute command: ".. execString)
-        likwid.stopCounters()
-        likwid.finalize()
-        likwid.putTopology()
-        likwid.putConfiguration()
+    local ret = likwid.startCounters()
+    if ret < 0 then
+        print_stdout(string.format("Error starting counters for cpu %d.",cpulist[ret * (-1)]))
         os.exit(1)
     end
+
+    local pid = nil
+    if pin_cpus then
+        pid = likwid.startProgram(execString, #cpulist, cpulist)
+    else
+        pid = likwid.startProgram(execString, 0, cpulist)
+    end
+
+    if not pid then
+        print_stdout("Failed to execute command: ".. execString)
+    end
+    start = likwid.startClock()
+    groupTime[activeGroup] = 0
     while true do
         if likwid.getSignalState() ~= 0 then
             likwid.killProgram()
             break
         end
-        if duration >= 1.E06 then
-            remain = sleep(duration/1.E06)
-            if remain > 0 or not likwid.checkProgram() then
-                io.stdout:flush()
-                break
-            end
-        else
-            status = usleep(duration)
-            if status ~= 0 or not likwid.checkProgram()then
-                io.stdout:flush()
-                break
-            end
+        local remain = likwid.sleep(duration)
+        if remain > 0 or not likwid.checkProgram(pid) then
+            io.stdout:flush()
+            break
         end
         if use_timeline == true then
             stop = likwid.stopClock()
-            likwid.readCounters()
+            likwid.stopCounters()
+            
             local time = likwid.getClock(start, stop)
-            alltime = alltime + time
-            int_results[alltime] = likwid.getResults()
-            local str = tostring(activeGroup) .. ","..tostring(nr_events) .. "," .. tostring(nr_threads) .. ","..tostring(alltime)
-            for ie, e in pairs(int_results[alltime][activeGroup]) do
-                for it, t in pairs(e) do
-                    
-                    groupsums[activeGroup][ie][it] = groupsums[activeGroup][ie][it] + t
-                    str = str .. "," .. tostring(groupsums[activeGroup][ie][it])
+            if likwid.getNumberOfMetrics(activeGroup) == 0 then
+                results = likwid.getLastResults()
+            else
+                results = likwid.getLastMetrics()
+            end
+            str = tostring(math.tointeger(activeGroup)) .. " "..tostring(#results[activeGroup]).." "..tostring(#cpulist).." "..tostring(time)
+            for i,l1 in pairs(results[activeGroup]) do
+                for j, value in pairs(l1) do
+                    str = str .. " " .. tostring(value)
                 end
             end
             io.stderr:write(str.."\n")
+            groupTime[activeGroup] = time
+            likwid.startCounters()
+        else
+            likwid.readCounters()
         end
         if #group_ids > 1 then
             likwid.switchGroup(activeGroup + 1)
             activeGroup = likwid.getIdOfActiveGroup()
+            if groupTime[activeGroup] == nil then
+                groupTime[activeGroup] = 0
+            end
             nr_events = likwid.getNumberOfEvents(activeGroup)
         end
-        start = likwid.startClock()
+        
     end
     stop = likwid.stopClock()
-    if use_timeline == true then
-        likwid.readCounters()
-        local time = likwid.getClock(start, stop)
-        alltime = alltime + time
-        int_results[time] = likwid.getResults()
-        local str = tostring(activeGroup) .. ","..tostring(nr_events) .. "," .. tostring(nr_threads) .. ","..tostring(alltime)
-        for ie, e in pairs(int_results[time][activeGroup]) do
-            for it, t in pairs(e) do
-                groupsums[activeGroup][ie][it] = groupsums[activeGroup][ie][it] + t
-                str = str .. "," .. tostring(groupsums[activeGroup][ie][it])
-            end
-        end
-        io.stderr:write(str.."\n")
-    end
 elseif use_stethoscope then
-    if duration >= 1.E06 then
-        sleep(duration/1.E06)
-    else
-        usleep(duration)
+    local ret = likwid.startCounters()
+    if ret < 0 then
+        print_stdout(string.format("Error starting counters for cpu %d.",cpulist[ret * (-1)]))
+        os.exit(1)
     end
+    likwid.sleep(duration)
 elseif use_marker then
+    local ret = likwid.startCounters()
+    if ret < 0 then
+        print_stdout(string.format("Error starting counters for cpu %d.",cpulist[ret * (-1)]))
+        os.exit(1)
+    end
     local ret = os.execute(execString)
     if ret == nil then
         print_stdout("Failed to execute command: ".. execString)
-        likwid.stopCounters()
-        likwid.finalize()
-        likwid.putTopology()
-        likwid.putConfiguration()
-        os.exit(1)
     end
 end
 
@@ -660,31 +720,31 @@ end
 
 
 if use_marker == true then
-    groups, results = likwid.getMarkerResults(markerFile, group_list, num_cpus)
-    if #groups == 0 and #results == 0 then
-        likwid.finalize()
-        likwid.putTopology()
-        likwid.putConfiguration()
-        os.exit(1)
-    end
-    likwid.print_markerOutput(groups, results, group_list, cpulist)
-else
-    results = likwid.getResults()
-    groups = {}
-    for g,gr in pairs(group_list) do
-        if groups[g] == nil then
-            groups[g] = {}
+    results, metrics = likwid.getMarkerResults(markerFile, cpulist)
+    if #results == 0 then
+        print_stdout("No regions could be found in Marker API result file")
+    else
+        for r=1, #results do
+            likwid.printOutput(results[r], metrics[r], cpulist, r, print_stats)
         end
-        groups[g]["ID"] = g
     end
-    likwid.printOutput(groups, results, group_list, cpulist)
+    os.remove(markerFile)
+elseif use_timeline == false then
+    results = likwid.getResults()
+    metrics = likwid.getMetrics()
+    likwid.printOutput(results, metrics, cpulist, nil, print_stats)
 end
 
 if outfile then
-    local suffix = string.match(outfile, ".-[^\\/]-%.?([^%.\\/]*)$")
-    local command = "<PREFIX>/share/likwid/filter/" .. suffix
-    local tmpfile = outfile:gsub("."..suffix,".tmp",1)
-    if likwid.access(command, "x") then
+    local suffix = ""
+    if string.match(outfile,"%.") then
+        suffix = string.match(outfile, ".-[^\\/]-%.?([^%.\\/]*)$")
+    end
+    local command = "<INSTALLED_PREFIX>/share/likwid/filter/" .. suffix
+    local tmpfile = outfile..".tmp"
+    if suffix == "" then
+        os.rename(tmpfile, outfile)
+    elseif suffix ~= "txt" and suffix ~= "csv" and likwid.access(command, "x") then
         print_stdout("Cannot find filter script, save output in CSV format to file "..outfile)
         os.rename(tmpfile, outfile)
     else
