@@ -1,4 +1,4 @@
-#!<PREFIX>/bin/likwid-lua
+#!<INSTALLED_BINPREFIX>/likwid-lua
 --[[
  * =======================================================================================
  *
@@ -7,13 +7,13 @@
  *      Description:  An application to use the timeline mode of likwid-perfctr to generate
  *                    realtime plots using feedGnuplot
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -30,12 +30,12 @@
  * =======================================================================================
 ]]
 
-package.path = '<PREFIX>/share/lua/?.lua;' .. package.path
+package.path = '<INSTALLED_PREFIX>/share/lua/?.lua;' .. package.path
 
 local likwid = require("likwid")
 
-PERFCTR="/home/rrze/unrz/unrz139/TMP/likwid-base/trunk/likwid-perfctr"
-FEEDGNUPLOT="/home/rrze/unrz/unrz139/TMP/likwid-base/trunk/perl/feedGnuplot"
+PERFCTR="<INSTALLED_BINPREFIX>/likwid-perfctr"
+FEEDGNUPLOT="<INSTALLED_BINPREFIX>/feedGnuplot"
 
 local predefined_plots = {
     FLOPS_DP = {
@@ -54,23 +54,25 @@ local predefined_plots = {
         y2title = nil,
         xtitle = "Time"
     },
-    L2_BAND = {
+    L2 = {
         perfgroup = "L2",
-        ymetricmatch = "L2 bandwidth [MBytes/s]",
+        ymetricmatch = "L2D load bandwidth [MBytes/s]",
         title = "L2 cache bandwidth",
-        ytitle = "Bandwidth [MBytes/s]",
-        y2title = nil,
+        ytitle = "Load Bandwidth [MBytes/s]",
+        y2metricmatch = "L2D evict bandwidth [MBytes/s]",
+        y2title = "Evict Bandwidth [MBytes/s]",
         xtitle = "Time"
     },
-    L3_BAND = {
+    L3 = {
         perfgroup = "L3",
-        ymetricmatch = "L3 bandwidth [MBytes/s]",
+        ymetricmatch = "L3 load bandwidth [MBytes/s]",
         title = "L3 cache bandwidth",
-        ytitle = "Bandwidth [MBytes/s]",
-        y2title = nil,
+        ytitle = "Load Bandwidth [MBytes/s]",
+        y2title = "Evict Bandwidth [MBytes/s]",
+        y2metricmatch = "L3 evict bandwidth [MBytes/s]",
         xtitle = "Time"
     },
-    MEM_BAND = {
+    MEM = {
         perfgroup = "MEM",
         ymetricmatch = "Memory bandwidth [MBytes/s]",
         title = "Memory bandwidth",
@@ -78,7 +80,7 @@ local predefined_plots = {
         y2title = nil,
         xtitle = "Time"
     },
-    QPI_BAND = {
+    QPI = {
         perfgroup = "QPI",
         ymetricmatch = "QPI data bandwidth [MByte/s]",
         title = "QPI bandwidth",
@@ -87,10 +89,10 @@ local predefined_plots = {
         xtitle = "Time",
         y2metricmatch = "QPI link bandwidth [MByte/s]"
     },
-    POWER = {
+    ENERGY = {
         perfgroup = "ENERGY",
         ymetricmatch = "Power [W]",
-        title = "Consumed power",
+        title = "Consumed energy",
         ytitle = "Power [W]",
         y2title = "Power DRAM [W]",
         y2metricmatch = "Power DRAM [W]",
@@ -137,6 +139,7 @@ local function usage()
     print("-C <list>\t\t Processor ids to pin threads and measure, e.g. 1,2-4,8")
     print("-g, --group <string>\t Preconfigured plot group or custom event set string with plot config. See man page for information.")
     print("-t, --time <time>\t Frequency in s, ms or us, e.g. 300ms, for the timeline mode of likwid-perfctr")
+    print("-f, --force\t\t Overwrite counter configuration although already in use")
     print("-d, --dump\t\t Print output as it is send to feedGnuplot.")
     print("-p, --plotdump\t\t Use dump functionality of feedGnuplot. Plots out plot configurations plus data to directly submit to gnuplot")
     print("--host <host>\t\t Run likwid-perfctr on the selected host using SSH. Evaluation and plotting is done locally.")
@@ -170,13 +173,14 @@ local nrgroups, allgroups = likwid.get_groups()
 local mfreq = 1.0
 local plotrange = 0
 local host = nil
+local force = false
 
 if #arg == 0 then
     usage()
     os.exit(0)
 end
 
-for opt,arg in likwid.getopt(arg, {"h","v","g:","C:","c:","t:","r:","a","d","p","help", "version","group:","time:","dump","range:","plotdump","all", "host:"}) do
+for opt,arg in likwid.getopt(arg, {"h","v","g:","C:","c:","t:","r:","a","d","p","f","help", "version","group:","time:","dump","range:","plotdump","all", "host:", "force"}) do
     if opt == "h" or opt == "help" then
         usage()
         os.exit(0)
@@ -203,6 +207,8 @@ for opt,arg in likwid.getopt(arg, {"h","v","g:","C:","c:","t:","r:","a","d","p",
         print_configs = true
     elseif opt == "host" then
         host = arg
+    elseif opt == "f" or opt == "force" then
+        force = true
     elseif opt == "?" then
         print("Invalid commandline option -"..arg)
         os.exit(1)
@@ -253,6 +259,7 @@ end
 
 for i, event_def in pairs(eventStrings) do
     local eventlist = likwid.stringsplit(event_def,",")
+
     event_string = nil
     plotgroup = nil
     plotgroupconfig = nil
@@ -312,17 +319,18 @@ for i, event_def in pairs(eventStrings) do
         for i,mconfig in pairs(groupdata["Metrics"]) do
             local mmatch = "%a*"..plotgroupconfig["ymetricmatch"]:gsub("%[","%%["):gsub("%]","%%]").."%a*"
             if mconfig["description"]:match(mmatch) then
-                formulalist = {{name=mconfig["description"], formula=mconfig["formula"]}}
+                formulalist = {{name=mconfig["description"], index=i}}
             end
             if plotgroupconfig["y2metricmatch"] ~= nil then
                 mmatch = "%a*"..plotgroupconfig["y2metricmatch"]:gsub("%[","%%["):gsub("%]","%%]").."%a*"
                 if mconfig["description"]:match(mmatch) then
-                    table.insert(formulalist, {name=mconfig["description"], formula=mconfig["formula"]})
+                    table.insert(formulalist, {name=mconfig["description"], index=i})
                 end
             end
         end
     end
-    for j,estr in pairs(likwid.stringsplit(outopts, ":")) do
+
+    --[[for j,estr in pairs(likwid.stringsplit(outopts, ":")) do
         if estr:match("^title=([%g%s]+)") then
             title = estr:match("^title=([%g%s]+)")
         elseif estr:match("^TITLE=([%g%s]+)") then
@@ -345,10 +353,22 @@ for i, event_def in pairs(eventStrings) do
             xtitle = estr:match("XTITLE=([%g%s]+)")
         elseif estr:match("[%g%s]+=[%g]+") then
             fname, form = estr:match("([%g%s]+)=([%g]+)")
-            if formulalist == nil then formulalist = {} end
-            table.insert(formulalist, {name=fname, formula=form})
+            if formulalist == nil then
+                formulalist = {}
+            end
+            if groupdata["Metrics"] ~= nil then
+                for i,mconfig in pairs(groupdata["Metrics"]) do
+                    if mconfig["description"]:match(fname) then
+                        table.insert(formulalist, {name=fname, index=i})
+                        break
+                    end
+                end
+            else
+                table.insert(formulalist, {name=fname, formula=form})
+            end
         end
-    end
+    end]]
+
     group_list[i]["eventstring"] = event_string
     group_list[i]["counterlist"] = {}
     for k=1,#groupdata["Events"] do
@@ -390,6 +410,9 @@ if pinning then
 else
     cmd = cmd .. string.format(" -c %s",table.concat(cpulist,","))
 end
+if force then
+    cmd = cmd .. " -f"
+end
 cmd = cmd .. string.format(" -t %s", timeline)
 
 for i, group in pairs(group_list) do
@@ -445,6 +468,7 @@ for i, group in pairs(group_list) do
             end
         end
     end
+    gnucmd = gnucmd .. " --set 'key outside bmargin bottom'"
     if plotdump then
         gnucmd = gnucmd .. " --dump"
     else
@@ -481,14 +505,11 @@ while true do
     local l = perfctr:read("*line")
     if l == nil or l:match("^%s*$") then
         break
-    elseif l:match("^ERROR") then
-        perfctr_exited = true
-        break
     end
-    if l:match("^%d+,%d+,%d+,[%d.]+,%d+") then
+    if l:match("^%d+ %d+ %d+ [%d.]+ %d+") then
         local data = {}
         local diff = {}
-        linelist = likwid.stringsplit(l, ",")
+        linelist = likwid.stringsplit(l, " ")
         group = tonumber(linelist[1])
         nr_events = tonumber(linelist[2])
         nr_threads = tonumber(linelist[3])
@@ -499,78 +520,18 @@ while true do
         table.remove(linelist, 1)
 
         for i=1,nr_events do
+            if data[i] == nil then data[i] = {} end
             for j=1,nr_threads do
-                if data[j] == nil then data[j] = {} end
-                data[j][group_list[group]["counterlist"][i]] = tonumber(linelist[1])
+                data[i][j] = tonumber(linelist[1])
                 table.remove(linelist, 1)
             end
         end
 
-        if time - oldtime <= mfreq + (0.1*mfreq) and time - oldtime >= mfreq - (0.1*mfreq) then
-            mtime[group] = time - oldtime
-        else
-            mtime[group] = mfreq
-        end
-
-        if olddata[group] == nil then
-            olddata[group] = {}
-            for l1,v1 in pairs(data) do
-                diff[l1] = {}
-                olddata[group][l1] = {}
-                if type(v1) == "table" then
-                    for l2,v2 in pairs(data[l1]) do
-                        diff[l1][l2] = data[l1][l2]
-                        olddata[group][l1][l2] = data[l1][l2]
-                    end
-                else
-                    diff[l1] = data[l1]
-                    olddata[group][l1] = data[group][l1]
-                end
-                diff[l1]["time"] = mtime[group]
-                diff[l1]["inverseClock"] = 1.0/clock
-            end
-        else
-            for l1,v1 in pairs(data) do
-                diff[l1] = {}
-                if type(v1) == "table" then
-                    for l2,v2 in pairs(v1) do
-                        diff[l1][l2] = data[l1][l2] - olddata[group][l1][l2]
-                        olddata[group][l1][l2] = data[l1][l2]
-                    end
-                else
-                    diff[l1] = data[l1] - olddata[group][l1]
-                    olddata[group][l1] = data[group][l1]
-                end
-                diff[l1]["time"] = mtime[group]
-                diff[l1]["inverseClock"] = 1.0/clock
-            end
-        end
-
-
         str = tostring(time)
-        if oldmetric[group] == nil then
-            oldmetric[group] = {}
-            for i, thread in pairs(diff) do
-                oldmetric[group][i] = {}
-                for j,fdesc in pairs(group_list[group]["formulas"]) do
-                    oldmetric[group][i][j] = likwid.calculate_metric(fdesc["formula"], thread)
-                    str = str .. " " .. tostring(oldmetric[group][i][j])
-                end
-            end
-        else
-            for i, thread in pairs(diff) do
-                for j,fdesc in pairs(group_list[group]["formulas"]) do
-                    local tmp = likwid.calculate_metric(fdesc["formula"], thread)
-                    if #group_list > 1 or fdesc["formula"]:match("(%/time)") == nil then
-                        str = str .. " " ..tostring(tmp)
-                    else
-                        if tmp - oldmetric[group][i][j] >= 0 then
-                            str = str .. " " .. tostring(tmp - oldmetric[group][i][j])
-                        else
-                            str = str .. " 0"
-                        end
-                    end
-                    oldmetric[group][i][j] = tmp
+        for f, flist in pairs(group_list[group]["formulas"]) do
+            if flist["index"] ~= nil then
+                for i=1,nr_threads do
+                    str = str .." ".. data[flist["index"]][i]
                 end
             end
         end
@@ -586,7 +547,7 @@ end
 
 if perfctr_exited == false then
     while likwid.getSignalState() == 0 do
-        sleep(1)
+        likwid.sleep(1E6)
     end
 end
 for i, group in pairs(group_list) do
