@@ -1,4 +1,4 @@
-#!<PREFIX>/bin/likwid-lua
+#!<INSTALLED_BINPREFIX>/likwid-lua
 --[[
  * =======================================================================================
  *
@@ -6,13 +6,13 @@
  *
  *      Description:  An application to pin a program including threads
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -28,7 +28,7 @@
  *
  * =======================================================================================
 ]]
-package.path = '<PREFIX>/share/lua/?.lua;' .. package.path
+package.path = '<INSTALLED_PREFIX>/share/lua/?.lua;' .. package.path
 
 local likwid = require("likwid")
 
@@ -92,7 +92,7 @@ sweep_sockets = false
 interleaved_policy = false
 print_domains = false
 cpu_list = {}
-skip_mask = "0x0"
+skip_mask = nil
 affinity = nil
 num_threads = 0
 
@@ -165,9 +165,16 @@ for opt,arg in likwid.getopt(arg, {"c:", "d:", "h", "i", "p", "q", "s:", "S", "t
         likwid.putAffinityInfo()
         likwid.putConfiguration()
         os.exit(1)
+    elseif opt == "!" then
+        print("Option requires an argument")
+        likwid.putTopology()
+        likwid.putAffinityInfo()
+        likwid.putConfiguration()
+        os.exit(1)
     end
 end
 
+
 if print_domains and num_threads > 0 then
     outstr = ""
     for i, cpu in pairs(cpu_list) do
@@ -193,6 +200,10 @@ end
 if num_threads == 0 then
     num_threads, cpu_list = likwid.cpustr_to_cpulist("N:0-"..cputopo["numHWThreads"]-1)
 end
+if (#arg == 0) then
+    print("Executable must be given on commandline")
+    os.exit(1)
+end
 
 if interleaved_policy then
     print("Set mem_policy to interleaved")
@@ -206,37 +217,49 @@ end
 
 local omp_threads = os.getenv("OMP_NUM_THREADS")
 if omp_threads == nil then
-    likwid.setenv("OMP_NUM_THREADS",tostring(num_threads))
+    likwid.setenv("OMP_NUM_THREADS",tostring(math.tointeger(num_threads)))
 elseif num_threads > tonumber(omp_threads) then
     print(string.format("Environment variable OMP_NUM_THREADS already set to %s but %d cpus required", omp_threads,num_threads))
 end
 
+likwid.setenv("KMP_AFFINITY","disabled")
+
+if os.getenv("CILK_NWORKERS") == nil then
+    likwid.setenv("CILK_NWORKERS", tostring(math.tointeger(num_threads)))
+end
+if skip_mask then
+    likwid.setenv("LIKWID_SKIP", skip_mask)
+end
 
 if num_threads > 1 then
-    local preload = os.getenv("LD_PRELOAD")
-    local pinString = tostring(cpu_list[2])
+    local pinString = tostring(math.tointeger(cpu_list[2]))
     for i=3,likwid.tablelength(cpu_list) do
-        pinString = pinString .. "," .. cpu_list[i]
+        pinString = pinString .. "," .. tostring(math.tointeger(cpu_list[i]))
     end
-    pinString = pinString .. "," .. cpu_list[1]
-    skipString = skip_mask
-
-    likwid.setenv("KMP_AFFINITY","disabled")
+    pinString = pinString .. "," .. tostring(math.tointeger(cpu_list[1]))
     likwid.setenv("LIKWID_PIN", pinString)
-    likwid.setenv("LIKWID_SKIP",skipString)
 
+    local preload = os.getenv("LD_PRELOAD")
     if preload == nil then
         likwid.setenv("LD_PRELOAD",likwid.pinlibpath)
     else
         likwid.setenv("LD_PRELOAD",likwid.pinlibpath .. ":" .. preload)
     end
+    local ldpath = os.getenv("LD_LIBRARY_PATH")
+    local libpath = likwid.pinlibpath:match("([/%g]+)/%g+.so")
+    if ldpath == nil then
+        likwid.setenv("LD_LIBRARY_PATH", libpath)
+    elseif not ldpath:match(libpath) then
+        likwid.setenv("LD_LIBRARY_PATH", libpath..":"..ldpath)
+    end
+else
+    likwid.setenv("LIKWID_PIN", tostring(math.tointeger(cpu_list[1])))
+    likwid.pinProcess(cpu_list[1], quiet)
 end
 
-likwid.pinProcess(cpu_list[1], quiet)
 local exec = table.concat(arg," ",1, likwid.tablelength(arg)-2)
-local err
-err = os.execute(exec)
-if (err == false) then
+local pid = likwid.startProgram(exec, num_threads, cpu_list)
+if (pid == nil) then
     print("Failed to execute command: ".. exec)
     likwid.putTopology()
     likwid.putAffinityInfo()
@@ -244,6 +267,8 @@ if (err == false) then
     os.exit(1)
 end
 
+likwid.waitpid(pid)
+
 likwid.putAffinityInfo()
 likwid.putTopology()
 likwid.putConfiguration()
