@@ -1,4 +1,4 @@
-#!<PREFIX>/bin/likwid-lua
+#!<INSTALLED_BINPREFIX>/likwid-lua
 --[[
  * =======================================================================================
  *
@@ -7,13 +7,13 @@
  *      Description:  An application to get information about power 
  *      consumption on architectures implementing the RAPL interface.
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -29,7 +29,7 @@
  *
  * =======================================================================================
 ]]
-package.path = '<PREFIX>/share/lua/?.lua;' .. package.path
+package.path = '<INSTALLED_PREFIX>/share/lua/?.lua;' .. package.path
 
 local likwid = require("likwid")
 
@@ -78,6 +78,8 @@ else
     access_mode = config["daemonMode"]
 end
 time_interval = 2.E06
+time_orig = "2s"
+read_interval = 30.E06
 sockets = {}
 domainList = {"PKG", "PP0", "PP1", "DRAM"}
 
@@ -132,10 +134,14 @@ for opt,arg in likwid.getopt(arg, {"V:", "c:", "h", "i", "M:", "p", "s:", "v", "
         likwid.setVerbosity(verbose)
     elseif (opt == "s") then
         time_interval = likwid.parse_time(arg)
+        time_orig = arg
         stethoscope = true
     elseif opt == "?" then
         print("Invalid commandline option -"..arg)
         os.exit(1)
+    elseif opt == "!" then
+        print("Option requires an argument")
+        os.exit(1)
     end
 end
 
@@ -236,7 +242,7 @@ if (stethoscope) and (time_interval < power["timeUnit"]) then
 end
 
 local execString = ""
-if (use_perfctr) then
+if use_perfctr then
     affinity = likwid.getAffinityInfo()
     argString = ""
     for i,socket in pairs(sockets) do
@@ -245,70 +251,118 @@ if (use_perfctr) then
             argString = argString .. "@"
         end
     end
-    execString = string.format("<PREFIX>/bin/likwid-perfctr -C %s -g CLOCK ",argString)
+    execString = string.format("<INSTALLED_PREFIX>/bin/likwid-perfctr -C %s -f -g CLOCK ",argString)
 end
 
 
 if #arg == 0 then
-    stethoscope = true
+    if use_perfctr then
+        execString = execString .. string.format(" -S %s ", time_orig)
+        stethoscope = false
+    else
+        stethoscope = true
+    end
 else
-    for i=1,#arg do
-        execString = execString .. arg[i] .. " "
+    if use_perfctr then
+        execString = execString .. table.concat(arg," ",1, likwid.tablelength(arg)-2)
+    else
+        execString = table.concat(arg," ",1, likwid.tablelength(arg)-2)
     end
 end
 
 if not print_info and not print_temp then
-    for i,socket in pairs(sockets) do
-        cpu = cpulist[i]
-        for idx, dom in pairs(domainList) do
-            if (power["domains"][dom]["supportStatus"]) then before[cpu][dom] = likwid.startPower(cpu, idx) end
+    if stethoscope or (#arg > 0 and not use_perfctr) then
+        for i,socket in pairs(sockets) do
+            cpu = cpulist[i]
+            for idx, dom in pairs(domainList) do
+                if (power["domains"][dom]["supportStatus"]) then before[cpu][dom] = likwid.startPower(cpu, idx) end
+            end
         end
-    end
-    time_before = likwid.startClock()
-    if (stethoscope) then
-        if time_interval >= 1.E06 then
-            sleep(time_interval/1.E06)
+
+        time_before = likwid.startClock()
+        if stethoscope then
+            if read_interval < time_interval then
+                while ((read_interval <= time_interval) and (time_interval > 0)) do
+                    likwid.sleep(read_interval)
+                    for i,socket in pairs(sockets) do
+                        cpu = cpulist[i]
+                        for idx, dom in pairs(domainList) do
+                            if (power["domains"][dom]["supportStatus"]) then after[cpu][dom] = likwid.stopPower(cpu, idx) end
+                        end
+                    end
+                    time_interval = time_interval - read_interval
+                    if time_interval < read_interval then
+                        read_interval = time_interval
+                    end
+                end
+            else
+                likwid.sleep(time_interval)
+            end
         else
-            usleep(time_interval)
+            local pid = likwid.startProgram(execString, 0, {})
+            if not pid then
+                print(string.format("Failed to execute %s!",execString))
+                likwid.finalize()
+                os.exit(1)
+            end
+            while true do
+                if likwid.getSignalState() ~= 0 then
+                    likwid.killProgram()
+                    break
+                end
+                local remain = likwid.sleep(read_interval)
+                for i,socket in pairs(sockets) do
+                    cpu = cpulist[i]
+                    for idx, dom in pairs(domainList) do
+                        if (power["domains"][dom]["supportStatus"]) then after[cpu][dom] = likwid.stopPower(cpu, idx) end
+                    end
+                end
+                if remain > 0 or not likwid.checkProgram(pid) then
+                    io.stdout:flush()
+                    break
+                end
+            end
         end
+        time_after = likwid.stopClock()
+
+        for i,socket in pairs(sockets) do
+            cpu = cpulist[i]
+            for idx, dom in pairs(domainList) do
+                if (power["domains"][dom]["supportStatus"]) then after[cpu][dom] = likwid.stopPower(cpu, idx) end
+            end
+        end
+        runtime = likwid.getClock(time_before, time_after)
+
+        print(likwid.hline)
+        print(string.format("Runtime: %g s",runtime))
+
+        for i,socket in pairs(sockets) do
+            cpu = cpulist[i]
+            print(string.format("Measure for socket %d on CPU %d", socket,cpu ))
+            for j, dom in pairs(domainList) do
+                if power["domains"][dom]["supportStatus"] then
+                    local energy = likwid.calcPower(before[cpu][dom], after[cpu][dom], 0)
+                    print(string.format("Domain %s:", dom))
+                    print(string.format("Energy consumed: %g Joules",energy))
+                    print(string.format("Power consumed: %g Watt",energy/runtime))
+                end
+            end
+            if i < #sockets then print("") end
+        end
+        print(likwid.hline)
     else
         err = os.execute(execString)
-        if (err == false) then
+        if err == false then
             print(string.format("Failed to execute %s!",execString))
+            likwid.putPowerInfo()
             likwid.finalize()
             os.exit(1)
         end
     end
-    time_after = likwid.stopClock()
-
-    for i,socket in pairs(sockets) do
-        cpu = cpulist[i]
-        for idx, dom in pairs(domainList) do
-            if (power["domains"][dom]["supportStatus"]) then after[cpu][dom] = likwid.stopPower(cpu, idx) end
-        end
-    end
-    runtime = likwid.getClock(time_before, time_after)
-    print(likwid.hline)
-    print(string.format("Runtime: %g s",runtime))
-
-    for i,socket in pairs(sockets) do
-        cpu = cpulist[i]
-        print(string.format("Measure for socket %d on CPU %d", socket,cpu ))
-        for j, dom in pairs(domainList) do
-            if power["domains"][dom]["supportStatus"] then
-                local energy = likwid.calcPower(before[cpu][dom], after[cpu][dom], 0)
-                print(string.format("Energy consumed: %g Joules",energy))
-                print(string.format("Power consumed: %g Watt",energy/runtime))
-            end
-        end
-        if i < #sockets then print("") end
-    end
-    print(likwid.hline)
 end
 
 if print_temp and (string.find(cpuinfo["features"],"TM2") ~= nil) then
     print(likwid.hline)
-    likwid.initTemp(cpulist[i]);
     print("Current core temperatures:");
     for i=1,cputopo["numSockets"] do
         local tag = "S" .. tostring(i-1)
@@ -316,10 +370,12 @@ if print_temp and (string.find(cpuinfo["features"],"TM2") ~= nil) then
             if domain["tag"] == tag then
                 for j=1,#domain["processorList"] do
                     local cpuid = domain["processorList"][j]
+                    likwid.initTemp(cpuid);
                     if (fahrenheit) then
-                        print(string.format("Socket %d Core %d: %u F",i-1,cpuid, 1.8*likwid.readTemp(cpuid)+32));
+                        local f = 1.8*tonumber(likwid.readTemp(cpuid))+32
+                        print(string.format("Socket %d Core %d: %.0f F",i-1,cpuid, f));
                     else
-                        print(string.format("Socket %d Core %d: %u C",i-1,cpuid, likwid.readTemp(cpuid)));
+                        print(string.format("Socket %d Core %d: %.0f C",i-1,cpuid, tonumber(likwid.readTemp(cpuid))));
                     end
                 end
             end
@@ -328,4 +384,5 @@ if print_temp and (string.find(cpuinfo["features"],"TM2") ~= nil) then
     print(likwid.hline)
 end
 
+likwid.putPowerInfo()
 likwid.finalize()
