@@ -5,13 +5,13 @@
  *
  *      Description:  Header File HPM access Module
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -31,11 +31,14 @@
 #ifndef ACCESS_H
 #define ACCESS_H
 
+void HPMmode(int mode);
 int HPMinit(void);
 int HPMinitialized(void);
 int HPMaddThread(int cpu_id);
-void HPMfinalize(void);
+void HPMfinalize();
 int HPMread(int cpu_id, PciDeviceIndex dev, uint32_t reg, uint64_t* data);
 int HPMwrite(int cpu_id, PciDeviceIndex dev, uint32_t reg, uint64_t data);
+int HPMcheck(PciDeviceIndex dev, int cpu_id);
+
 
 #endif
