@@ -5,13 +5,13 @@
  *
  *      Description:  Types file for libperfctr module.
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,16 +38,21 @@ typedef struct LikwidThreadResults{
     double time;
     TimerData startTime;
     int groupID;
+    int cpuID;
     uint32_t count;
     double StartPMcounters[NUM_PMC];
+    int StartOverflows[NUM_PMC];
     double PMcounters[NUM_PMC];
 } LikwidThreadResults;
 
 typedef struct {
     bstring  tag;
     int groupID;
+    int threadCount;
+    int eventCount;
     double*  time;
     uint32_t*  count;
+    int* cpulist;
     double** counters;
 } LikwidResults;
 
