@@ -6,14 +6,14 @@
  *      Description:  Header File Power Module
  *                    Implements Intel RAPL Interface.
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -36,7 +36,6 @@
 #include <types.h>
 #include <registers.h>
 #include <bitUtil.h>
-#include <msr.h>
 #include <error.h>
 #include <access.h>
 
@@ -84,7 +83,7 @@ power_start(PowerData* data, int cpuId, PowerType type)
             uint64_t result = 0;
             data->before = 0;
             CHECK_MSR_READ_ERROR(HPMread(cpuId, MSR_DEV, power_regs[type], &result))
-            data->before = extractBitField(result,32,0);
+            data->before = field64(result, 0, 32);
             data->domain = type;
             return 0;
         }
@@ -111,7 +110,7 @@ power_stop(PowerData* data, int cpuId, PowerType type)
             uint64_t result = 0;
             data->after = 0;
             CHECK_MSR_READ_ERROR(HPMread(cpuId, MSR_DEV, power_regs[type], &result))
-            data->after = extractBitField(result,32,0);
+            data->after = field64(result, 0, 32);
             data->domain = type;
             return 0;
         }
@@ -149,7 +148,7 @@ power_read(int cpuId, uint64_t reg, uint32_t *data)
             uint64_t result = 0;
             *data = 0;
             CHECK_MSR_READ_ERROR(HPMread(cpuId, MSR_DEV, reg, &result))
-            *data = extractBitField(result,32,0);
+            *data = field64(result, 0, 32);
             return 0;
         }
         else
@@ -185,7 +184,7 @@ power_tread(int socket_fd, int cpuId, uint64_t reg, uint32_t *data)
             uint64_t result = 0;
             *data = 0;
             CHECK_MSR_READ_ERROR(HPMread(cpuId, MSR_DEV, reg, &result))
-            *data = extractBitField(result,32,0);
+            *data = field64(result, 0, 32);
             return 0;
         }
         else
