@@ -5,14 +5,14 @@
  *
  *      Description:  Register Defines for the perfmon module
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -92,20 +92,20 @@
 #define MSR_UNCORE_PMC6                0x3B6
 #define MSR_UNCORE_PMC7                0x3B7
 /* 
- * Perfmon V4 (starting with Haswell, according to 
+ * Perfmon V3 (starting with Haswell, according to 
  * Intel software developers guide also for SandyBridge,
- * IvyBridge not mentioned in this section) 
+ * IvyBridge not mentioned in this section)
  */
-#define MSR_UNC_PERF_GLOBAL_CTRL       MSR_UNCORE_PERF_GLOBAL_CTRL
-#define MSR_UNC_PERF_GLOBAL_STATUS     MSR_UNCORE_PERF_GLOBAL_STATUS
-#define MSR_UNC_PERF_GLOBAL_OVF_CTRL   MSR_UNCORE_PERF_GLOBAL_OVF_CTRL
-#define MSR_UNC_PERF_FIXED_CTRL        MSR_UNCORE_FIXED_CTR0
-#define MSR_UNC_PERF_FIXED_CTR         MSR_UNCORE_FIXED_CTR_CTRL
-#define MSR_UNC_ARB_PERFEVTSEL0        MSR_UNCORE_PMC2
-#define MSR_UNC_ARB_PERFEVTSEL1        MSR_UNCORE_PMC3
-#define MSR_UNC_ARB_CTR0               MSR_UNCORE_PMC0
-#define MSR_UNC_ARB_CTR1               MSR_UNCORE_PMC1
-#define MSR_UNC_CBO_CONFIG             MSR_UNCORE_ADDR_OPCODE_MATCH
+#define MSR_UNC_PERF_GLOBAL_CTRL       0x391
+#define MSR_UNC_PERF_GLOBAL_STATUS     0x392
+#define MSR_UNC_PERF_GLOBAL_OVF_CTRL   0x393
+#define MSR_UNC_PERF_FIXED_CTRL        0x394
+#define MSR_UNC_PERF_FIXED_CTR         0x395
+#define MSR_UNC_ARB_PERFEVTSEL0        0x3B2
+#define MSR_UNC_ARB_PERFEVTSEL1        0x3B3
+#define MSR_UNC_ARB_CTR0               0x3B0
+#define MSR_UNC_ARB_CTR1               0x3B1
+#define MSR_UNC_CBO_CONFIG             0x396
 #define MSR_UNC_CBO_0_PERFEVTSEL0      0x700
 #define MSR_UNC_CBO_0_PERFEVTSEL1      0x701
 #define MSR_UNC_CBO_0_CTR0             0x706
@@ -122,6 +122,38 @@
 #define MSR_UNC_CBO_3_PERFEVTSEL1      0x731
 #define MSR_UNC_CBO_3_CTR0             0x736
 #define MSR_UNC_CBO_3_CTR1             0x737
+/* Perfmon V4 starting with Skylake */
+#define MSR_V4_PERF_GLOBAL_STATUS       0x38E
+#define MSR_V4_PERF_GLOBAL_STATUS_SET   0x391
+#define MSR_V4_PERF_GLOBAL_STATUS_RESET 0x390
+#define MSR_V4_PERF_GLOBAL_INUSE        0x392
+#define MSR_V4_PEBS_FRONTEND            0x3F7
+#define MSR_V4_UNC_PERF_GLOBAL_CTRL     0xE01
+#define MSR_V4_UNC_PERF_GLOBAL_STATUS   0xE02
+#define MSR_V4_UNC_PERF_FIXED_CTRL      0x394
+#define MSR_V4_UNC_PERF_FIXED_CTR       0x395
+#define MSR_V4_ARB_PERF_FIXED_CTRL0      0x3B2
+#define MSR_V4_ARB_PERF_FIXED_CTR0       0x3B0
+#define MSR_V4_ARB_PERF_FIXED_CTRL1      0x3B3
+#define MSR_V4_ARB_PERF_FIXED_CTR1       0x3B1
+#define MSR_V4_C0_PERF_FIXED_CTRL0      0x700
+#define MSR_V4_C0_PERF_FIXED_CTR0       0x706
+#define MSR_V4_C0_PERF_FIXED_CTRL1      0x701
+#define MSR_V4_C0_PERF_FIXED_CTR1       0x707
+#define MSR_V4_C1_PERF_FIXED_CTRL0      0x710
+#define MSR_V4_C1_PERF_FIXED_CTR0       0x716
+#define MSR_V4_C1_PERF_FIXED_CTRL1      0x711
+#define MSR_V4_C1_PERF_FIXED_CTR1       0x717
+#define MSR_V4_C2_PERF_FIXED_CTRL0      0x720
+#define MSR_V4_C2_PERF_FIXED_CTR0       0x726
+#define MSR_V4_C2_PERF_FIXED_CTRL1      0x721
+#define MSR_V4_C2_PERF_FIXED_CTR1       0x727
+#define MSR_V4_C3_PERF_FIXED_CTRL0      0x730
+#define MSR_V4_C3_PERF_FIXED_CTR0       0x736
+#define MSR_V4_C3_PERF_FIXED_CTRL1      0x731
+#define MSR_V4_C3_PERF_FIXED_CTR1       0x737
+/* V4 Uncore registers the same as in V3 */
+
 /* Xeon Phi */
 #define MSR_MIC_TSC                   0x010
 #define MSR_MIC_PERFEVTSEL0           0x028
@@ -132,6 +164,10 @@
 #define MSR_MIC_PERF_GLOBAL_STATUS    0x02D
 #define MSR_MIC_PERF_GLOBAL_OVF_CTRL  0x02E
 #define MSR_MIC_PERF_GLOBAL_CTRL      0x02F
+/* Xeon Phi (Knights Landing)*/
+#define MSR_MIC2_PMC0                 0x4C1
+#define MSR_MIC2_PMC1                 0x4C2
+#define MSR_MIC2_TURBO_RATIO_LIMIT    0x1AD
 
 
 /* Core v1/v2 type uncore
@@ -707,6 +743,84 @@
 #define MSR_UNC_V3_C17_PMON_CTR2           0xF1A
 #define MSR_UNC_V3_C17_PMON_CTR3           0xF1B
 
+#define MSR_UNC_V3_C18_PMON_BOX_CTL        0xF20
+#define MSR_UNC_V3_C18_PMON_BOX_STATUS     0xF27
+#define MSR_UNC_V3_C18_PMON_BOX_FILTER0    0xF25
+#define MSR_UNC_V3_C18_PMON_BOX_FILTER1    0xF26
+#define MSR_UNC_V3_C18_PMON_CTL0           0xF21
+#define MSR_UNC_V3_C18_PMON_CTL1           0xF22
+#define MSR_UNC_V3_C18_PMON_CTL2           0xF23
+#define MSR_UNC_V3_C18_PMON_CTL3           0xF24
+#define MSR_UNC_V3_C18_PMON_CTR0           0xF28
+#define MSR_UNC_V3_C18_PMON_CTR1           0xF29
+#define MSR_UNC_V3_C18_PMON_CTR2           0xF2A
+#define MSR_UNC_V3_C18_PMON_CTR3           0xF2B
+
+#define MSR_UNC_V3_C19_PMON_BOX_CTL        0xF30
+#define MSR_UNC_V3_C19_PMON_BOX_STATUS     0xF37
+#define MSR_UNC_V3_C19_PMON_BOX_FILTER0    0xF35
+#define MSR_UNC_V3_C19_PMON_BOX_FILTER1    0xF36
+#define MSR_UNC_V3_C19_PMON_CTL0           0xF31
+#define MSR_UNC_V3_C19_PMON_CTL1           0xF32
+#define MSR_UNC_V3_C19_PMON_CTL2           0xF33
+#define MSR_UNC_V3_C19_PMON_CTL3           0xF34
+#define MSR_UNC_V3_C19_PMON_CTR0           0xF38
+#define MSR_UNC_V3_C19_PMON_CTR1           0xF39
+#define MSR_UNC_V3_C19_PMON_CTR2           0xF3A
+#define MSR_UNC_V3_C19_PMON_CTR3           0xF3B
+
+#define MSR_UNC_V3_C20_PMON_BOX_CTL        0xF40
+#define MSR_UNC_V3_C20_PMON_BOX_STATUS     0xF47
+#define MSR_UNC_V3_C20_PMON_BOX_FILTER0    0xF45
+#define MSR_UNC_V3_C20_PMON_BOX_FILTER1    0xF46
+#define MSR_UNC_V3_C20_PMON_CTL0           0xF41
+#define MSR_UNC_V3_C20_PMON_CTL1           0xF42
+#define MSR_UNC_V3_C20_PMON_CTL2           0xF43
+#define MSR_UNC_V3_C20_PMON_CTL3           0xF44
+#define MSR_UNC_V3_C20_PMON_CTR0           0xF48
+#define MSR_UNC_V3_C20_PMON_CTR1           0xF49
+#define MSR_UNC_V3_C20_PMON_CTR2           0xF4A
+#define MSR_UNC_V3_C20_PMON_CTR3           0xF4B
+
+#define MSR_UNC_V3_C21_PMON_BOX_CTL        0xF50
+#define MSR_UNC_V3_C21_PMON_BOX_STATUS     0xF57
+#define MSR_UNC_V3_C21_PMON_BOX_FILTER0    0xF55
+#define MSR_UNC_V3_C21_PMON_BOX_FILTER1    0xF56
+#define MSR_UNC_V3_C21_PMON_CTL0           0xF51
+#define MSR_UNC_V3_C21_PMON_CTL1           0xF52
+#define MSR_UNC_V3_C21_PMON_CTL2           0xF53
+#define MSR_UNC_V3_C21_PMON_CTL3           0xF54
+#define MSR_UNC_V3_C21_PMON_CTR0           0xF58
+#define MSR_UNC_V3_C21_PMON_CTR1           0xF59
+#define MSR_UNC_V3_C21_PMON_CTR2           0xF5A
+#define MSR_UNC_V3_C21_PMON_CTR3           0xF5B
+
+#define MSR_UNC_V3_C22_PMON_BOX_CTL        0xF60
+#define MSR_UNC_V3_C22_PMON_BOX_STATUS     0xF67
+#define MSR_UNC_V3_C22_PMON_BOX_FILTER0    0xF65
+#define MSR_UNC_V3_C22_PMON_BOX_FILTER1    0xF66
+#define MSR_UNC_V3_C22_PMON_CTL0           0xF61
+#define MSR_UNC_V3_C22_PMON_CTL1           0xF62
+#define MSR_UNC_V3_C22_PMON_CTL2           0xF63
+#define MSR_UNC_V3_C22_PMON_CTL3           0xF64
+#define MSR_UNC_V3_C22_PMON_CTR0           0xF68
+#define MSR_UNC_V3_C22_PMON_CTR1           0xF69
+#define MSR_UNC_V3_C22_PMON_CTR2           0xF6A
+#define MSR_UNC_V3_C22_PMON_CTR3           0xF6B
+
+#define MSR_UNC_V3_C23_PMON_BOX_CTL        0xF70
+#define MSR_UNC_V3_C23_PMON_BOX_STATUS     0xF77
+#define MSR_UNC_V3_C23_PMON_BOX_FILTER0    0xF75
+#define MSR_UNC_V3_C23_PMON_BOX_FILTER1    0xF76
+#define MSR_UNC_V3_C23_PMON_CTL0           0xF71
+#define MSR_UNC_V3_C23_PMON_CTL1           0xF72
+#define MSR_UNC_V3_C23_PMON_CTL2           0xF73
+#define MSR_UNC_V3_C23_PMON_CTL3           0xF74
+#define MSR_UNC_V3_C23_PMON_CTR0           0xF78
+#define MSR_UNC_V3_C23_PMON_CTR1           0xF79
+#define MSR_UNC_V3_C23_PMON_CTR2           0xF7A
+#define MSR_UNC_V3_C23_PMON_CTR3           0xF7B
+
 /* Sbox */
 #define MSR_UNC_V3_S0_PMON_BOX_CTL         0x720
 #define MSR_UNC_V3_S0_PMON_BOX_STATUS      0x725
@@ -1185,6 +1299,9 @@
 #define MSR_DRAM_ENERGY_STATUS          0x619
 #define MSR_DRAM_PERF_STATUS            0x61B
 #define MSR_DRAM_POWER_INFO             0x61C
+#define MSR_PLATFORM_ENERGY_STATUS      0x64D
+#define MSR_PLATFORM_POWER_LIMIT        0x65C
+
 /* Intel Silvermont's RAPL registers */
 #define MSR_PKG_POWER_INFO_SILVERMONT   0x66E
 
@@ -1195,12 +1312,22 @@
 
 /* Turbo Boost Interface */
 #define MSR_IA32_MISC_ENABLE            0x1A0
+#define MSR_PREFETCH_ENABLE             0x1A4
 #define MSR_PLATFORM_INFO               0x0CE
 #define MSR_TURBO_POWER_CURRENT_LIMIT   0x1AC
 #define MSR_TURBO_RATIO_LIMIT           0x1AD
 #define MSR_TURBO_RATIO_LIMIT1          0x1AE
 #define MSR_TURBO_RATIO_LIMIT2          0x1AF
+#define MSR_TURBO_RATIO_LIMIT3          0x1AC
 
+/* MISC Intel register */
+#define MSR_MPERF                       0xE7
+#define MSR_APERF                       0xE8
+#define MSR_PPERF                       0x64E
+#define MSR_WEIGHTED_CORE_C0            0x658
+#define MSR_ANY_CORE_C0                 0x659
+#define MSR_ANY_GFXE_C0                 0x65A
+#define MSR_CORE_GFXE_OVERLAP_C0        0x65B
 /*
  * AMD
  */
