@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of registers.
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -70,6 +70,8 @@ typedef enum {
     PMC187, PMC188, PMC189, PMC190, PMC191, PMC192,
     PMC193, PMC194, PMC195, PMC196, PMC197, PMC198,
     PMC199, PMC200, PMC201, PMC202, PMC203, PMC204,
+    PMC205, PMC206, PMC207, PMC208, PMC209, PMC210,
+    PMC211, PMC212, PMC213, PMC214, PMC215, PMC216,
     NUM_PMC
 } RegisterIndex;
 
@@ -93,12 +95,14 @@ typedef enum {
     CBOX9, CBOX10, CBOX11,
     CBOX12, CBOX13, CBOX14,
     CBOX15, CBOX16, CBOX17,
+    CBOX18, CBOX19, CBOX20,
+    CBOX21, CBOX22, CBOX23,
     PBOX,
     UBOX,
     UBOXFIX,
     IBOX0, IBOX1,
-    QBOX0, QBOX1,
-    QBOX0FIX, QBOX1FIX,
+    QBOX0, QBOX1, QBOX2,
+    QBOX0FIX, QBOX1FIX, QBOX2FIX,
     NUM_UNITS, NOTYPE, MAX_UNITS
 } RegisterType;
 
@@ -128,6 +132,7 @@ static char* RegisterTypeNames[MAX_UNITS] = {
     [BBOX1] = "Home Agent box 1",
     [RBOX0] = "Routing box 0",
     [RBOX1] = "Routing box 1",
+    [RBOX2] = "Routing box 2",
     [WBOX] = "Power control box",
     [WBOX0FIX] = "Power control box fixed counter 0",
     [WBOX1FIX] = "Power control box fixed counter 1",
@@ -157,6 +162,12 @@ static char* RegisterTypeNames[MAX_UNITS] = {
     [CBOX15] = "Caching Agent box 15",
     [CBOX16] = "Caching Agent box 16",
     [CBOX17] = "Caching Agent box 17",
+    [CBOX18] = "Caching Agent box 18",
+    [CBOX19] = "Caching Agent box 19",
+    [CBOX20] = "Caching Agent box 20",
+    [CBOX21] = "Caching Agent box 21",
+    [CBOX22] = "Caching Agent box 22",
+    [CBOX23] = "Caching Agent box 23",
     [PBOX] = "Physical Layer box",
     [UBOX] = "System Configuration box",
     [UBOXFIX] = "System Configuration box fixed counter",
