@@ -5,13 +5,13 @@
  *
  *      Description:  Header File of topology backend using the hwloc library
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,7 +38,7 @@
 
 extern hwloc_topology_t hwloc_topology;
 
-int hwloc_record_objs_of_type_below_obj(hwloc_topology_t t, hwloc_obj_t obj, hwloc_obj_type_t type, int* index, uint32_t **list);
+int likwid_hwloc_record_objs_of_type_below_obj(hwloc_topology_t t, hwloc_obj_t obj, hwloc_obj_type_t type, int* index, uint32_t **list);
 
 
 
@@ -46,6 +46,7 @@ void hwloc_init_cpuInfo(cpu_set_t cpuSet);
 void hwloc_init_cpuFeatures(void);
 void hwloc_init_nodeTopology(cpu_set_t cpuSet);
 void hwloc_init_cacheTopology(void);
+void hwloc_close(void);
 
 
 #endif
