@@ -5,14 +5,14 @@
  *
  *      Description:  Marker API interface of module perfmon
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -52,19 +52,6 @@
 #include <access.h>
 
 #include <perfmon.h>
-#include <perfmon_core2_counters.h>
-#include <perfmon_haswell_counters.h>
-#include <perfmon_interlagos_counters.h>
-#include <perfmon_kabini_counters.h>
-#include <perfmon_k10_counters.h>
-#include <perfmon_nehalem_counters.h>
-#include <perfmon_phi_counters.h>
-#include <perfmon_pm_counters.h>
-#include <perfmon_sandybridge_counters.h>
-#include <perfmon_ivybridge_counters.h>
-#include <perfmon_westmereEX_counters.h>
-#include <perfmon_silvermont_counters.h>
-
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
@@ -73,7 +60,13 @@ static int likwid_init = 0;
 static int numberOfGroups = 0;
 static int* groups;
 static int threads2Cpu[MAX_NUM_THREADS];
+static pthread_t threads2Pthread[MAX_NUM_THREADS];
+static int realThreads2Cpu[MAX_NUM_THREADS] = { [ 0 ... (MAX_NUM_THREADS-1)] = -1};
 static int num_cpus = 0;
+static int registered_cpus = 0;
+static pthread_mutex_t globalLock = PTHREAD_MUTEX_INITIALIZER;
+static int use_locks = 0;
+static pthread_mutex_t threadLocks[MAX_NUM_THREADS] = { [ 0 ... (MAX_NUM_THREADS-1)] = PTHREAD_MUTEX_INITIALIZER};
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 
@@ -81,33 +74,6 @@ static int num_cpus = 0;
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
-void str2BitMask(const char* str, BitMask* mask)
-{
-    char* endptr;
-    errno = 0;
-    struct bstrList* tokens;
-    bstring q = bfromcstralloc (60, str);
-    tokens = bsplit(q,' ');
-
-    for (int i=0; i<tokens->qty; i++)
-    {
-        uint64_t val =  strtoull((char*) tokens->entry[i]->data, &endptr, 16);
-        if ((errno == ERANGE && val == LONG_MAX ) || (errno != 0 && val == 0))
-        {
-          ERROR;
-        }
-
-        if (endptr == str)
-        {
-          ERROR_PLAIN_PRINT(No digits were found);
-        }
-
-        mask->mask[i] = val;
-    }
-
-    bstrListDestroy(tokens);
-    bdestroy(q);
-}
 
 static int getProcessorID(cpu_set_t* cpu_set)
 {
@@ -136,6 +102,32 @@ static int getThreadID(int cpu_id)
     return -1;
 }
 
+static double
+calculateMarkerResult(RegisterIndex index, uint64_t start, uint64_t stop, int overflows)
+{
+    double result = 0.0;
+
+    if (overflows == 0)
+    {
+        result = (double) (stop - start);
+    }
+    else if (overflows > 0)
+    {
+        result += (double) ((perfmon_getMaxCounterValue(counter_map[index].type) - start) + stop);
+        overflows--;
+    }
+    result += (double) (overflows * perfmon_getMaxCounterValue(counter_map[index].type));
+    if (counter_map[index].type == POWER)
+    {
+        result *= power_getEnergyUnit(getCounterTypeOffset(index));
+    }
+    else if (counter_map[index].type == THERMAL)
+    {
+        result = (double)stop;
+    }
+    return result;
+}
+
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
 void likwid_markerInit(void)
@@ -160,8 +152,7 @@ void likwid_markerInit(void)
     }
     else if (likwid_init == 0)
     {
-        fprintf(stderr, "Cannot initalize LIKWID marker API, environment variables are not set\n");
-        fprintf(stderr, "You have to set the -m commandline switch for likwid-perfctr\n");
+        fprintf(stderr, "Running without Marker API. Activate Marker API with -m on commandline.\n");
         return;
     }
     else
@@ -169,7 +160,6 @@ void likwid_markerInit(void)
         return;
     }
 
-    verbosity = atoi(getenv("LIKWID_DEBUG"));
     if (!lock_check())
     {
         fprintf(stderr,"Access to performance counters is locked.\n");
@@ -181,15 +171,17 @@ void likwid_markerInit(void)
     affinity_init();
     hashTable_init();
 
-    for(int i=0; i<MAX_NUM_THREADS; i++) thread_sockets[i] = -1;
     for(int i=0; i<MAX_NUM_NODES; i++) socket_lock[i] = LOCK_INIT;
 
-    accessClient_setaccessmode(atoi(modeStr));
-    perfmon_verbosity = verbosity;
+    HPMmode(atoi(modeStr));
 
+    if (getenv("LIKWID_DEBUG") != NULL)
+    {
+        perfmon_verbosity = atoi(getenv("LIKWID_DEBUG"));
+        verbosity = perfmon_verbosity;
+    }
 
     bThreadStr = bfromcstr(cThreadStr);
-    threadTokens = bstrListCreate();
     threadTokens = bsplit(bThreadStr,',');
     num_cpus = threadTokens->qty;
     for (i=0; i<num_cpus; i++)
@@ -198,6 +190,25 @@ void likwid_markerInit(void)
     }
     bdestroy(bThreadStr);
     bstrListDestroy(threadTokens);
+    
+    if (getenv("LIKWID_PIN") != NULL)
+    {
+        likwid_pinThread(threads2Cpu[0]);
+        if (getenv("OMP_NUM_THREADS") != NULL)
+        {
+            if (ownatoi(getenv("OMP_NUM_THREADS")) > num_cpus)
+            {
+                use_locks = 1;
+            }
+        }
+        if (getenv("CILK_NWORKERS") != NULL)
+        {
+            if (ownatoi(getenv("CILK_NWORKERS")) > num_cpus)
+            {
+                use_locks = 1;
+            }
+        }
+    }
 
     i = perfmon_init(num_cpus, threads2Cpu);
     if (i<0)
@@ -207,7 +218,6 @@ void likwid_markerInit(void)
     }
 
     bEventStr = bfromcstr(eventStr);
-    eventStrings = bstrListCreate();
     eventStrings = bsplit(bEventStr,'|');
     numberOfGroups = eventStrings->qty;
     groups = malloc(numberOfGroups * sizeof(int));
@@ -222,27 +232,43 @@ void likwid_markerInit(void)
         groups[i] = perfmon_addEventSet(bdata(eventStrings->entry[i]));
     }
     bstrListDestroy(eventStrings);
+    bdestroy(bEventStr);
+
+    for (i=0; i<num_cpus; i++)
+    {
+        hashTable_initThread(threads2Cpu[i]);
+        for(int j=0; j<groupSet->groups[groups[0]].numberOfEvents;j++)
+        {
+            groupSet->groups[groups[0]].events[j].threadCounter[i].init = TRUE;
+            groupSet->groups[groups[0]].state = STATE_START;
+        }
+    }
 
     groupSet->activeGroup = 0;
 }
 
 void likwid_markerThreadInit(void)
 {
+    int myID;
     if ( !likwid_init )
     {
         return;
     }
+    
+    pthread_mutex_lock(&globalLock);
+    myID = registered_cpus++;
+    pthread_mutex_unlock(&globalLock);
 
-    int cpu_id = likwid_getProcessorId();
-    int thread_id = getThreadID(cpu_id);
-
-    HPMaddThread(cpu_id);
-    initThreadArch(cpu_id);
-    hashTable_initThread(cpu_id);
-
-    for(int i=0; i<groupSet->groups[groupSet->activeGroup].numberOfEvents;i++)
+    if (getenv("LIKWID_PIN") != NULL)
     {
-        groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].init = TRUE;
+        cpu_set_t cpuset;
+        CPU_ZERO(&cpuset);
+        sched_getaffinity(gettid(), sizeof(cpu_set_t), &cpuset);
+        if ((CPU_COUNT(&cpuset) > 1) || (likwid_getProcessorId() != threads2Cpu[myID % num_cpus]))
+        {
+            likwid_pinThread(threads2Cpu[myID % num_cpus]);
+            DEBUG_PRINT(DEBUGLEV_DEVELOP, "Pin thread %lu to CPU %d\n", gettid(), threads2Cpu[myID % num_cpus]);
+        }
     }
 }
 
@@ -275,54 +301,66 @@ void likwid_markerClose(void)
 {
     FILE *file = NULL;
     LikwidResults* results = NULL;
-    int numberOfThreads;
-    int numberOfRegions;
+    int numberOfThreads = 0;
+    int numberOfRegions = 0;
+    char* markerfile = NULL;
+    int lineidx = 0;
+    char line[1024];
+
     if ( ! likwid_init )
     {
-        
         return;
     }
     hashTable_finalize(&numberOfThreads, &numberOfRegions, &results);
-    file = fopen(getenv("LIKWID_FILEPATH"),"w");
+    if ((numberOfThreads == 0)||(numberOfThreads == 0))
+    {
+        fprintf(stderr, "No threads or regions defined in hash table\n");
+        return;
+    }
+    markerfile = getenv("LIKWID_FILEPATH");
+    if (markerfile == NULL)
+    {
+        fprintf(stderr, "Is the application executed with LIKWID wrapper? No file path for the Marker API output defined.\n");
+        return;
+    }
+    file = fopen(markerfile,"w");
 
     if (file != NULL)
     {
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Creating Marker file %s with %d regions %d groups and %d threads, markerfile, numberOfRegions, numberOfGroups, numberOfThreads);
         fprintf(file,"%d %d %d\n",numberOfThreads, numberOfRegions, numberOfGroups);
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, %d %d %d, numberOfThreads, numberOfRegions, numberOfGroups);
         for (int i=0; i<numberOfRegions; i++)
         {
-            if (results[i].count[0] == 0)
-            {
-                continue;
-            }
             fprintf(file,"%d:%s\n",i,bdata(results[i].tag));
+            DEBUG_PRINT(DEBUGLEV_DEVELOP, %d:%s, i,bdata(results[i].tag));
         }
         for (int i=0; i<numberOfRegions; i++)
         {
-            if (results[i].count[0] == 0)
-            {
-                continue;
-            }
             for (int j=0; j<numberOfThreads; j++)
             {
                 fprintf(file,"%d ",i);
                 fprintf(file,"%d ",results[i].groupID);
-                fprintf(file,"%d ",j);
+                fprintf(file,"%d ",results[i].cpulist[j]);
                 fprintf(file,"%u ",results[i].count[j]);
                 fprintf(file,"%e ",results[i].time[j]);
                 fprintf(file,"%d ",groupSet->groups[results[i].groupID].numberOfEvents);
-
+                lineidx = sprintf(&(line[0]), "%d %d %d %u %e %d ", i, results[i].groupID,results[i].cpulist[j],results[i].count[j],results[i].time[j],groupSet->groups[results[i].groupID].numberOfEvents);
                 for (int k=0; k<groupSet->groups[results[i].groupID].numberOfEvents; k++)
                 {
                     fprintf(file,"%e ",results[i].counters[j][k]);
+                    lineidx += sprintf(&(line[lineidx]), "%e ", results[i].counters[j][k]);
                 }
                 fprintf(file,"\n");
+                DEBUG_PRINT(DEBUGLEV_DEVELOP, %s,line);
             }
         }
         fclose(file);
     }
     else
     {
-        fprintf(stderr, "Cannot open file %s\n", getenv("LIKWID_FILEPATH"));
+        fprintf(stderr, "Cannot open file %s\n", markerfile);
+        fprintf(stderr, "%s", strerror(errno));
     }
 
     for (int i=0;i<numberOfRegions; i++)
@@ -334,6 +372,7 @@ void likwid_markerClose(void)
         free(results[i].time);
         bdestroy(results[i].tag);
         free(results[i].count);
+        free(results[i].cpulist);
         free(results[i].counters);
     }
 
@@ -357,7 +396,7 @@ int likwid_markerRegisterRegion(const char* regionTag)
     char groupSuffix[10];
     sprintf(groupSuffix, "-%d", groupSet->activeGroup);
     bcatcstr(tag, groupSuffix);
-    int cpu_id = hashTable_get(tag, &results);;
+    int cpu_id = hashTable_get(tag, &results);
     bdestroy(tag);
     return 0;
 }
@@ -369,6 +408,11 @@ int likwid_markerStartRegion(const char* regionTag)
     {
         return -EFAULT;
     }
+    int myCPU = likwid_getProcessorId();
+    if (getThreadID(myCPU) < 0)
+    {
+        return -EFAULT;
+    }
 
     bstring tag = bfromcstralloc(100, regionTag);
     LikwidThreadResults* results;
@@ -379,15 +423,18 @@ int likwid_markerStartRegion(const char* regionTag)
     int cpu_id = hashTable_get(tag, &results);
     int thread_id = getThreadID(cpu_id);
     perfmon_readCountersCpu(cpu_id);
-
+    results->cpuID = cpu_id;
     for(int i=0;i<groupSet->groups[groupSet->activeGroup].numberOfEvents;i++)
     {
         DEBUG_PRINT(DEBUGLEV_DEVELOP, START [%s] READ EVENT [%d=%d] EVENT %d VALUE %llu , regionTag, thread_id, cpu_id, i,
                         LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData);
-        groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].startData =
-                groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData;
+        //groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].startData =
+        //        groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData;
+        
+        results->StartPMcounters[i] = groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData;
+        results->StartOverflows[i] = groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].overflows;
     }
-    results->groupID = groupSet->activeGroup;
+    
     bdestroy(tag);
     timer_start(&(results->startTime));
     return 0;
@@ -404,28 +451,54 @@ int likwid_markerStopRegion(const char* regionTag)
 
     TimerData timestamp;
     timer_stop(&timestamp);
+    double result = 0.0;
     int cpu_id;
+    int myCPU = likwid_getProcessorId();
+    if (getThreadID(myCPU) < 0)
+    {
+        return -EFAULT;
+    }
     int thread_id;
     bstring tag = bfromcstr(regionTag);
     char groupSuffix[100];
     LikwidThreadResults* results;
     sprintf(groupSuffix, "-%d", groupSet->activeGroup);
     bcatcstr(tag, groupSuffix);
+    if (use_locks == 1)
+    {
+        pthread_mutex_lock(&threadLocks[myCPU]);
+    }
     
     cpu_id = hashTable_get(tag, &results);
     thread_id = getThreadID(cpu_id);
+    results->groupID = groupSet->activeGroup;
     results->startTime.stop.int64 = timestamp.stop.int64;
     results->time += timer_print(&(results->startTime));
     results->count++;
     bdestroy(tag);
-    
+
     perfmon_readCountersCpu(cpu_id);
-    
+
     for(int i=0;i<groupSet->groups[groupSet->activeGroup].numberOfEvents;i++)
     {
         DEBUG_PRINT(DEBUGLEV_DEVELOP, STOP [%s] READ EVENT [%d=%d] EVENT %d VALUE %llu, regionTag, thread_id, cpu_id, i,
                         LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData);
-        results->PMcounters[i] += perfmon_getResult(groupSet->activeGroup, i, thread_id);
+        result = calculateMarkerResult(groupSet->groups[groupSet->activeGroup].events[i].index, results->StartPMcounters[i],
+                                        groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData,
+                                        groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].overflows - 
+                                        results->StartOverflows[i]);
+        if (counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != THERMAL)
+        {
+            results->PMcounters[i] += result;
+        }
+        else
+        {
+            results->PMcounters[i] = result;
+        }
+    }
+    if (use_locks == 1)
+    {
+        pthread_mutex_unlock(&threadLocks[myCPU]);
     }
     return 0;
 }
@@ -435,10 +508,14 @@ void likwid_markerGetRegion(const char* regionTag, int* nr_events, double* event
 {
     if (! likwid_init)
     {
+        *nr_events = 0;
+        *time = 0;
+        *count = 0;
         return;
     }
     int length = 0;
     int cpu_id;
+    int myCPU = likwid_getProcessorId();
     int thread_id;
     bstring tag = bfromcstr(regionTag);
     char groupSuffix[100];
@@ -447,7 +524,7 @@ void likwid_markerGetRegion(const char* regionTag, int* nr_events, double* event
     bcatcstr(tag, groupSuffix);
 
     cpu_id = hashTable_get(tag, &results);
-    thread_id = getThreadID(cpu_id);
+    thread_id = getThreadID(myCPU);
     *count = results->count;
     *time = results->time;
     length = MIN(groupSet->groups[groupSet->activeGroup].numberOfEvents, *nr_events);
@@ -456,17 +533,26 @@ void likwid_markerGetRegion(const char* regionTag, int* nr_events, double* event
         events[i] = results->PMcounters[i];
     }
     *nr_events = length;
+    bdestroy(tag);
     return;
 }
 
 
 int  likwid_getProcessorId()
 {
+    int i;
     cpu_set_t  cpu_set;
     CPU_ZERO(&cpu_set);
     sched_getaffinity(gettid(),sizeof(cpu_set_t), &cpu_set);
-
-    return getProcessorID(&cpu_set);
+    if (CPU_COUNT(&cpu_set) > 1)
+    {
+        return sched_getcpu();
+    }
+    else
+    {
+        return getProcessorID(&cpu_set);
+    }
+    return -1;
 }
 
 #ifdef HAS_SCHEDAFFINITY
@@ -483,7 +569,7 @@ int  likwid_pinThread(int processorId)
 
     if (ret != 0)
     {
-        ERROR;
+        ERROR_PRINT("ERROR: Pinning of thread to CPU %d failed\n", processorId);
         return FALSE;
     }
 
@@ -503,7 +589,7 @@ int  likwid_pinProcess(int processorId)
 
     if (ret < 0)
     {
-        ERROR;
+        ERROR_PRINT("ERROR: Pinning of process to CPU %d failed\n", processorId);
         return FALSE;
     }
 
