@@ -6,14 +6,14 @@
  *      Description:  Implementation of Linux NUMA interface. Selects between hwloc and
  *                    procfs/sysfs backends.
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -50,7 +50,6 @@
 
 #include <error.h>
 #include <bstrlib.h>
-//#include <strUtil.h>
 
 #include <numa.h>
 #include <numa_proc.h>
@@ -64,11 +63,12 @@
 
 /* #####   EXPORTED VARIABLES   ########################################### */
 NumaTopology numa_info = {0,NULL};
+static int numaInitialized = 0;
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
-/* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
+
 int str2int(const char* str)
 {
     char* endptr;
@@ -92,6 +92,8 @@ int str2int(const char* str)
     return (int) val;
 }
 
+/* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
+
 int
 empty_numa_init()
 {
@@ -134,24 +136,39 @@ const struct numa_functions numa_funcs = {
 int numa_init(void)
 {
     const struct numa_functions funcs = numa_funcs;
-
+    int ret = 0;
     if (init_config == 0)
     {
         init_configuration();
     }
+    if (numaInitialized == 1)
+    {
+        return 0;
+    }
 
-    if (access(config.topologyCfgFileName, R_OK) && numa_info.numberOfNodes <= 0)
+    if ((config.topologyCfgFileName != NULL) && (!access(config.topologyCfgFileName, R_OK)) && (numa_info.nodes != NULL))
+    {
+        /* If we read in the topology file, the NUMA related stuff is already initialized */
+        numaInitialized = 1;
+        return 0;
+    }
+    else
     {
         cpu_set_t cpuSet;
         CPU_ZERO(&cpuSet);
         sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);
         if (cpuid_topology.activeHWThreads < cpuid_topology.numHWThreads)
         {
-            return proc_numa_init();
+            ret = proc_numa_init();
         }
-        return funcs.numa_init();
+        else
+        {
+            ret = funcs.numa_init();
+        }
+        if (ret == 0)
+            numaInitialized = 1;
     }
-    return 0;
+    return ret;
 }
 
 void numa_setInterleaved(int* processorList, int numberOfProcessors)
@@ -175,6 +192,10 @@ void numa_finalize(void)
 void numa_finalize(void)
 {
     int i;
+    if (!numaInitialized)
+    {
+        return;
+    }
     for(i=0;i<numa_info.numberOfNodes;i++)
     {
         if (numa_info.nodes[i].processors)
@@ -185,11 +206,27 @@ void numa_finalize(void)
         {
             free(numa_info.nodes[i].distances);
         }
+        numa_info.nodes[i].id = 0;
+        numa_info.nodes[i].totalMemory = 0;
+        numa_info.nodes[i].freeMemory = 0;
+        numa_info.nodes[i].numberOfProcessors = 0;
+        numa_info.nodes[i].numberOfDistances = 0;
     }
     if (numa_info.nodes)
     {
         free(numa_info.nodes);
     }
+    numa_info.numberOfNodes = 0;
+    numaInitialized = 0;
     return;
 }
+
+int likwid_getNumberOfNodes()
+{
+    if (numaInitialized)
+    {
+        return numa_info.numberOfNodes;
+    }
+    return 0;
+}
 #endif
