@@ -6,13 +6,13 @@
  *      Description:  Overloaded library for pthread_create call. 
  *                    Implements pinning of threads together with likwid-pin.
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -60,7 +60,20 @@ static char * sosearchpaths[] = {
     NULL
 };
 
-int
+
+#ifdef COLOR
+#define color_print(format,...) do { \
+        color_on(BRIGHT, COLOR); \
+        printf(format, ##__VA_ARGS__); \
+        color_reset(); \
+    } while(0)
+#else
+#define color_print(format,...) do { \
+        printf(format, ##__VA_ARGS__); \
+    } while(0)
+#endif
+
+int __attribute__ ((visibility ("default") ))
 pthread_create(pthread_t* thread,
         const pthread_attr_t* attr,
         void* (*start_routine)(void *),
@@ -74,9 +87,11 @@ pthread_create(pthread_t* thread,
     static int npinned = 0;
     static int ncalled = 0;
     static int overflow = 0;
+    static int overflowed = 0;
     static int silent = 0;
     static int pin_ids[MAX_NUM_THREADS];
-    static uint64_t skipMask = 0;
+    static uint64_t skipMask = 0x0;
+    static int ncpus = 0;
 
 
     /* On first entry: Get Evironment Variable and initialize pin_ids */
@@ -86,19 +101,14 @@ pthread_create(pthread_t* thread,
         char *token, *saveptr;
         char *delimiter = ",";
         int i = 0;
-        int ncpus = 0;
+        cpu_set_t cpuset;
 
         str = getenv("LIKWID_SKIP");
         if (str != NULL)
         {
             skipMask = strtoul(str, &str, 16);
         }
-        else
-        {
-            printf("[pthread wrapper] ERROR: Environment Variabel LIKWID_SKIP not set!\n");
-        }
-
-        if ( skipMask == 0 )
+        else if ( skipMask == 0x0 )
         {
             dlerror();    /* Clear any existing error */
             dlsym(RTLD_DEFAULT,"__kmpc_begin");
@@ -108,20 +118,15 @@ pthread_create(pthread_t* thread,
             }
         }
 
+
         if (getenv("LIKWID_SILENT") != NULL)
         {
             silent = 1;
         }
-#ifdef COLOR
-        else
-        {
-            color_on(BRIGHT, COLOR);
-        }
-#endif
 
         if (!silent)
         {
-            printf("[pthread wrapper] ");
+            color_print("[pthread wrapper] \n");
         }
 
         str = getenv("LIKWID_PIN");
@@ -138,35 +143,32 @@ pthread_create(pthread_t* thread,
                     pin_ids[i++] = strtoul(token, &token, 10);
                 }
             }
-            ncpus--; /* last ID is the first (the process was pinned to) */
+            CPU_ZERO(&cpuset);
+            CPU_SET(pin_ids[ncpus-1], &cpuset);
+            ret = sched_setaffinity(getpid(), sizeof(cpu_set_t), &cpuset);
+            if (!silent)
+            {
+                color_print("[pthread wrapper] MAIN -> %d\n",pin_ids[ncpus-1]);
+            }
+            //ncpus--; /* last ID is the first (the process was pinned to) */
         }
         else
         {
-            printf("[pthread wrapper] ERROR: Environment Variabel LIKWID_PIN not set!\n");
+            color_print("[pthread wrapper] ERROR: Environment Variabel LIKWID_PIN not set!\n");
         }
 
         if (!silent)
         {
-            printf("[pthread wrapper] PIN_MASK: ");
+            color_print("[pthread wrapper] PIN_MASK: ");
 
-            for (int i=0;i<ncpus;i++)
+            for (int i=0;i<ncpus-1;i++)
             {
-                printf("%d->%d  ",i,pin_ids[i]);
+                color_print("%d->%d  ",i,pin_ids[i]);
             }
-            printf("\n");
-            printf("[pthread wrapper] SKIP MASK: 0x%llX\n",LLU_CAST skipMask);
+            color_print("\n[pthread wrapper] SKIP MASK: 0x%llX\n",LLU_CAST skipMask);
         }
 
-        overflow = ncpus;
-    }
-    else
-    {
-#ifdef COLOR
-        if (!silent)
-        {
-            color_on(BRIGHT, COLOR);
-        }
-#endif
+        overflow = ncpus-1;
     }
 
     /* Handle dll related stuff */
@@ -187,7 +189,7 @@ pthread_create(pthread_t* thread,
 
     if (!handle)
     {
-        printf("%s\n", dlerror());
+        color_print("%s\n", dlerror());
         return -1;
     }
 
@@ -196,7 +198,7 @@ pthread_create(pthread_t* thread,
 
     if ((error = dlerror()) != NULL)
     {
-        printf("%s\n", error);
+        color_print("%s\n", error);
         return -2;
     }
 
@@ -211,39 +213,39 @@ pthread_create(pthread_t* thread,
         {
             if (!silent)
             {
-                printf("\tthreadid %lu -> SKIP \n", *thread);
+                color_print("\tthreadid %lu -> SKIP \n", *thread);
             }
         }
         else
         {
             CPU_ZERO(&cpuset);
-            CPU_SET(pin_ids[npinned], &cpuset);
+            CPU_SET(pin_ids[npinned%ncpus], &cpuset);
             pthread_setaffinity_np(*thread, sizeof(cpu_set_t), &cpuset);
-
-            if (npinned == overflow)
+            if ((npinned == overflow) && (!overflowed))
             {
                 if (!silent)
                 {
-                    printf("Roundrobin placement triggered\n");
-                    printf("\tthreadid %lu -> core %d - OK", *thread, pin_ids[npinned]);
+                    color_print("Roundrobin placement triggered\n\tthreadid %lu -> core %d - OK", *thread, pin_ids[npinned%ncpus]);
                 }
-                npinned = 0;
+                overflowed = 1;
+                npinned = (npinned+1)%ncpus;
             }
             else
             {
                 if (!silent)
                 {
-                    printf("\tthreadid %lu -> core %d - OK", *thread, pin_ids[npinned]);
+                    color_print("\tthreadid %lu -> core %d - OK", *thread, pin_ids[npinned%ncpus]);
                 }
                 npinned++;
+                if ((npinned >= ncpus) && (overflowed))
+                {
+                    npinned = 0;
+                }
             }
 
             if (!silent)
             {
-#ifdef COLOR
-                color_reset();
-#endif
-                printf("\n");
+                color_print("\n");
             }
         }
     }
