@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the topology backends
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -44,7 +44,7 @@
 #include <configuration.h>
 
 
-static volatile int init = 0;
+static int topology_initialized = 0;
 CpuInfo cpuid_info;
 CpuTopology cpuid_topology;
 
@@ -60,6 +60,7 @@ static char* atom_32_str = "Intel Atom 32nm processor";
 static char* atom_22_str = "Intel Atom 22nm processor";
 static char* atom_silvermont_str = "Intel Atom (Silvermont) processor";
 static char* atom_airmont_str = "Intel Atom (Airmont) processor";
+static char* atom_goldmont_str = "Intel Atom (Goldmont) processor";
 static char* nehalem_bloom_str = "Intel Core Bloomfield processor";
 static char* nehalem_lynn_str = "Intel Core Lynnfield processor";
 static char* nehalem_west_str = "Intel Core Westmere processor";
@@ -72,10 +73,12 @@ static char* haswell_ep_str = "Intel Xeon Haswell EN/EP/EX processor";
 static char* broadwell_str = "Intel Core Broadwell processor";
 static char* broadwell_d_str = "Intel Xeon D Broadwell processor";
 static char* broadwell_ep_str = "Intel Xeon Broadwell EN/EP/EX processor";
+static char* skylake_str = "Intel Skylake processor";
 static char* nehalem_ex_str = "Intel Nehalem EX processor";
 static char* westmere_ex_str = "Intel Westmere EX processor";
 static char* xeon_mp_string = "Intel Xeon MP processor";
-static char* xeon_phi_string = "Intel Xeon Phi Coprocessor";
+static char* xeon_phi_string = "Intel Xeon Phi (Knights Corner) Coprocessor";
+static char* xeon_phi2_string = "Intel Xeon Phi (Knights Landing) Coprocessor";
 static char* barcelona_str = "AMD Barcelona processor";
 static char* shanghai_str = "AMD Shanghai processor";
 static char* istanbul_str = "AMD Istanbul processor";
@@ -97,6 +100,7 @@ static char* short_core2 = "core2";
 static char* short_atom = "atom";
 static char* short_pm = "pentiumm";
 static char* short_silvermont = "silvermont";
+static char* short_goldmont = "goldmont";
 static char* short_nehalem = "nehalem";
 static char* short_nehalemEX = "nehalemEX";
 static char* short_westmere = "westmere";
@@ -104,12 +108,15 @@ static char* short_westmereEX = "westmereEX";
 static char* short_haswell = "haswell";
 static char* short_haswell_ep = "haswellEP";
 static char* short_broadwell = "broadwell";
+static char* short_broadwell_d = "broadwellD";
 static char* short_broadwell_ep = "broadwellEP";
 static char* short_ivybridge = "ivybridge";
 static char* short_ivybridge_ep = "ivybridgeEP";
 static char* short_sandybridge = "sandybridge";
 static char* short_sandybridge_ep = "sandybridgeEP";
+static char* short_skylake = "skylake";
 static char* short_phi = "phi";
+static char* short_phi2 = "phi2";
 static char* short_k8 = "k8";
 static char* short_k10 = "k10";
 static char* short_k15 = "interlagos";
@@ -228,6 +235,12 @@ static int readTopologyFile(const char* filename)
             break;
         }
     }
+    if (numHWThreads < 0 || numCacheLevels < 0 || numberOfNodes < 0)
+    {
+        ERROR_PRINT(Cannot read topology information from file %s, filename);
+        fclose(fp);
+        return -1;
+    }
 
     tmpNumberOfProcessors = (int*) malloc(numberOfNodes *sizeof(int));
     fseek(fp, 0, SEEK_SET);
@@ -303,14 +316,14 @@ static int readTopologyFile(const char* filename)
                 {
                     cpuid_topology.threadPool[thread].apicId = tmp;
                 }
-                
+
             }
             else if (strcmp(field, "cacheLevels") == 0)
             {
                 int level;
                 char type[128];
                 sscanf(line, "%s %s %d %s", structure, field, &level, value);
-                
+
                 cpuid_topology.cacheLevels[level-1].level = level-1;
                 if (strcmp(value, "type") == 0)
                 {
@@ -318,19 +331,19 @@ static int readTopologyFile(const char* filename)
                     if (strcmp(type, "UNIFIEDCACHE") == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].type = UNIFIEDCACHE;
-                    } 
+                    }
                     else if (strcmp(type, "DATACACHE") == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].type = DATACACHE;
-                    } 
+                    }
                     else if (strcmp(type, "INSTRUCTIONCACHE") == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].type = INSTRUCTIONCACHE;
-                    } 
+                    }
                     else if (strcmp(type, "ITLB") == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].type = ITLB;
-                    } 
+                    }
                     else if (strcmp(type, "DTLB") == 0)
                     {
                         cpuid_topology.cacheLevels[level-1].type = DTLB;
@@ -368,7 +381,7 @@ static int readTopologyFile(const char* filename)
                         cpuid_topology.cacheLevels[level-1].inclusive = tmp;
                     }
                 }
-                
+
             }
         }
         else if (strcmp(structure, "cpuid_info") == 0)
@@ -377,7 +390,7 @@ static int readTopologyFile(const char* filename)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.family = tmp;
-                
+
             }
             else if (strcmp(field, "model") == 0)
             {
@@ -395,55 +408,55 @@ static int readTopologyFile(const char* filename)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.stepping = tmp;
-                
+
             }
             else if (strcmp(field, "clock") == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.clock = tmp;
-                
+
             }
             else if (strcmp(field, "turbo") == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.turbo = tmp;
-                
+
             }
             else if (strcmp(field, "isIntel") == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.isIntel = tmp;
-                
+
             }
             else if (strcmp(field, "featureFlags") == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.featureFlags = tmp;
-                
+
             }
             else if (strcmp(field, "perf_version") == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.perf_version = tmp;
-                
+
             }
             else if (strcmp(field, "perf_num_ctr") == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.perf_num_ctr = tmp;
-                
+
             }
             else if (strcmp(field, "perf_width_ctr") == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.perf_width_ctr = tmp;
-                
+
             }
             else if (strcmp(field, "perf_num_fixed_ctr") == 0)
             {
                 sscanf(line, "%s %s = %d", structure, field, &tmp);
                 cpuid_info.perf_num_fixed_ctr = tmp;
-                
+
             }
             else if (strcmp(field, "features") == 0)
             {
@@ -459,7 +472,7 @@ static int readTopologyFile(const char* filename)
             {
                 int id;
                 sscanf(line, "%s %s %d %s", structure, field, &id, value);
-                    
+
                 if (strcmp(value,"numberOfProcessors") == 0)
                 {
                     sscanf(line, "%s %s %d %s = %d", structure, field, &id, value, &tmp);
@@ -592,13 +605,24 @@ int topology_setName(void)
                     break;
                 case BROADWELL_D:
                     cpuid_info.name = broadwell_d_str;
-                    cpuid_info.short_name = short_broadwell;
+                    cpuid_info.short_name = short_broadwell_d;
                     break;
                 case BROADWELL_E:
                     cpuid_info.name = broadwell_ep_str;
                     cpuid_info.short_name = short_broadwell_ep;
                     break;
 
+                case SKYLAKE1:
+                case SKYLAKE2:
+                    cpuid_info.name = skylake_str;
+                    cpuid_info.short_name = short_skylake;
+                    break;
+
+                case XEON_PHI2:
+                    cpuid_info.name = xeon_phi2_string;
+                    cpuid_info.short_name = short_phi2;
+                    break;
+
                 case NEHALEM_EX:
                     cpuid_info.name = nehalem_ex_str;
                     cpuid_info.short_name = short_nehalemEX;
@@ -643,6 +667,10 @@ int topology_setName(void)
                     cpuid_info.name = atom_airmont_str;
                     cpuid_info.short_name = short_silvermont;
                     break;
+                case ATOM_SILVERMONT_GOLD:
+                    cpuid_info.name = atom_goldmont_str;
+                    cpuid_info.short_name = short_goldmont;
+                    break;
 
                 default:
                     cpuid_info.name = unknown_intel_str;
@@ -652,7 +680,7 @@ int topology_setName(void)
             break;
 
         case MIC_FAMILY:
-            switch ( cpuid_info.model ) 
+            switch ( cpuid_info.model )
             {
                 case XEON_PHI:
                     cpuid_info.name = xeon_phi_string;
@@ -747,7 +775,7 @@ int topology_setName(void)
             cpuid_info.name = kabini_str;
             cpuid_info.short_name = short_k16;
             break;
-            
+
         default:
             return EXIT_FAILURE;
             break;
@@ -761,11 +789,13 @@ const struct topology_functions topology_funcs = {
     .init_cpuFeatures = cpuid_init_cpuFeatures,
     .init_nodeTopology = cpuid_init_nodeTopology,
     .init_cacheTopology = cpuid_init_cacheTopology,
+    .close_topology = NULL,
 #else
     .init_cpuInfo = hwloc_init_cpuInfo,
     .init_nodeTopology = hwloc_init_nodeTopology,
     .init_cacheTopology = hwloc_init_cacheTopology,
     .init_cpuFeatures = proc_init_cpuFeatures,
+    .close_topology = hwloc_close,
 #endif
     .init_fileTopology = initTopologyFile,
 };
@@ -776,7 +806,7 @@ void topology_setupTree(void)
     uint32_t i;
     TreeNode* currentNode;
     HWThread* hwThreadPool = cpuid_topology.threadPool;
-    
+
     tree_init(&cpuid_topology.topologyTree, 0);
     for (i=0; i<  cpuid_topology.numHWThreads; i++)
     {
@@ -784,6 +814,7 @@ void topology_setupTree(void)
         if (!tree_nodeExists(cpuid_topology.topologyTree,
                     hwThreadPool[i].packageId))
         {
+            //printf("Insert Socket %d\n", hwThreadPool[i].packageId);
             tree_insertNode(cpuid_topology.topologyTree,
                     hwThreadPool[i].packageId);
         }
@@ -791,6 +822,7 @@ void topology_setupTree(void)
                 hwThreadPool[i].packageId);
         if (!tree_nodeExists(currentNode, hwThreadPool[i].coreId))
         {
+            //printf("Insert Core %d at Socket %d\n", hwThreadPool[i].coreId, hwThreadPool[i].packageId);
             tree_insertNode(currentNode, hwThreadPool[i].coreId);
         }
         currentNode = tree_getNode(currentNode, hwThreadPool[i].coreId);
@@ -799,6 +831,7 @@ void topology_setupTree(void)
             /*
                printf("WARNING: Thread already exists!\n");
                */
+            //printf("Insert HWThread %d from Core %d at Socket %d\n", hwThreadPool[i].apicId, hwThreadPool[i].coreId, hwThreadPool[i].packageId);
             tree_insertNode(currentNode, hwThreadPool[i].apicId);
             affinity_thread2tile_lookup[hwThreadPool[i].apicId] = hwThreadPool[i].coreId;
         }
@@ -814,19 +847,24 @@ void topology_setupTree(void)
 
 int topology_init(void)
 {
+    int ret = 0;
+    cpu_set_t cpuSet;
     struct topology_functions funcs = topology_funcs;
 
-    if (init)
+    if (topology_initialized)
     {
         return EXIT_SUCCESS;
     }
-    init = 1;
 
-    init_configuration();
-
-    if (access(config.topologyCfgFileName, R_OK))
+    if (init_configuration())
     {
-        cpu_set_t cpuSet;
+        ERROR_PLAIN_PRINT(Cannot initialize configuration module to check for topology file name);
+        return EXIT_FAILURE;
+    }
+
+    if ((config.topologyCfgFileName == NULL) || access(config.topologyCfgFileName, R_OK))
+    {
+standard_init:
         CPU_ZERO(&cpuSet);
         sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);
         if (cpu_count(&cpuSet) < sysconf(_SC_NPROCESSORS_CONF))
@@ -854,11 +892,12 @@ int topology_init(void)
     }
     else
     {
-        cpu_set_t cpuSet;
         CPU_ZERO(&cpuSet);
         sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);
         DEBUG_PRINT(DEBUGLEV_INFO, Reading topology information from %s, config.topologyCfgFileName);
-        readTopologyFile(config.topologyCfgFileName);
+        ret = readTopologyFile(config.topologyCfgFileName);
+        if (ret < 0)
+            goto standard_init;
         cpuid_topology.activeHWThreads = 0;
         for (int i=0;i<cpuid_topology.numHWThreads;i++)
         {
@@ -873,12 +912,18 @@ int topology_init(void)
     }
 
 
+    topology_initialized = 1;
     return EXIT_SUCCESS;
 }
 
 
 void topology_finalize(void)
 {
+    struct topology_functions funcs = topology_funcs;
+    if (!topology_initialized)
+    {
+        return;
+    }
     if (cpuid_info.features != NULL)
     {
         free(cpuid_info.features);
@@ -904,6 +949,33 @@ void topology_finalize(void)
         tree_destroy(cpuid_topology.topologyTree);
         cpuid_topology.topologyTree = NULL;
     }
+    if (topology_funcs.close_topology != NULL)
+    {
+        topology_funcs.close_topology();
+    }
+    cpuid_info.family = 0;
+    cpuid_info.model = 0;
+    cpuid_info.stepping = 0;
+    cpuid_info.clock = 0;
+    cpuid_info.turbo = 0;
+    cpuid_info.name = NULL;
+    cpuid_info.short_name = NULL;
+    cpuid_info.isIntel = 0;
+    cpuid_info.supportUncore = 0;
+    cpuid_info.featureFlags = 0;
+    cpuid_info.perf_version = 0;
+    cpuid_info.perf_num_ctr = 0;
+    cpuid_info.perf_width_ctr = 0;
+    cpuid_info.perf_num_fixed_ctr = 0;
+
+    cpuid_topology.numHWThreads = 0;
+    cpuid_topology.activeHWThreads = 0;
+    cpuid_topology.numSockets = 0;
+    cpuid_topology.numCoresPerSocket = 0;
+    cpuid_topology.numThreadsPerCore = 0;
+    cpuid_topology.numCacheLevels = 0;
+
+    topology_initialized = 0;
 }
 
 
@@ -933,8 +1005,11 @@ void print_supportedCPUs (void)
     printf("\t%s\n",atom_silvermont_str);
     printf("\t%s\n",atom_airmont_str);
     printf("\t%s\n",xeon_phi_string);
-    printf("\t%s\n\n",broadwell_str);
-
+    printf("\t%s\n",broadwell_str);
+    printf("\t%s\n",broadwell_d_str);
+    printf("\t%s\n",broadwell_ep_str);
+    printf("\t%s\n",skylake_str);
+    printf("\n");
     printf("Supported AMD processors:\n");
     printf("\t%s\n",opteron_sc_str);
     printf("\t%s\n",opteron_dc_e_str);
@@ -944,7 +1019,8 @@ void print_supportedCPUs (void)
     printf("\t%s\n",istanbul_str);
     printf("\t%s\n",magnycours_str);
     printf("\t%s\n",interlagos_str);
-    printf("\t%s\n\n",kabini_str);
+    printf("\t%s\n",kabini_str);
+    printf("\n");
 }
 
 
