@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the hwloc based topology backend
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Authors:  Thomas Roehl (tr), thomas.roehl@googlemail.com
  *
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -50,11 +50,12 @@ hwloc_topology_t hwloc_topology = NULL;
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 #ifdef LIKWID_USE_HWLOC
-int hwloc_record_objs_of_type_below_obj(hwloc_topology_t t, hwloc_obj_t obj, hwloc_obj_type_t type, int* index, uint32_t **list)
+int likwid_hwloc_record_objs_of_type_below_obj(hwloc_topology_t t, hwloc_obj_t obj, hwloc_obj_type_t type, int* index, uint32_t **list)
 {
     int i;
     int count = 0;
     hwloc_obj_t walker;
+    if (!obj) return 0;
     if (!obj->arity) return 0;
     for (i=0;i<obj->arity;i++)
     {
@@ -67,7 +68,7 @@ int hwloc_record_objs_of_type_below_obj(hwloc_topology_t t, hwloc_obj_t obj, hwl
             }
             count++;
         }
-        count += hwloc_record_objs_of_type_below_obj(t, walker, type, index, list);
+        count += likwid_hwloc_record_objs_of_type_below_obj(t, walker, type, index, list);
     }
     return count;
 }
@@ -76,30 +77,39 @@ void hwloc_init_cpuInfo(cpu_set_t cpuSet)
 {
     int i;
     hwloc_obj_t obj;
-
-    hwloc_topology_init(&hwloc_topology);
-    hwloc_topology_set_flags(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_IO );
-    hwloc_topology_load(hwloc_topology);
-    obj = hwloc_get_obj_by_type(hwloc_topology, HWLOC_OBJ_SOCKET, 0);
+    if (perfmon_verbosity <= 1)
+    {
+        setenv("HWLOC_HIDE_ERRORS", "1", 1);
+    }
+    likwid_hwloc_topology_init(&hwloc_topology);
+    likwid_hwloc_topology_set_flags(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_IO );
+    likwid_hwloc_topology_load(hwloc_topology);
+    obj = likwid_hwloc_get_obj_by_type(hwloc_topology, HWLOC_OBJ_SOCKET, 0);
 
     cpuid_info.model = 0;
     cpuid_info.family = 0;
     cpuid_info.isIntel = 0;
+    cpuid_info.stepping = 0;
     cpuid_info.osname = malloc(MAX_MODEL_STRING_LENGTH * sizeof(char));
+    cpuid_info.osname[0] = '\0';
+    if (!obj)
+    {
+        return;
+    }
 
     const char * info;
-    if (info = hwloc_obj_get_info_by_name(obj, "CPUModelNumber"))
+    if ((info = likwid_hwloc_obj_get_info_by_name(obj, "CPUModelNumber")))
         cpuid_info.model = atoi(info);
-    if (info = hwloc_obj_get_info_by_name(obj, "CPUFamilyNumber"))
+    if ((info = likwid_hwloc_obj_get_info_by_name(obj, "CPUFamilyNumber")))
        cpuid_info.family = atoi(info);
-    if (info = hwloc_obj_get_info_by_name(obj, "CPUVendor"))
+    if ((info = likwid_hwloc_obj_get_info_by_name(obj, "CPUVendor")))
         cpuid_info.isIntel = strcmp(info, "GenuineIntel") == 0;
-    if (info = hwloc_obj_get_info_by_name(obj, "CPUModel"))
+    if ((info = likwid_hwloc_obj_get_info_by_name(obj, "CPUModel")))
         strcpy(cpuid_info.osname, info);
-    if (info = hwloc_obj_get_info_by_name(obj, "CPUStepping"))
+    if ((info = likwid_hwloc_obj_get_info_by_name(obj, "CPUStepping")))
         cpuid_info.stepping = atoi(info);
 
-    cpuid_topology.numHWThreads = hwloc_get_nbobjs_by_type(hwloc_topology, HWLOC_OBJ_PU);
+    cpuid_topology.numHWThreads = likwid_hwloc_get_nbobjs_by_type(hwloc_topology, HWLOC_OBJ_PU);
     DEBUG_PRINT(DEBUGLEV_DEVELOP, HWLOC CpuInfo Family %d Model %d Stepping %d isIntel %d numHWThreads %d activeHWThreads %d,
                             cpuid_info.family,
                             cpuid_info.model,
@@ -137,19 +147,19 @@ void hwloc_init_nodeTopology(cpu_set_t cpuSet)
         hwThreadPool[i].inCpuSet = 0;
     }
 
-    maxNumLogicalProcs = hwloc_get_nbobjs_by_type(hwloc_topology, HWLOC_OBJ_PU);
-    maxNumCores = hwloc_get_nbobjs_by_type(hwloc_topology, HWLOC_OBJ_CORE);
-    if (hwloc_get_nbobjs_by_type(hwloc_topology, socket_type) == 0)
+    maxNumLogicalProcs = likwid_hwloc_get_nbobjs_by_type(hwloc_topology, HWLOC_OBJ_PU);
+    maxNumCores = likwid_hwloc_get_nbobjs_by_type(hwloc_topology, HWLOC_OBJ_CORE);
+    if (likwid_hwloc_get_nbobjs_by_type(hwloc_topology, socket_type) == 0)
     {
         socket_type = HWLOC_OBJ_NODE;
     }
     maxNumLogicalProcsPerCore = maxNumLogicalProcs/maxNumCores;
     for (uint32_t i=0; i< cpuid_topology.numHWThreads; i++)
     {
-        obj = hwloc_get_obj_by_type(hwloc_topology, HWLOC_OBJ_PU, i);
+        int skip = 0;
+        obj = likwid_hwloc_get_obj_by_type(hwloc_topology, HWLOC_OBJ_PU, i);
         if (!obj)
         {
-            printf("No obj for CPU %d\n", i);
             continue;
         }
         id = obj->os_index;
@@ -158,10 +168,31 @@ void hwloc_init_nodeTopology(cpu_set_t cpuSet)
         hwThreadPool[id].threadId = obj->sibling_rank;
         while (obj->type != HWLOC_OBJ_CORE) {
             obj = obj->parent;
+            if (!obj)
+            {
+                skip = 1;
+                break;
+            }
+        }
+        if (skip)
+        {
+            hwThreadPool[id].coreId = 0;
+            hwThreadPool[id].packageId = 0;
+            continue;
         }
         hwThreadPool[id].coreId = obj->os_index;
         while (obj->type != socket_type) {
             obj = obj->parent;
+            if (!obj)
+            {
+                skip = 1;
+                break;
+            }
+        }
+        if (skip)
+        {
+            hwThreadPool[id].packageId = 0;
+            continue;
         }
         hwThreadPool[id].packageId = obj->os_index;
         /*DEBUG_PRINT(DEBUGLEV_DEVELOP, HWLOC Thread Pool PU %d Thread %d Core %d Socket %d,
@@ -188,27 +219,29 @@ void hwloc_init_cacheTopology(void)
     hwloc_obj_t obj;
     int depth;
     int d;
+    const char* info;
 
     /* Sum up all depths with caches */
-    depth = hwloc_topology_get_depth(hwloc_topology);
+    depth = likwid_hwloc_topology_get_depth(hwloc_topology);
     for (d = 0; d < depth; d++)
     {
-        if (hwloc_get_depth_type(hwloc_topology, d) == HWLOC_OBJ_CACHE)
+        if (likwid_hwloc_get_depth_type(hwloc_topology, d) == HWLOC_OBJ_CACHE)
             maxNumLevels++;
     }
     cachePool = (CacheLevel*) malloc(maxNumLevels * sizeof(CacheLevel));
     /* Start at the bottom of the tree to get all cache levels in order */
-    depth = hwloc_topology_get_depth(hwloc_topology);
+    depth = likwid_hwloc_topology_get_depth(hwloc_topology);
     id = 0;
+    
     for(d=depth-1;d >= 0; d--)
     {
         /* We only need caches, so skip other levels */
-        if (hwloc_get_depth_type(hwloc_topology, d) != HWLOC_OBJ_CACHE)
+        if (likwid_hwloc_get_depth_type(hwloc_topology, d) != HWLOC_OBJ_CACHE)
         {
             continue;
         }
         /* Get the cache object */
-        obj = hwloc_get_obj_by_depth(hwloc_topology, d, 0);
+        obj = likwid_hwloc_get_obj_by_depth(hwloc_topology, d, 0);
         /* All caches have this attribute, so safe to access */
         switch (obj->attr->cache.type)
         {
@@ -236,15 +269,24 @@ void hwloc_init_cacheTopology(void)
             cachePool[id].sets = cachePool[id].size /
                 (cachePool[id].associativity * cachePool[id].lineSize);
         }
+
         /* Count all HWThreads below the current cache */
-        cachePool[id].threads = hwloc_record_objs_of_type_below_obj(
+        cachePool[id].threads = likwid_hwloc_record_objs_of_type_below_obj(
                         hwloc_topology, obj, HWLOC_OBJ_PU, NULL, NULL);
 
-        char* info;
-        if (info = hwloc_obj_get_info_by_name(obj, "inclusiveness"))
+        while (!(info = likwid_hwloc_obj_get_info_by_name(obj, "inclusiveness")) && obj->next_cousin)
+        {
+            obj = obj->next_cousin; // If some PU/core are not bindable because of cgroup, hwloc may not know the inclusiveness of some of their cache.
+        }
+        if(info)
+        {
             cachePool[id].inclusive = info[0]=='t';
+        }
         else
+        {
             ERROR_PLAIN_PRINT(Processor is not supported);
+            break;
+        }
         id++;
     }
 
@@ -253,6 +295,14 @@ void hwloc_init_cacheTopology(void)
     return;
 }
 
+void hwloc_close(void)
+{
+    if (hwloc_topology)
+    {
+        hwloc_topology_destroy(hwloc_topology);
+    }
+}
+
 #else
 
 void hwloc_init_cpuInfo(void)
