@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the procfs/sysfs based topology backend
  *
- *      Version:   4.0
- *      Released:  16.6.2015
+ *      Version:   4.1
+ *      Released:  19.5.2016
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -30,36 +30,28 @@
  */
 
 #include <topology_proc.h>
-
+#include <cpuid.h>
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
-/* this was taken from the linux kernel */
-#define CPUID                              \
-    __asm__ volatile ("cpuid"                             \
-            : "=a" (eax),     \
-            "=b" (ebx),     \
-            "=c" (ecx),     \
-            "=d" (edx)      \
-            : "0" (eax), "2" (ecx))
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 static int get_cpu_perf_data(void)
 {
     uint32_t eax = 0x0U, ebx = 0x0U, ecx = 0x0U, edx = 0x0U;
     int largest_function = 0;
     eax = 0x00;
-    CPUID;
+    CPUID(eax, ebx, ecx, edx);
     largest_function = eax;
     if (cpuid_info.family == P6_FAMILY && 0x0A <= largest_function)
     {
         eax = 0x0A;
-        CPUID;
+        CPUID(eax, ebx, ecx, edx);
         cpuid_info.perf_version   =  (eax&0xFFU);
         cpuid_info.perf_num_ctr   =   ((eax>>8)&0xFFU);
         cpuid_info.perf_width_ctr =  ((eax>>16)&0xFFU);
         cpuid_info.perf_num_fixed_ctr =  (edx&0xFU);
 
         eax = 0x06;
-        CPUID;
+        CPUID(eax, ebx, ecx, edx);
         if (eax & (1<<1))
         {
             cpuid_info.turbo = 1;
@@ -84,6 +76,7 @@ int get_listPosition(int ownid, bstring list)
             return i;
         }
     }
+    bstrListDestroy(tokens);
     return -1;
 }
 
@@ -119,8 +112,10 @@ int fillList(int* outList, int outOffset, bstring list)
                     current++;
                 }
             }
+            bstrListDestroy(range);
         }
     }
+    bstrListDestroy(tokens);
     return current;
 }
 
@@ -129,7 +124,7 @@ static int readCacheInclusiveIntel(int level)
     uint32_t eax = 0x0U, ebx = 0x0U, ecx = 0x0U, edx = 0x0U;
     eax = 0x04;
     ecx = level;
-    CPUID;
+    CPUID(eax, ebx, ecx, edx);
     return edx & 0x2;
 }
 
@@ -138,16 +133,16 @@ static int readCacheInclusiveAMD(int level)
     uint32_t eax = 0x0U, ebx = 0x0U, ecx = 0x0U, edx = 0x0U;
     eax = 0x8000001D;
     ecx = level;
-    CPUID;
+    CPUID(eax, ebx, ecx, edx);
     return (edx & (0x1<<1));
 }
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 void proc_init_cpuInfo(cpu_set_t cpuSet)
 {
-    int i;
+    int i = 0;
     int HWthreads = 0;
-    FILE *fp;
+    FILE *fp = NULL;
 
     int (*ownatoi)(const char*);
     char* (*ownstrcpy)(char*,const char*);
@@ -173,6 +168,8 @@ void proc_init_cpuInfo(cpu_set_t cpuSet)
     {
         bstring src = bread ((bNread) fread, fp);
         struct bstrList* tokens = bsplit(src,(char) '\n');
+        bdestroy(src);
+        fclose(fp);
         for (i=0;i<tokens->qty;i++)
         {
             if (binstr(tokens->entry[i],0,countString) != BSTR_ERR)
@@ -447,6 +444,7 @@ void proc_init_nodeTopology(cpu_set_t cpuSet)
                             hwThreadPool[i].threadId,
                             hwThreadPool[i].coreId,
                             hwThreadPool[i].packageId)
+        bdestroy(cpudir);
     }
     cpuid_topology.threadPool = hwThreadPool;
     return;
@@ -620,6 +618,7 @@ void proc_init_cacheTopology(void)
                 break;
         }
     }
+    bdestroy(cpudir);
     cpuid_topology.numCacheLevels = nrCaches;
     cpuid_topology.cacheLevels = cachePool;
     return;
