@@ -16,6 +16,8 @@ bench_marker = "./likwid-bench-marker"
 bench_papi = "./likwid-bench-papi"
 perfctr = "../../likwid-perfctr"
 topology = "../../likwid-topology"
+topology_name = re.compile("^CPU name:\s+(.*)")
+topology_stepping = re.compile("^CPU stepping:\s+(\d*)")
 topology_type = re.compile("^CPU type:\s+(.*)")
 topology_sockets = re.compile("^Sockets:\s+(\d+)")
 topology_corespersocket = re.compile("^Cores per socket:\s+(\d+)")
@@ -25,10 +27,18 @@ testfolder = "TESTS"
 resultfolder = "RESULTS"
 hostname = socket.gethostname()
 picture_base = ".."
+topology_outputfile = "topology.dat"
+nrThreads = 1
 
 gnu_colors = ["red","blue","green","black"]#,"brown", "gray","violet", "cyan", "magenta","orange","#4B0082","#800000","turquoise","#006400","yellow"]
 gnu_marks = [5,13,9,2]#,3,4,6,7,8,9,10,11,12,14,15]
 
+units = { "L2" : "MByte/s", "L3" : "MByte/s", "MEM" : "MByte/s", "HA" : "MByte/s",
+          "FLOPS_SP" : "MFLOP/s", "FLOPS_DP" : "MFLOP/s", "FLOPS_AVX" : "MFLOP/s",
+          "DATA": "Load/Store ratio", "BRANCH" : "Instructions per branch",
+          "CLOCK" : "Instructions", "UOPS" : "UOPs"}
+translate_group = {"CLOCK" : "INST_RETIRED_ANY", "UOPS" : "UOPS_RETIRED_ANY"}
+
 wiki = False
 papi = False
 only_wiki = False
@@ -44,7 +54,7 @@ corrected_set = {}
 marker_set = {}
 papi_set = {}
 
-if not os.path.exists(bench_plain) or not os.path.exists(bench_marker):
+if not os.path.exists(bench_marker):
     print "Please run make before using likwid-accuracy.py"
     sys.exit(1)
 if not os.path.exists(perfctr):
@@ -59,6 +69,7 @@ def usage():
     print "-s/--sets:\tSpecifiy testgroups (comma separated). Can also be set in SET.txt"
 #    print "--wiki:\t\tBesides testing write out results in Google code wiki syntax"
 #    print "--only_wiki:\tDo not run benchmarks, read results from file and write out results in Google code wiki syntax"
+    print "-c <nrThreads>:\tSet number of threads. The accuracy tool uses the E notation of likwid like E:N:<nrThreads>:1:2. Default is 1 thread."
     print "Picture options:"
     print "--pgf:\t\tCreate TeX document for each test with PGFPlot"
     print "--gnuplot:\tCreate GNUPlot script for each test"
@@ -66,30 +77,6 @@ def usage():
     print "--script:\tActivate recording of commands in a bash script"
     print "--scriptname:\tRecord commands to create pictures in file (default: %s)" % (os.path.join(os.path.join(resultfolder,hostname),scriptfilename))
 
-def get_system_info():
-    name = None
-    sockets = 0
-    corespersocket = 0
-    threadspercore = 0
-    
-    p = subprocess.Popen(topology, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
-    p.wait()
-    if p.returncode != 0:
-        name = "Unknown system"
-        return
-    for line in p.stdout.read().split("\n"):
-        if not line.strip() or line.startswith("*") or line.startswith("-"): continue
-        if line.startswith("CPU type"):
-            name = topology_type.match(line).group(1).strip()
-        if line.startswith("Sockets"):
-            sockets = int(topology_sockets.match(line).group(1))
-        if line.startswith("Cores per socket"):
-            corespersocket = int(topology_corespersocket.match(line).group(1))
-        if line.startswith("Threads per core"):
-            threadspercore = int(topology_threadspercore.match(line).group(1))
-        if name and sockets > 0 and corespersocket > 0 and threadspercore > 0:
-            break
-    return name, sockets, corespersocket, threadspercore
 
 def get_groups():
     groups = {}
@@ -100,7 +87,6 @@ def get_groups():
     for line in p.stdout.read().split("\n"):
         if line.startswith("-") or not line.strip(): continue
         if line.startswith("Available"): continue
-        print(re.split("\s*", line.strip()))
         linelist = re.split("\s+", line.strip())
         name = linelist[0]
         description = " ".join(linelist[1:])
@@ -115,52 +101,101 @@ def get_test_groups(groupdict):
         setfp = open("SET.txt",'r')
         setlist = setfp.read().strip().split("\n")
         setfp.close()
-    
+
     filelist = glob.glob(testfolder+"/*.txt")
     for name in setlist:
-        tests = []
-        file = os.path.join(testfolder, name) + ".txt"
-        if not os.path.exists(file): continue
-        fp = open(file,'r')
-        finput = fp.read().strip().split("\n")
-        fp.close()    
-        for line in finput:
-            if line.startswith("TEST"):
-                tests.append(line.split(" ")[1])
-        groups[name] = tests
-                
-            
-    return groups
-    
-def get_values_from_file(file, lineoffset, linecount):
-    results = []
-    fp = open(file,'r')
-    finput = fp.read().strip().split("\n")
-    fp.close()
-    try:
-        for line in finput[lineoffset:lineoffset+linecount]:
-            results.append(float(line.split(" ")[1]))
-    except:
-        print "Cannot read file %s from %d to %d" % (file, lineoffset,lineoffset+linecount, )
-        for line in finput[lineoffset:lineoffset+linecount]:
-            print line
-    return results
+        if name in get_groups():
+            tests = []
+            file = os.path.join(testfolder, name) + ".txt"
+            if not os.path.exists(file): continue
+            fp = open(file,'r')
+            finput = fp.read().strip().split("\n")
+            fp.close()
+            for line in finput:
+                if line.startswith("TEST"):
+                    tests.append(line.split(" ")[1])
+            groups[name] = tests
 
-def write_pgf(group, test, plain_file, correct_file, marker_file, papi_file=None, execute=False, script=None):
-    filename = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+".tex")
+
+    return groups
+
+def write_topology(path):
+    try:
+        f = open(os.path.join(path, topology_outputfile),"w")
+    except:
+        print "Cannot write topology file %s" % (os.path.join(path, topology_outputfile),)
+        return
+    p = subprocess.Popen(topology, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+    p.wait()
+    if p.returncode != 0:
+        return
+    f.write(p.stdout.read())
+    f.close()
+
+def approx(in1, in2):
+    if in1 > (0.95*in2) or in1 < (1.05*in2):
+        return 1
+    return 0
+
+def legend(file1, file2):
+    input1 = []
+    input2 = []
+    numbers1 = []
+    numbers2 = []
+    try:
+        f=open(file1,"r")
+        input1 = f.read().strip().split("\n")
+        f.close()
+    except:
+        print "Cannot open file "+file1
+    try:
+        f=open(file2,"r")
+        input2 = f.read().strip().split("\n")
+        f.close()
+    except:
+        print "Cannot open file "+file2
+    if len(input1) == 0 and len(input2) == 0:
+        return "no"
+    for line in input1:
+        numbers1.append(line.split(" ")[1])
+    for line in input2:
+        numbers2.append(line.split(" ")[1])
+    if float(numbers1[0]) > float(numbers1[-1]) and float(numbers2[0]) > float(numbers2[-1]):
+        return "no"
+    elif float(numbers1[0]) < float(numbers1[-1]) and float(numbers2[0]) < float(numbers2[-1]):
+        return "so"
+    elif approx(float(numbers1[0]), float(numbers1[-1])) and approx(float(numbers2[0]), float(numbers2[-1])):
+        return "so"
+    return "no"
+
+
+def write_pgf(group, test, plain_file, marker_file, scale=0.0,papi_file=None, execute=False, script=None):
+    printgrp = group
+    if translate_group.has_key(group):
+        printgrp = translate_group[group]
+    filename = os.path.join(os.path.join(resultfolder,hostname),printgrp+"_"+test+".tex")
+    sizelist = []
+    sizeindex = []
+    lentry = "north east"
+    if legend(plain_file, marker_file) == "so":
+        lentry = "south east"
+    for i,variant in enumerate(test_set[group][test]["variants"]):
+        sizelist.append(variant)
+        sizeindex.append(str((i+0.5)*test_set[group][test]["RUNS"]))
     fp = open(filename,'w')
     fp.write("\documentclass{article}\n")
     fp.write("\usepackage{pgfplots}\n")
     fp.write("\\begin{document}\n")
     fp.write("% cut from here\n")
     fp.write("\\begin{tikzpicture}\n")
-    fp.write("\\begin{axis}[xlabel={Run}, ylabel={MFlops/s / MBytes/s},title={%s\_%s},legend pos=south east,xtick=data,width=.75\\textwidth]\n" % (group.replace("_","\_"),test.replace("_","\_"),))
+    fp.write("\\begin{axis}[xmin=0,xmax=%d,xlabel={Size - %d runs each}, ylabel={%s},title={Group %s - Test %s},legend pos=%s,xtick=data,width=.75\\textwidth,xticklabels={%s},xtick={%s}]\n" % (test_set[group][test]["RUNS"]*len(test_set[group][test]["variants"]),test_set[group][test]["RUNS"],units[group],printgrp.replace("_","\_"),test.replace("_","\_"),lentry,",".join(sizelist),",".join(sizeindex)))
     fp.write("\\addplot+[red,mark=square*,mark options={draw=red, fill=red}] table {%s};\n" % (os.path.basename(plain_file),))
-    fp.write("\\addlegendentry{plain};\n")
-    fp.write("\\addplot+[blue,mark=*,mark options={draw=blue, fill=blue}] table {%s};\n" % (os.path.basename(correct_file),))
-    fp.write("\\addlegendentry{corrected};\n")
+    fp.write("\\addlegendentry{bench};\n")
+    if scale > 0.0:
+        fp.write("\\addplot+[blue,mark=*,mark options={draw=blue, fill=blue}] table[x index=0, y expr=\\thisrowno{1}*%f] {%s};\n" % (scale, os.path.basename(plain_file),))
+        fp.write("\\addlegendentry{scaled bench};\n")
     fp.write("\\addplot+[green,mark=diamond*,mark options={draw=green, fill=green}] table {%s};\n" % (os.path.basename(marker_file),))
-    fp.write("\\addlegendentry{marker};\n")
+    fp.write("\\addlegendentry{perfctr};\n")
     if papi and papi_file:
         fp.write("\\addplot+[black,mark=triangle*,mark options={draw=black, fill=black}] table {%s};\n" % (os.path.basename(papi_file),))
         fp.write("\\addlegendentry{papi};\n")
@@ -179,22 +214,34 @@ def write_pgf(group, test, plain_file, correct_file, marker_file, papi_file=None
     if script:
         script.write("pdflatex %s\n" % (os.path.basename(filename),))
     return filename
-    
-def write_gnuplot(group, test, plain_file, correct_file, marker_file, papi_file=None, execute=False, script=None):
-    filename = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+".plot")
+
+def write_gnuplot(group, test, plain_file, marker_file, scale = 1.0, papi_file=None, execute=False, script=None):
+    printgrp = group
+    if translate_group.has_key(group):
+        printgrp = translate_group[group]
+    filename = os.path.join(os.path.join(resultfolder,hostname),printgrp+"_"+test+".plot")
     fp = open(filename,'w')
     for i,color in enumerate(gnu_colors):
         fp.write("set style line %d linetype 1 linecolor rgb '%s' lw 2 pt %s\n" % (i+1, color,gnu_marks[i]))
     fp.write("set terminal jpeg\n")
-    fp.write("set title '%s_%s'\n" % (group, test,))
-    fp.write("set output '%s'\n" % (os.path.basename(os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+".jpg")),))
+    fp.write("set encoding utf8\n")
+    fp.write("set title 'Group %s - Test %s'\n" % (printgrp, test,))
+    if legend(plain_file, marker_file) == "no":
+        fp.write("set key top right\n")
+    else:
+        fp.write("set key bottom right\n")
+    fp.write("set output '%s'\n" % (os.path.basename(os.path.join(os.path.join(resultfolder,hostname),printgrp+"_"+test+".jpg")),))
     fp.write("set xlabel 'Size - %d runs each'\n" % (test_set[group][test]["RUNS"],))
-    fp.write("set ylabel 'MFlops/s / MBytes/s'\n")
+    fp.write("set ylabel '%s'\n" % (units[group],))
+    fp.write("set yrange  [0:]\n")
     #fp.write("set xtics 0,%d,%d\n" % (test_set[group][test]["RUNS"], test_set[group][test]["RUNS"]*len(test_set[group][test]["variants"]),))
     fp.write("set xtics %d\n" % (test_set[group][test]["RUNS"]*len(test_set[group][test]["variants"]),))
     for i,variant in enumerate(test_set[group][test]["variants"]):
         fp.write("set xtics add (\"%s\" %f)\n" % (variant, (i*test_set[group][test]["RUNS"])+(0.5*test_set[group][test]["RUNS"]),))
-    plot_string = "plot '%s' using 1:2 title 'plain' with linespoints ls 1, \\\n '%s' using 1:2 title 'corrected' with linespoints ls 2, \\\n '%s' using 1:2 title 'marker' with linespoints ls 3" % (os.path.basename(plain_file), os.path.basename(correct_file), os.path.basename(marker_file),)
+    plot_string = "plot '%s' using 1:2 title 'bench' with linespoints ls 1, \\\n"  % (os.path.basename(plain_file),)
+    if scale > 0.0:
+        plot_string = plot_string+" '%s' using 1:($2*%f) title 'scaled bench' with linespoints ls 2, \\\n" % (os.path.basename(plain_file), scale,)
+    plot_string = plot_string+" '%s' using 1:2 title 'perfctr' with linespoints ls 3" % (os.path.basename(marker_file),)
     if papi and papi_file:
         plot_string += ", \\\n '%s' using 1:2 title 'papi' with linespoints ls 4\n" % (os.path.basename(papi_file),)
     fp.write(plot_string+"\n")
@@ -212,9 +259,12 @@ def write_gnuplot(group, test, plain_file, correct_file, marker_file, papi_file=
     return filename
 
 def write_grace(group, test, plain_file, correct_file, marker_file, papi_file=None, execute=False, script=None):
-    filename = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+".bat")
-    agrname = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+".agr")
-    pngname = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+".png")
+    printgrp = group
+    if translate_group.has_key(group):
+        printgrp = translate_group[group]
+    filename = os.path.join(os.path.join(resultfolder,hostname),printgrp+"_"+test+".bat")
+    agrname = os.path.join(os.path.join(resultfolder,hostname),printgrp+"_"+test+".agr")
+    pngname = os.path.join(os.path.join(resultfolder,hostname),printgrp+"_"+test+".png")
     if execute or script:
         plain_file = os.path.basename(plain_file)
         marker_file = os.path.basename(marker_file)
@@ -228,15 +278,18 @@ def write_grace(group, test, plain_file, correct_file, marker_file, papi_file=No
     out_options = "-hdevice PNG -printfile %s " % (pngname,)
     out_options += "-saveall %s" % (agrname,)
     fp = open(filename,'w')
-    fp.write("title \"%s_%s\"\n" % (group, test,))
+    fp.write("title \"Group %s - Test %s\"\n" % (printgrp, test,))
     fp.write("xaxis label \"Run\"\n")
     fp.write("xaxis label char size 1.2\n")
-    fp.write("xaxis ticklabel char size 1.2\n")
-    fp.write("yaxis label \"MFlops/s / MBytes/s\"\n")
+    fp.write("xaxis ticklabel char size 1.2\n" % (units[group],))
+    fp.write("yaxis label \"%s\"\n")
     fp.write("yaxis label char size 1.2\n")
     fp.write("yaxis ticklabel char size 1.2\n")
-    fp.write("legend 0.8,0.7\n")
-    fp.write("s0 legend \"plain\"\n")
+    if legend(plain_file, marker_file) == "no":
+        fp.write("legend 0.8,0.7\n")
+    else:
+        fp.write("legend 0.2,0.7\n")
+    fp.write("s0 legend \"bench\"\n")
     fp.write("s0 symbol 2\n")
     fp.write("s0 symbol size 1\n")
     fp.write("s0 symbol color 2\n")
@@ -250,7 +303,7 @@ def write_grace(group, test, plain_file, correct_file, marker_file, papi_file=No
     fp.write("s0 line linestyle 1\n")
     fp.write("s0 line linewidth 2\n")
     fp.write("s0 line pattern 1\n")
-    fp.write("s1 legend \"corrected\"\n")
+    fp.write("s1 legend \"scaled bench\"\n")
     fp.write("s1 symbol 3\n")
     fp.write("s1 symbol size 1\n")
     fp.write("s1 symbol color 4\n")
@@ -264,7 +317,7 @@ def write_grace(group, test, plain_file, correct_file, marker_file, papi_file=No
     fp.write("s1 line linestyle 1\n")
     fp.write("s1 line linewidth 2\n")
     fp.write("s1 line pattern 1\n")
-    fp.write("s2 legend \"marker\"\n")
+    fp.write("s2 legend \"perfctr\"\n")
     fp.write("s2 symbol 4\n")
     fp.write("s2 symbol size 1\n")
     fp.write("s2 symbol color 3\n")
@@ -305,8 +358,10 @@ def write_grace(group, test, plain_file, correct_file, marker_file, papi_file=No
         script.write("gracebat %s -param %s %s\n" % (cmd_options, os.path.basename(filename),out_options,))
     return filename
 
+
+
 try:
-    opts, args = getopt.getopt(sys.argv[1:], "hs:", ["help", "sets=","script","scriptname=","wiki","only_wiki","pgf","gnuplot","grace","papi"])
+    opts, args = getopt.getopt(sys.argv[1:], "hs:c:", ["help", "sets=","script","scriptname=","wiki","only_wiki=","pgf","gnuplot","grace","papi"])
 except getopt.GetoptError as err:
     print str(err)
     usage()
@@ -324,8 +379,15 @@ for o, a in opts:
         wiki = True
     if o == "--only_wiki":
         only_wiki = True
+        hostname = a
     if o == "--papi":
         papi = True
+    if o == "-c":
+        try:
+            nrThreads = int(a)
+        except:
+            print "Argument to -c not valid. Must be a number"
+            sys.exit(1)
     if o == "--pgf":
         out_pgf = True
     if o == "--gnuplot":
@@ -356,45 +418,55 @@ if len(sets) == 0:
     fp.close()
 for line in sets:
     if not line.strip() or line.startswith("#"): continue
-    if os.path.exists("%s/%s.txt" % (testfolder,line.strip(),)):
-        test_set[line.strip()] = {}
-        plain_set[line.strip()] = {}
-        corrected_set[line.strip()] = {}
-        marker_set[line.strip()] = {}
-        papi_set[line.strip()] = {}
-        testfp = open("%s/%s.txt" % (testfolder,line.strip(),),'r')
+    filename = "%s/%s.txt" % (testfolder,line.strip(),)
+    if os.path.exists(filename):
+        groupname = line.strip()
+        testfp = open(filename,'r')
+        for line in testfp.read().split("\n"):
+            if line.startswith("GROUP"):
+                match = re.match("^GROUP\s+(\.+)")
+                if match:
+                    groupname = match.group(1)
+                    break
+        testfp.close()
+        test_set[groupname] = {}
+        plain_set[groupname] = {}
+        corrected_set[groupname] = {}
+        marker_set[groupname] = {}
+        papi_set[groupname] = {}
+        testfp = open(filename,'r')
         test = None
         for i,testline in enumerate(testfp.read().split("\n")):
             if test and not testline.strip(): test = None
             if testline.startswith("REGEX_BENCH"):
-                test_set[line.strip()]["REGEX_BENCH"] = re.compile(" ".join(testline.split(" ")[1:]))
+                test_set[groupname]["REGEX_BENCH"] = re.compile(" ".join(testline.split(" ")[1:]))
             if testline.startswith("REGEX_PERF"):
-                test_set[line.strip()]["REGEX_PERF"] = re.compile(" ".join(testline.split(" ")[1:]))
+                test_set[groupname]["REGEX_PERF"] = re.compile(" ".join(testline.split(" ")[1:]))
             if testline.startswith("REGEX_PAPI"):
-                test_set[line.strip()]["REGEX_PAPI"] = re.compile(" ".join(testline.split(" ")[1:]))
+                test_set[groupname]["REGEX_PAPI"] = re.compile(" ".join(testline.split(" ")[1:]))
             if testline.startswith("TEST"):
                 test = testline.split(" ")[1]
-                test_set[line.strip()][test] = {}
-                test_set[line.strip()][test]["WA_FACTOR"] = 1.0
-                plain_set[line.strip()][test] = {}
-                corrected_set[line.strip()][test] = {}
-                marker_set[line.strip()][test] = {}
-                papi_set[line.strip()][test] = {}
+                test_set[groupname][test] = {}
+                test_set[groupname][test]["WA_FACTOR"] = 0.0
+                plain_set[groupname][test] = {}
+                corrected_set[groupname][test] = {}
+                marker_set[groupname][test] = {}
+                papi_set[groupname][test] = {}
             if testline.startswith("RUNS") and test:
-                test_set[line.strip()][test]["RUNS"] = int(testline.split(" ")[1])
+                test_set[groupname][test]["RUNS"] = int(testline.split(" ")[1])
             if testline.startswith("WA_FACTOR") and test:
-                test_set[line.strip()][test]["WA_FACTOR"] = float(testline.split(" ")[1])
+                test_set[groupname][test]["WA_FACTOR"] = float(testline.split(" ")[1])
             if testline.startswith("VARIANT") and test:
                 linelist = re.split("\s+",testline);
                 variant = linelist[1]
-                if not test_set[line.strip()][test].has_key("variants"):
-                    test_set[line.strip()][test]["variants"] = []
-                test_set[line.strip()][test][variant] = linelist[2]
-                test_set[line.strip()][test]["variants"].append(linelist[1])
-                plain_set[line.strip()][test][variant] = []
-                corrected_set[line.strip()][test][variant] = []
-                marker_set[line.strip()][test][variant] = []
-                papi_set[line.strip()][test][variant] = []
+                if not test_set[groupname][test].has_key("variants"):
+                    test_set[groupname][test]["variants"] = []
+                test_set[groupname][test][variant] = linelist[2]
+                test_set[groupname][test]["variants"].append(linelist[1])
+                plain_set[groupname][test][variant] = []
+                corrected_set[groupname][test][variant] = []
+                marker_set[groupname][test][variant] = []
+                papi_set[groupname][test][variant] = []
         testfp.close()
 
 
@@ -407,14 +479,15 @@ if not os.path.exists(resultfolder):
     os.mkdir(resultfolder)
 if not os.path.exists(os.path.join(resultfolder,hostname)):
     os.mkdir(os.path.join(resultfolder,hostname))
-
+write_topology(os.path.join(resultfolder,hostname))
 if not only_wiki:
     scriptfile = os.path.join(os.path.join(resultfolder,hostname),scriptfilename)
     script = open(scriptfile,'w')
     script.write("#!/bin/bash\n")
 
     for group in test_set.keys():
-        perfctr_string = "%s -c S0:0 -g %s -m " % (perfctr,group,)
+        perfctr_string = "%s -C E:N:%d:1:2 -g %s -m " % (perfctr,nrThreads, group,)
+        no_scale = False
         for test in test_set[group].keys():
             if test.startswith("REGEX"): continue
             file_plain = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_plain.dat")
@@ -423,7 +496,6 @@ if not only_wiki:
             file_marker = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_marker.dat")
             raw_marker = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_marker.raw")
             outfp_plain = open(file_plain,'w')
-            rawfp_plain = open(raw_plain,'w')
             outfp_correct = open(file_correct,'w')
             outfp_marker = open(file_marker,'w')
             rawfp_marker = open(raw_marker,'w')
@@ -436,49 +508,17 @@ if not only_wiki:
                 file_papi = None
                 raw_papi = None
             counter = 1
+            print "Group %s Test %s" % (group, test,)
             for size in test_set[group][test]["variants"]:
                 if size.startswith("RUNS"): continue
-                bench_options = "-t %s -w S0:%s:1" % (test, size,)
+                print "Size "+size+": ",
+                bench_options = "-t %s -w N:%s:%d" % (test, size, nrThreads)
                 for i in range(0,test_set[group][test]["RUNS"]):
-                    # Run with plain likwid-bench
                     print "*",
                     sys.stdout.flush()
-                    p = subprocess.Popen(bench_plain+" "+bench_options, shell=True, stdout=subprocess.PIPE,stderr=subprocess.STDOUT)
-                    try:
-                        p.wait()
-                        stdout = p.stdout.read()
-                        p.stdout.close()
-                    except:
-                        sys.exit(1)
-                    for line in stdout.split("\n"):
-                        #if p.returncode != 0: print line
-                        match = test_set[group]["REGEX_BENCH"].match(line)
-                        if match:
-                            value = float(match.group(1)) * test_set[group][test]["WA_FACTOR"]
-                            plain_set[group][test][size].append(match.group(1))
-                            corrected_set[group][test][size].append(str(value))
-                            outfp_plain.write(str(counter)+" "+match.group(1)+"\n")
-                            outfp_correct.write(str(counter)+" "+str(value)+"\n")
-                        rawfp_plain.write(line+"\n")
-                    # Run with papi instrumented likwid-bench
-                    if papi:
-                        os.environ["PAPI_BENCH"] = str(group)
-                        p = subprocess.Popen(bench_papi+" "+bench_options, shell=True, stdout=subprocess.PIPE,stderr=subprocess.STDOUT)
-                        try:
-                            p.wait()
-                            stdout = p.stdout.read()
-                            p.stdout.close()
-                        except:
-                            sys.exit(1)
-                        for line in stdout.split("\n"):
-                            #if p.returncode != 0: print line
-                            match = test_set[group]["REGEX_PAPI"].match(line)
-                            if match:
-                                papi_set[group][test][size].append(match.group(1))
-                                outfp_papi.write(str(counter)+" "+match.group(1)+"\n")
-                            rawfp_papi.write(line+"\n")
                     # Run with LIKWID instrumented likwid-bench and likwid-perfctr
-                    p = subprocess.Popen(perfctr_string+" "+bench_marker+" "+bench_options, shell=True, stdout=subprocess.PIPE,stderr=subprocess.STDOUT)
+                    rawfp_marker.write(perfctr_string+" "+bench_marker+" "+bench_options+"\n")
+                    p = subprocess.Popen(perfctr_string+" "+bench_marker+" "+bench_options, shell=True, stdout=subprocess.PIPE,stderr=subprocess.STDOUT,executable="/bin/bash")
                     stdout = ""
                     try:
                         p.wait()
@@ -486,108 +526,50 @@ if not only_wiki:
                         p.stdout.close()
                     except:
                         sys.exit(1)
+                    found_bench = False
+                    found_perfctr = False
                     for line in stdout.split("\n"):
-                        #if p.returncode != 0: print line
-                        match = test_set[group]["REGEX_PERF"].match(line)
-                        if match:
-                            marker_set[group][test][size].append(float(match.group(1)))
-                            outfp_marker.write(str(counter)+" "+str(float(match.group(1)))+"\n")
+                        if p.returncode == 0:
+                            match = test_set[group]["REGEX_PERF"].match(line)
+                            if match:
+                                marker_set[group][test][size].append(float(match.group(1)))
+                                outfp_marker.write(str(counter)+" "+str(float(match.group(1)))+"\n")
+                                found_perfctr = True
+                            match = test_set[group]["REGEX_BENCH"].match(line)
+                            if match:
+                                found_bench = True
+                                value = float(match.group(1)) * test_set[group][test]["WA_FACTOR"]
+                                plain_set[group][test][size].append(match.group(1))
+                                corrected_set[group][test][size].append(str(value))
+                                outfp_plain.write(str(counter)+" "+match.group(1)+"\n")
+                                outfp_correct.write(str(counter)+" "+str(value)+"\n")
                         rawfp_marker.write(line+"\n")
+                    if not found_bench:
+                        value = str(test_set[group][test]["WA_FACTOR"])
+                        plain_set[group][test][size].append(value)
+                        corrected_set[group][test][size].append(value)
+                        outfp_plain.write(str(counter)+" "+value+"\n")
+                        outfp_correct.write(str(counter)+" "+value+"\n")
+                        no_scale = True
+                    if not found_perfctr:
+                        marker_set[group][test][size].append(0)
+                        outfp_marker.write(str(counter)+" "+str(0)+"\n")
                     counter += 1
                 print("")
             outfp_plain.close()
-            rawfp_plain.close()
             outfp_correct.close()
             outfp_marker.close()
             rawfp_marker.close()
             if papi:
                 outfp_papi.close()
                 rawfp_papi.close()
-            if out_pgf: pgf_file = write_pgf(group, test, file_plain, file_correct, file_marker, file_papi, script=script)
-            if out_gnuplot: plot_file = write_gnuplot(group, test, file_plain, file_correct, file_marker, file_papi, script=script)
+            if no_scale:
+                test_set[group][test]["WA_FACTOR"] = 0.0
+            if out_pgf:
+                pgf_file = write_pgf(group, test, file_plain, file_marker, test_set[group][test]["WA_FACTOR"],file_papi, script=script)
+            if out_gnuplot: plot_file = write_gnuplot(group, test, file_plain,file_marker, test_set[group][test]["WA_FACTOR"], file_papi, script=script)
             if out_grace: grace_file = write_grace(group, test, file_plain, file_correct, file_marker, file_papi, script=script)
 
 
     script.close()
     os.chmod(scriptfile, stat.S_IRWXU)
-if only_wiki:
-    for group in test_set.keys():
-        for test in test_set[group].keys():
-            if test.startswith("REGEX"): continue
-            filename = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_plain.dat")
-            for i,size in enumerate(test_set[group][test]["variants"]):
-                start = i*test_set[group][test]["RUNS"]
-                end = (i+1)*test_set[group][test]["RUNS"]
-                runs = test_set[group][test]["RUNS"]
-                print "Read file %s for size %s from %d to %d" % (filename,size, start, end,)
-                plain_set[group][test][size] = get_values_from_file(filename, start, runs)
-                if len(plain_set[group][test][size]) == 0: plain_set[group][test][size].append(0)
-            filename = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_marker.dat")
-            for i,size in enumerate(test_set[group][test]["variants"]):
-                start = i*test_set[group][test]["RUNS"]
-                end = (i+1)*test_set[group][test]["RUNS"]
-                runs = test_set[group][test]["RUNS"]
-                print "Read file %s for size %s from %d to %d" % (filename,size, start, end,)
-                marker_set[group][test][size] = get_values_from_file(filename, start, runs)
-                if len(marker_set[group][test][size]) == 0: marker_set[group][test][size].append(0)
-
-
-if wiki or only_wiki:
-    name, sockets, corespersocket, threadspercore = get_system_info();
-    groups = get_groups()
-    testable_groups = get_test_groups(groups)
-    if testable_groups.has_key("FLOPS_DP"): del testable_groups["FLOPS_DP"]
-
-    print "# Accuracy Tests for %s\n" % (name,)
-    print "## Hardware description"
-    print "Sockets: %d<br>" % (sockets,)
-    print "Cores per socket: %d<br>" % (corespersocket,)
-    print "Threads per core: %d<br>" % (threadspercore,)
-    print "Total number of processing units: %d<br>" % (sockets * corespersocket * threadspercore)
-    print
-    print "## Available groups"
-    print "Each architecture defines a different set of performance groups. These groups help users to measure their derived metrics. Besides the event and counter defintion, a performance groups contains derived metrics that are calculated based on the measured data.<br>Here all the groups available for the %s are listed:<br>\n" % (name,)
-    print "| Name | Description |"
-    print "| ---- | ----------- |"
-    for grp in groups.keys():
-        print "| %s | %s |" % (grp, groups[grp],)
-    print
-    print "## Available verification tests"
-    print "Not all groups can be tested for accuracy. We don't have a test application for each performance group. Here only the groups are listed that can be verified. Each group is followed by the low-level benchmarks that are performed for comparison.<br>\n"
-    #print testable_groups
-    print "| Group | Tests |"
-    print "|-------|-------|"
-    for grp in testable_groups.keys():
-        print "| %s | %s |" % (grp, ", ".join (testable_groups[grp]))
-    print
-    print "## Accuracy comparison"
-    print "For each varification group, the tests are performed twice. Once in a plain manner without measuring but calculating the resulting values and once through an instumented code with LIKWID.<br>\n"
-    
-    
-    for grp in testable_groups.keys():
-        print "### Verification of Group %s" % (grp,)
-        for test in testable_groups[grp]:
-            #print grp, test, test_set[grp][test]
-            print "#### Verification of Group %s with Test %s\n" % (grp, test,)
-            print "| *Stream size* | *Iterations* |"
-            print "|---------------|--------------|"
-            for variant in test_set[grp][test]["variants"]:
-                print "| %s | %s |" % (variant, test_set[grp][test][variant], )
-            print 
-            print "Each data size is tested %d times, hence the first %d entries on the x-axis correspond to the %d runs for the first data size of %s and so on.<br>\n" % (test_set[grp][test]["RUNS"],test_set[grp][test]["RUNS"],test_set[grp][test]["RUNS"],test_set[grp][test]["variants"][0],)
-            print "%s/images/%s/%s_%s.png" % (picture_base,hostname, grp, test,)
-            print
-            file_plain = os.path.join(os.path.join(resultfolder,hostname),grp+"_"+test+"_plain.dat")
-            file_marker = os.path.join(os.path.join(resultfolder,hostname),grp+"_"+test+"_marker.dat")
-            print "| Variant | Plain (Min) | LIKWID (Min) | Plain (Max) | LIKWID (Max) | Plain (Avg) | LIKWID (Avg) |"
-            print "| ------- | ------- | ------- | ------- | ------- | ------- | ------- |"
-            for i, variant in enumerate(test_set[grp][test]["variants"]):
-                results_plain = get_values_from_file(file_plain, i*test_set[grp][test]["RUNS"], test_set[grp][test]["RUNS"])
-                results_correct = get_values_from_file(file_correct, i*test_set[grp][test]["RUNS"], test_set[grp][test]["RUNS"])
-                results_marker = get_values_from_file(file_marker, i*test_set[grp][test]["RUNS"], test_set[grp][test]["RUNS"])
-                if results_plain == []: results_plain.append(0)
-                if results_marker == []: results_marker.append(0)
-                if results_correct == []: results_correct.append(0)
-                print "| %s | %d | %d | %d | %d | %d | %d |" % (variant, min(results_correct), min(results_marker), max(results_correct), max(results_marker), int(statistics.mean(results_correct)), int(statistics.mean(results_marker)),)
-            print
-            print
