@@ -1,13 +1,13 @@
 #!/bin/bash
-
 if [ $# -ne 1 ]; then
     echo "You need to give application to test on commandline"
     exit 1
 fi
 
-EXECPATH=../..
+EXECPATH=/usr/local/bin
 EXEC=$1
 TMPFILE=/tmp/testout
+FREQ="2.3"
 
 f_grep() {
     ARG="$1"
@@ -41,15 +41,25 @@ if [ ! -e ${EXEC}.txt ]; then
     echo "Cannot find testfile ${EXEC}.txt"
     exit 1
 fi
+if [ "${EXEC}" == "likwid-setFrequencies" ]; then
+    FREQ=$(likwid-setFrequencies -l | grep -v frequencies | awk '{print $2}')
+    CURFREQ=$(likwid-setFrequencies -p | head -n2 | tail -n 1 | rev | awk '{print $2}' | rev)
+fi
+if [ "${EXEC}" == "likwid-mpirun" ]; then
+    if [ -z "$(which mpiexec)" ] && [ -z "$(which mpiexec.hydra)" ] && [ -z "$(which mpirun)" ]; then
+        echo "Cannot find MPI implementation, neither mpiexec, mpiexec.hydra nor mpirun can be found in any directory in PATH"
+        exit 1
+    fi
+fi
 
 while read -r LINE || [[ -n $LINE ]]; do
     if [ -z "${LINE}" ]; then continue; fi
     if [[ "${LINE}" =~ \#.* ]]; then continue; fi
-
     OPTIONS=$(echo "${LINE}" | cut -d '|' -f 1)
+    OPTIONS=${OPTIONS//'FREQ'/"${FREQ}"}
     RESULTS=$(echo "${LINE}" | cut -d '|' -f 2-)
     NUM_RESULTS="${RESULTS//[^|]}"
-    EXITCODE=$(${EXECPATH}/${EXEC} ${OPTIONS} 1>${TMPFILE} 2>&1  ; echo $?)
+    EXITCODE=$(${EXECPATH}/${EXEC} ${OPTIONS} 1>${TMPFILE} 2>&1 </dev/null; echo $?)
     STATE=0
     for ((i=1;i<=${#NUM_RESULTS}+1;i++)); do
         RESULT=$(echo ${RESULTS} | cut -d '|' -f ${i})
@@ -78,3 +88,9 @@ while read -r LINE || [[ -n $LINE ]]; do
 done < ${EXEC}.txt
 
 
+if [ "${EXEC}" == "likwid-setFrequencies" ]; then
+    ${EXEC} -f "${CURFREQ}"
+fi
+
+rm -f /tmp/topo.txt /tmp/test /tmp/test.txt /tmp/out.txt /tmp/out
+
